#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <ctype.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/utsname.h>
#include <sys/time.h>
#include <netdb.h>

#include "connection.h"
#include "global.h"
#include "transfer.h"
#include "resume.h"
#include "interface.h"
#include "support.h"
#include "callbacks.h"
#include "share.h"
#include "hotlist.h"
#include "commands.h"
#include "chat.h"
#include "dirselect.h"
#include "scheme.h"
#include "mail.h"
#include "handler.h"
#include "search.h"
#include "resume.h"
#include "server.h"
#include "preferences.h"

///////////////////////////
// CONNECTION
///////////////////////////

void
on_connect_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  GList *combo4_items = NULL;
  GtkWidget* combo;
  char server_str[1024];
  server_t* server;
  GList* dlist;
  
  if (global.connect_win) return;
  if (global.status.connecting) return;

  global.connect_win = create_connection_win();

  temp = lookup_widget(global.connect_win, "button84");
  set_up_button(temp, "pix11", "pix12", "frame58");
  set_button_state(temp, global.connect_button);
  
  combo = lookup_widget(global.connect_win, "combo4");
  if (global.servers) {
    dlist = g_list_first(global.servers);
    while (dlist) {
      server = (server_t*)(dlist->data);
      sprintf(server_str, "%s %d", server->socket.description, 
	      server->socket.port);
      combo4_items = g_list_append (combo4_items, strdup(server_str));
      dlist = g_list_next(dlist);
    }
  }

  if (global.last_server) {
    combo4_items = g_list_insert (combo4_items, "Last Server", 0);
  }

  gtk_combo_set_popdown_strings (GTK_COMBO (combo), combo4_items);
  g_list_free (combo4_items);

  gtk_widget_show(global.connect_win);
}

void
on_disconnect_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if(global.status.connected) {
    connection_close(&global.napster.socket, NULL);
  }
  lib_del_flag(FLAG_SHARED, NULL);
  global.user.level = L_USER;
}

void
on_connection_ok_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  GtkStatusbar* status;
  GtkWidget* entry;
  int res;
  server_t* server;
  int row;

  button = GTK_BUTTON(lookup_widget(global.connect_win, "connection_ok"));
  gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);

  server = search_server();
  if (!server) return;

  entry = lookup_widget(global.connect_win, "connection_name");
  server->nick = strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
  if (!server->nick || strlen(server->nick) < 1) {
    if (strlen(global.user.username) < 1) {
      status = GTK_STATUSBAR(lookup_widget(global.connect_win, "connect_status"));
      gtk_statusbar_push(status, 1, " Enter a Nickname");
      client_message("Message", "No nickname specified");
      gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
      
      temp = lookup_widget(global.connect_win, "button84");
      set_button_state(temp, 1);
      return;
    } else {
      gtk_entry_set_text(GTK_ENTRY(entry), global.user.username);
    }
  }
  if (strchr(server->nick, ' ')) {
    status = GTK_STATUSBAR(lookup_widget(global.connect_win, "connect_status"));
    gtk_statusbar_push(status, 1, " No spaces in nick allowed");
    client_message("Message", "No spaces in allowed in the username");
    gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
    return;
  }
  entry = lookup_widget(global.connect_win, "connection_pass");
  server->passwd = strdup(gtk_entry_get_text(GTK_ENTRY(entry)));

  temp = lookup_widget(global.win, "clist11");
  row = gtk_clist_find_row_from_data(GTK_CLIST(temp), server);
  if (row >= 0)
    gtk_clist_set_text(GTK_CLIST(temp), row, 4, server->nick);
  
  if (!server->passwd || strlen(server->passwd) < 1) {
    if (strlen(global.user.password) < 1) {
      status = GTK_STATUSBAR(lookup_widget(global.connect_win, "connect_status"));
      gtk_statusbar_push(status, 1, " Enter a Password");
      client_message("Message", "No password specified");
      gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
      
      temp = lookup_widget(global.connect_win, "button84");
      set_button_state(temp, 1);
      return;
    } else {
      gtk_entry_set_text(GTK_ENTRY(entry), global.user.password);
    }
  }

  if (global.status.connected) {
    on_disconnect_activate(NULL, NULL);
  }
  res = connect_napster();

  if (res < 0) {
    global.status.connecting = 0;
    gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
  }
}

void
on_connection_cancel_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* win;

  win = global.connect_win;
  if (global.status.connecting) {
    connection_close(&global.napster.socket, "Canceled!");
  } else {
    gtk_widget_hide(win);
    gtk_widget_destroy(win);
    global.connect_win = NULL;
  }
}

void
on_connection_win_destroy              (GtkObject       *object,
                                        gpointer         user_data)
{
  GtkWidget* win;

  win = GTK_WIDGET(object);
  if (global.status.connecting) {
    connection_close(&global.napster.socket, "Canceled!");
  }

  gtk_widget_hide(win);
  gtk_widget_destroy(win);
  global.connect_win = NULL;
}


void
on_preferences_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  if (global.options_win) return;
  global.options_win = create_options_win();

  temp = lookup_widget(global.options_win, "text8");
  gtk_text_set_word_wrap(GTK_TEXT(temp), 1);
  gtk_widget_set_style(temp, global.style[2]);  
  temp = lookup_widget(global.options_win, "text9");
  gtk_text_set_word_wrap(GTK_TEXT(temp), 1);

  setup_preferences(P_TREE);
  setup_preferences(P_ALL);

  gtk_widget_show(global.options_win);
}


void
on_exit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (global.status.building) {
    global.status.exiting = 1;
    return;
  }
  
  if(global.status.connected) {
    on_disconnect_activate(NULL, NULL);
  }
  global_exit();
  gtk_main_quit();
}


void
on_join_chatrooms_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  if (!global.status.connected) {
    g_warning("not connected");
    return;
  }
  
  temp = lookup_widget(global.win, "notebook1");
  gtk_notebook_set_page(GTK_NOTEBOOK(temp), 1);
  temp = lookup_widget(global.win, "notebook3");
  gtk_notebook_set_page(GTK_NOTEBOOK(temp), 1);
  temp = lookup_widget(global.win, "channel_list");
  
  if (GTK_CLIST(temp)->rows == 0) {
    on_hidden1_activate(NULL, NULL);
  }
  
}


void
on_button6_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  
  temp = lookup_widget(GTK_WIDGET(button), "search_artist");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  temp = lookup_widget(GTK_WIDGET(button), "search_results");
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), (gfloat)100);
  temp = lookup_widget(GTK_WIDGET(button), "combo_bitrate");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  temp = lookup_widget(GTK_WIDGET(button), "combo_bitrate2");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  temp = lookup_widget(GTK_WIDGET(button), "combo_freq");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  temp = lookup_widget(GTK_WIDGET(button), "combo_freq2");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  temp = lookup_widget(GTK_WIDGET(button), "combo_speed");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  temp = lookup_widget(GTK_WIDGET(button), "combo_speed2");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  temp = lookup_widget(GTK_WIDGET(button), "entry70");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  temp = lookup_widget(GTK_WIDGET(button), "entry71");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  temp = lookup_widget(GTK_WIDGET(button), "entry69");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
}


void
on_button7_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  send_search_request(FALSE);
}



void
on_button29_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  if (!global.dir_select_win) {
    global.dir_select_win = create_dir_select_win();
    temp = lookup_widget(global.dir_select_win, "ctree1");
    gtk_signal_connect (GTK_OBJECT (temp), "tree_expand",
			GTK_SIGNAL_FUNC (on_ctree1_tree_expand),
			NULL);
    gtk_clist_set_compare_func(GTK_CLIST(temp), (GtkCListCompareFunc)dir_compare);
    update_dir_tree();
  }
  gtk_object_set_data(GTK_OBJECT(global.dir_select_win),
		      "mode", (void*)2);
  gtk_widget_show(global.dir_select_win);
}


void
on_button26_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  if (!global.dir_select_win) {
    global.dir_select_win = create_dir_select_win();
    temp = lookup_widget(global.dir_select_win, "ctree1");
    gtk_signal_connect (GTK_OBJECT (temp), "tree_expand",
			GTK_SIGNAL_FUNC (on_ctree1_tree_expand),
			NULL);
    gtk_clist_set_compare_func(GTK_CLIST(temp), (GtkCListCompareFunc)dir_compare);
    update_dir_tree();
  }
  gtk_object_set_data(GTK_OBJECT(global.dir_select_win),
		      "mode", (void*)2);
  gtk_widget_show(global.dir_select_win);
}

void
on_button30_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  apply_preferences();
  gtk_widget_hide(global.options_win);
  gtk_widget_destroy(global.options_win);
  global.options_win = NULL;
}


void
on_button31_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(global.options_win);
  gtk_widget_destroy(global.options_win);
  global.options_win = NULL;
}


void
on_ok_button1_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* win;

  gchar* filename;
  win = lookup_widget(GTK_WIDGET(button), "fileselection1");
  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(win));


}


void
on_cancel_button1_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* win;

  win = lookup_widget(GTK_WIDGET(button), "fileselection1");
  gtk_widget_hide(win);
  gtk_widget_destroy(win);
}

void
on_button33_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  temp = lookup_widget(GTK_WIDGET(button), "add_user_win");
  gtk_widget_hide(temp);
  gtk_widget_destroy(temp);
}

void
on_button10_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{

  GtkWidget* win;

  win = create_add_user_win();
  gtk_object_set_data(GTK_OBJECT(win), "ref", (void*)1);
  gtk_window_set_title (GTK_WINDOW (win), "Add User To Hotlist");
  gtk_widget_show(win);
}

void
on_refresh1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  lib_refresh();
}

void
on_button24_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkNotebook* notebook;
  int current;
  GtkWidget* page;
  GtkWidget* label;
  char* user;

  notebook = GTK_NOTEBOOK(lookup_widget(global.win, "notebook3"));
  current = gtk_notebook_get_current_page(notebook);

  if (current < 2) return;

  user = is_string_in_list(global.private_users, global.current_room);
  if (user) {
    global.private_users = 
      g_list_remove(global.private_users, user);
    gtk_notebook_remove_page(notebook, current);
  } else {
    if (global.status.connected) {
      page = gtk_notebook_get_nth_page(notebook, current);
      if (page == NULL){
	g_warning("ops");
	return;
      }
      label = gtk_notebook_get_tab_label(notebook, page);
      send_command(CMD_CLIENT_PART, GTK_LABEL(label)->label);
    } else {
      gtk_notebook_remove_page(notebook, current);
    }
  }

}

void
on_chat_entry_activate                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
#define MAX_HISTORY   1000
  GtkEntry* entry;
  GtkWidget* temp;
  char t[2048];
  char t2[2048];
  char* text;
  char* text2;
  char* pos;
  char* pos2;
  command_t* command;
  int i1, i2;
  int type;
  struct utsname uts;
  time_t tim;
  int cnt;

  entry = GTK_ENTRY(editable);
  text = strdup(gtk_entry_get_text(entry));
  if (strlen(text) <= 0) return;

  global.chat_history = g_list_append(global.chat_history, strdup(text));

  if (global.parse_color) text2 = cparse(text);
  else text2 = text;

  while (g_list_length(global.chat_history) > MAX_HISTORY) {
    pos = g_list_nth_data(global.chat_history, 0);
    free(pos);
    global.chat_history = g_list_remove(global.chat_history, pos);
  }
  global.history_pos = g_list_length(global.chat_history);
  
  if (text2[0] == '/') {
    if (global.commands == NULL) {
      g_warning("no commands declared!");
      return;
    }
    pos = strchr(text2, ' ');
    if (pos) pos[0] = 0;
    command = command_is_command(text2+1);
    if (!command) {
      client_message("Error", "Unknown command! Type /help for a list of commands.");
      gtk_entry_set_text(entry, "");
      return;
    }
    if ((command->id >= 100) && (!opennap_version(0,0))) {
      opennap_notify();
      free(text);
      return;
    }
    switch (command->id) {
    case C_NONE:
      g_warning("command with id C_NONE");
      break;
    case C_WHISPER:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /whisper");
	break;
      }

      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	client_message("Error", "Wrong format in command /whisper");
	break;
      }
      pos2[0] = 0;
      send_private(pos+1, pos2+1);
      break;
    case C_WHOIS:
      if ((!pos) || (pos[1] == 0)) 
	send_command(CMD_CLIENT_WHOIS, global.napster.nick);
      else 
	send_command(CMD_CLIENT_WHOIS, pos+1);
      break;
      // correct
    case C_EMOTE:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /emote");
	break;
      }
      send_emote(pos+1);
      break;
    case C_JOIN:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /join");
	break;
      }
      join_channel(pos+1);
      break;
    case C_SEND:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /send");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if (pos2) {
	pos2[0] = 0;
	type = atoi(pos+1);
	send_command(type, pos2+1);
      } else {
	type = atoi(pos+1);
	send_command(type, "");
      }
      break;
    case C_BROWSE:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /browse");
	break;
      }
      add_to_hotlist(pos+1);
      break;
    case C_BAN:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /ban");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	send_command(CMD_CLIENT_BAN, pos+1);
      } else {
	pos2[0] = 0;
	strcpy(t2, pos+1);
	strcat(t2, " \"");
	strcat(t2, pos2+1);
	strcat(t2, "\"");
	send_command(CMD_CLIENT_BAN, t2);
      }
      break;
    case C_UNBAN:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /ban");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	send_command(CMD_CLIENT_UNBAN, pos+1);
      } else {
	pos2[0] = 0;
	strcpy(t2, pos+1);
	strcat(t2, " \"");
	strcat(t2, pos2+1);
	strcat(t2, "\"");
	send_command(CMD_CLIENT_UNBAN, t2);
      }
      break;
    case C_CHCLEAR:
      if ((!pos) || (pos[1] == 0)) {
	if (!in_channel()) {
	  client_message("Message", "/chclear is not supported without argument in this page");
	} else {
	  send_command(CMD_CLIENT_CLEAR_CHANNEL,
		       global.current_room);
	}
      } else {
	send_command(CMD_CLIENT_CLEAR_CHANNEL, pos+1);
      }
      break;
    case C_DISC:
      on_disconnect_activate(NULL, NULL);
      break;
    case C_CLOAK:
      send_command(CMD_CLIENT_CLOAK, "");
      if (global.status.connected) {
	global.status.whois_hide++;
	send_command(CMD_CLIENT_WHOIS, global.napster.nick);
      }
      break;
    case C_EMAIL:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /email");
	break;
      }
      send_command(CMD_CLIENT_CHANGE_EMAIL, pos+1);
      break;
    case C_SPEED:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /speed");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	send_command(CMD_CLIENT_CHANGE_SPEED, pos+1);
      } else {
	send_command(CMD_CLIENT_ALTER_SPEED, pos+1);
      }
      break;
    case C_PASSWORD:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /password");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	send_command(CMD_CLIENT_CHANGE_PASS, pos+1);
      } else {
	send_command(CMD_CLIENT_ALTER_PASS, pos+1);
      }
      break;
    case C_DATAPORT:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /dataport");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	global.network.port = atoi(pos+1);
	create_upload_port(TRUE);
      } else {
	send_command(CMD_CLIENT_ALTER_PORT, pos+1);
      }
      break;
    case C_KILL:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /kill");
	break;
      }
      send_command(CMD_CLIENT_KILL, pos+1);
      break;
    case C_LEVEL:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /level");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	client_message("Error", "Wrong format in command /level");
	break;
      }
      send_command(CMD_CLIENT_SETUSERLEVEL, pos+1);
      break;
    case C_LINKS:
      send_command(CMD_SERVER_LINKS, "");
      break;
    case C_LIST:
      on_join_chatrooms_activate(NULL, user_data);
      break;
    case C_LISTBANS:
      if ((!pos) || (pos[1] == 0)) {
	show_bans(NULL);
      } else {
	show_bans(pos+1);
      }
      break;
    case C_PART:
      on_button24_clicked(NULL, NULL);
      break;
    case C_WALLOP:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /wallop");
	break;
      }
      send_command(CMD_CLIENT_WALLOP, pos+1);
      break;
    case C_STATS:
      send_command(CMD_SERVER_STATS, "");
      break;
    case C_SCHEME:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /scheme");
	break;
      }
      global.colors = scheme_load(global.colors, pos+1);
      break;
    case C_ALIAS:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /alias");
	break;
      }

      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	client_message("Error", "Wrong format in command /alias");
	break;
      }
      pos2[0] = 0;
      command_make_alias(pos2+1, pos+1);
      break;
    case C_AFK:
      if ((!pos) || (pos[1] == 0))
	set_afk(NULL);
      else set_afk(pos+1);
      break;
    case C_PUBLIC:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /public");
	break;
      }
      if (!in_channel()) {
	client_message("Message", "/public not supported in this page");
	break;
      }
      strcpy(t2, global.current_room);
      strcat(t2, " ");
      strcat(t2, pos+1);
      send_command(CMD_CLIENT_PUBLIC, t2);
      break;
    case C_REPLY:
      if (global.reply == NULL) {
	client_message("Message", "Noone to reply");
	break;
      }
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /reply");
	break;
      }
      send_private(global.reply, pos+1);
      break;
    case C_PRIVATE:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /private");
	break;
      }
      if (is_string_in_list(global.private_users, pos+1)) {
	client_message("Message", "User is already in private list");
	return;
      }
      
      global.private_users = 
	g_list_append(global.private_users, strdup(pos+1));
      
      create_private_page(pos+1);
      break;
    case C_MAIL:
      if ((!pos) || (pos[1] == 0)) {
	print_mail(NULL);
	break;
      }

      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	print_mail(pos+1);
      } else {
	pos2[0] = 0;
	add_mail(pos+1, pos2+1);
      }
      break;
    case C_DELMAIL:
      if ((!pos) || (pos[1] == 0)) {
	del_mail(NULL);
	break;
      }
      del_mail(pos+1);

      break;
    case C_CONN:
      on_connect_activate(NULL, NULL);
      temp = lookup_widget(global.connect_win, "combo_entry12");
      gtk_entry_set_text(GTK_ENTRY(temp), "Last Server");
      on_connection_ok_clicked(NULL, NULL);
      break;
    case C_MOTD:
      send_command(CMD_SERVER_MOTD, "");
      break;
    case C_QUIT:
      on_exit_activate(NULL, NULL);
      break;
    case C_IGNORE:
      if ((!pos) || (pos[1] == 0)) {
	on_show_ignore_list_activate(NULL, NULL);
	break;
      }
      send_command(CMD_CLIENT_IGNORE_USER, pos+1);
      break;
    case C_MUZZLE:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /muzzle");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	send_command(CMD_CLIENT_MUZZLE, pos+1);
      } else {
	pos2[0] = 0;
	strcpy(t2, pos+1);
	strcat(t2, " \"");
	strcat(t2, pos2+1);
	strcat(t2, "\"");
	send_command(CMD_CLIENT_MUZZLE, t2);
      }
      break;
    case C_UNMUZZLE:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /unmuzzle");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	send_command(CMD_CLIENT_UNMUZZLE, pos+1);
      } else {
	pos2[0] = 0;
	strcpy(t2, pos+1);
	strcat(t2, " \"");
	strcat(t2, pos2+1);
	strcat(t2, "\"");
	send_command(CMD_CLIENT_UNMUZZLE, t2);
      }
      break;
    case C_TOPIC:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /topic");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	if (!in_channel()) {
	  client_message("Error", "Switch to a channel win or specifiy a second parameter");
	} else {
	  strcpy(t2, global.current_room);
	  strcat(t2, " ");
	  strcat(t2, pos+1);
	  send_command(CMD_SERVER_TOPIC, t2);
	}
      } else {
	send_command(CMD_SERVER_TOPIC, pos+1);
      }
      break;
    case C_VERSION:
      send_command(CMD_CLIENT_SERVER_VERSION, "");
      break;
    case C_CLEAR:
      sprintf(t, "%s_text", global.current_room);
      temp = lookup_widget(global.win, t);
      gtk_text_set_point(GTK_TEXT(temp), 0);
      gtk_text_forward_delete(GTK_TEXT(temp),
			      gtk_text_get_length(GTK_TEXT(temp)));
      break;
    case C_UNIGNORE:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Please specify a username who is ignored");
	break;
      }
      send_command(CMD_CLIENT_UNIGNORE_USER, pos+1);
      break;
    case C_CHBAN:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "The first argument has to be a channel");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	client_message("Error", "The second argument has to be a user");
	break;
      }
      pos2[0] = 0;
      strcpy(t2, pos+1);
      pos = pos2;
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	strcat(t2, " ");
	strcat(t2, pos+1);
	send_command(CMD_CLIENT_CHANNEL_BAN, t2);
      } else {
	pos2[0] = 0;
	strcat(t2, " ");
	strcat(t2, pos+1);
	strcat(t2, " \"");
	strcat(t2, pos2+1);
	strcat(t2, "\"");
	send_command(CMD_CLIENT_CHANNEL_BAN, t2);
      }
      break;
    case C_CHUNBAN:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /chunban");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	send_command(CMD_CLIENT_CHANNEL_CLEAR_BANS, pos+1);
	break;
      }
      pos2[0] = 0;
      strcpy(t, pos+1);
      pos = pos2;
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	strcat(t2, " ");
	strcat(t2, pos+1);
	send_command(CMD_CLIENT_CHANNEL_UNBAN, t2);
      } else {
	pos2[0] = 0;
	strcat(t2, " ");
	strcat(t2, pos+1);
	strcat(t2, " \"");
	strcat(t2, pos2+1);
	strcat(t2, "\"");
	send_command(CMD_CLIENT_CHANNEL_UNBAN, t2);
      }
      break;
    case C_NUKE:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /nuke");
	break;
      }
      send_command(CMD_CLIENT_NUKE, pos+1);
      break;
    case C_UNNUKE:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /unnuke");
	break;
      }
      send_command(CMD_CLIENT_UNNUKE, pos+1);
      break;
    case C_GLOBAL:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /global");
	break;
      }
      send_command(CMD_CLIENT_ANNOUNCE, pos+1);
      break;
    case C_KICK:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /kick");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	client_message("Error", "Wrong format in command /kick");
	break;
      }
      pos2[0] = 0;
      strcpy(t2, pos+1);
      pos = pos2;
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	strcat(t2, " ");
	strcat(t2, pos+1);
	send_command(CMD_CLIENT_KICK, t2);
      } else {
	pos2[0] = 0;
	strcat(t2, " ");
	strcat(t2, pos+1);
	strcat(t2, " \"");
	strcat(t2, pos2+1);
	strcat(t2, "\"");
	send_command(CMD_CLIENT_KICK, t2);
      }
      break;
    case C_CVERSION:
      if (in_channel()) {
	strcpy(t2, global.current_room);
	strcat(t2, " ");
      } else {
	t2[0] = 0;
	client_message("Message", t);
      }

      strcat(t2, "Lopster ");
      if (uname(&uts) < 0) {
	strcat(t2, VERSION);
      } else {
	strcat(t2, VERSION);
	strcat(t2, " / ");
	strcat(t2, uts.nodename);
	strcat(t2, " ");
	strcat(t2, uts.sysname);
	strcat(t2, " ");
	strcat(t2, uts.release);
      }   
      if (in_channel()) {
	send_command(CMD_CLIENT_PUBLIC, t2);
      } else {
	client_message("Message", t2);
      }
      break;
    case C_PING:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /ping");
	break;
      }
      time(&tim);
      strcpy(t, ctime(&tim));
      t[strlen(t)-1] = 0;
      client_message("Message", "pinging %s [%s]", pos+1, t);
      send_command(CMD_CLIENT_PING, pos+1);
      break;
    case C_ECHO:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /echo");
	break;
      }
      chat_print_colored(global.current_room, "text", pos+1);
      chat_print("text", "\n");
      break;
    case C_CHLIMIT:
      if ((!pos) || (pos[1] == 0)) {
	if (in_channel())
	  send_command(CMD_CLIENT_CHANNEL_LIMIT, global.current_room);
	else
	  client_message("Error", "please specify a channel or switch to a channel");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	if (isdigit(pos[1])) {
	  if (in_channel()) {
	    strcpy(t2, global.current_room);
	    strcat(t2, " ");
	    strcat(t2, pos+1);
	    send_command(CMD_CLIENT_CHANNEL_LIMIT, t2);
	  } else
	    client_message("Error", "please specify a channel or switch to a channel");
	} else {
	  send_command(CMD_CLIENT_CHANNEL_LIMIT, pos+1);
	}
      } else {
	send_command(CMD_CLIENT_CHANNEL_LIMIT, pos+1);
      }
      break;
    case C_CHLEVEL:
      if ((!pos) || (pos[1] == 0)) {
	if (in_channel())
	  send_command(CMD_CLIENT_SET_CHAN_LEVEL, global.current_room);
	else
	  client_message("Error", "please specify a channel or a level switch to a channel");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	if (level2int(pos+1) >= 0) {
	  if (in_channel()) {
	    strcpy(t2, global.current_room);
	    strcat(t2, " ");
	    strcat(t2, pos+1);
	    send_command(CMD_CLIENT_SET_CHAN_LEVEL, t2);
	  } else
	    client_message("Error", "please specify a channel or switch to a channel");
	} else {
	  send_command(CMD_CLIENT_SET_CHAN_LEVEL, pos+1);
	}
      } else {
	send_command(CMD_CLIENT_SET_CHAN_LEVEL, pos+1);
      }
      break;
    case C_SSTATS:
      send_command(CMD_CLIENT_USAGE_STATS, "");
      break;
    case C_CHMODE:
      if ((!pos) || (pos[1] == 0)) {
	if (in_channel())
	  send_command(CMD_CLIENT_CHANNEL_MODE, global.current_room);
	else
	  client_message("Error", "please specify a channel or switch to a channel");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	if ((pos[1] == '+') || (pos[1] == '-')) {
	  if (in_channel()) {
	    strcpy(t2, global.current_room);
	    strcat(t2, " ");
	    strcat(t2, pos+1);
	    send_command(CMD_CLIENT_CHANNEL_MODE, t2);
	  } else
	    client_message("Error", "please specify a channel or switch to a channel");
	} else {
	  send_command(CMD_CLIENT_CHANNEL_MODE, pos+1);
	}
      } else {
	send_command(CMD_CLIENT_CHANNEL_MODE, pos+1);
      }
      break;
    case C_SPING:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /sping");
	break;
      }
      time(&tim);
      strcpy(t, ctime(&tim));
      t[strlen(t)-1] = 0;
      client_message("Ping", "pinging server %s [%s]", pos+1, t);
      send_command(CMD_CLIENT_PING_SERVER, pos+1);
      break;
    case C_USERMODE:
      if ((!pos) || (pos[1] == 0)) {
	send_command(CMD_CLIENT_USER_MODE, "");
      } else {
	send_command(CMD_CLIENT_USER_MODE, pos+1);
      }
      break;
    case C_CHWALLOP:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /wallop");
	break;
      }
      strcpy(t2, global.current_room);
      strcat(t2, " ");
      strcat(t2, pos+1);
      send_command(CMD_CLIENT_CHANNEL_WALLOP, t2);
      break;
    case C_OP:
      if ((!pos) || (pos[1] == 0)) {
	if (in_channel())
	  send_command(CMD_CLIENT_OP, global.current_room);
	else
	  client_message("Error", "please specify a channel or a user switch to a channel");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	if (pos2) pos2[0] = 0;
	if (user_search(pos+1)) {
	  strcpy(t2, global.current_room);
	  strcat(t2, " ");
	  strcat(t2, pos+1);
	  send_command(CMD_CLIENT_OP, t2);
	} else
	  send_command(CMD_CLIENT_OP, pos+1);
      } else {
	send_command(CMD_CLIENT_OP, pos+1);
      }
      break;
    case C_DEOP:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /deop");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	if (in_channel()) {
	  strcpy(t2, global.current_room);
	  strcat(t2, " ");
	  strcat(t2, pos+1);
	  send_command(CMD_CLIENT_DEOP, t2);
	} else 
	  client_message("Error", "please specify a channel or switch to a channel");
      } else {
	pos2[0] = 0;
	strcpy(t2, pos+1);
	strcat(t2, " ");
	strcat(t2, pos2+1);
	send_command(CMD_CLIENT_DEOP, t2);
      }
      break;
    case C_KILLGHOST:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "You have to specify the ghost");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	sprintf(t, "ghost %s %s", pos+1, global.user.password);
      } else {
	sprintf(t, "ghost %s", pos+1);
      }
      send_private("NickServ", t);
      break;
    case C_FRIEND:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "You have to specify a user");
	break;
      }
      toggle_friend(pos+1);
      break;
    case C_ENEMY:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "You have to specify a user");
	break;
      }
      toggle_enemy(pos+1);
      break;
    case C_HOTADD:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "You have to specify a user");
	break;
      }
      add_to_hotlist(pos+1);
      break;
    case C_HOTREM:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "You have to specify a user");
	break;
      }
      remove_from_hotlist(pos+1);
      break;
    case C_LOCATE:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "You have to specify a user");
	break;
      }
      send_command(CMD_CLIENT_WHICH_SERVER, pos+1);
      break;
    case C_CLIENTS:
      show_clients();
      break;
    case C_INVITE:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "You have to specify at least a user");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	if (in_channel()) {
	  strcpy(t2, global.current_room);
	  strcat(t2, " ");
	  strcat(t2, pos+1);
	  send_command(CMD_CLIENT_CHANNEL_INVITE, t2);
	} else
	  client_message("Message", "please specify a channel or switch to a channel");
      } else {
	send_command(CMD_CLIENT_CHANNEL_INVITE, pos+1);
      }
      break;
    case C_CHMUZZLE:
      if ((!pos) || (pos[1] == 0)) {
	if (in_channel())
	  send_command(CMD_CLIENT_CHANNEL_MUZZLE, global.current_room);
	else
	  client_message("Message", "please specify a channel or a user or switch to a channel");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	if (pos2) pos2[0] = 0;
	if (user_search(pos+1)) {
	  strcpy(t2, global.current_room);
	  strcat(t2, " ");
	  strcat(t2, pos+1);
	  send_command(CMD_CLIENT_CHANNEL_MUZZLE, t2);
	} else {
	  send_command(CMD_CLIENT_CHANNEL_MUZZLE, pos+1);
	}
      } else {
	send_command(CMD_CLIENT_CHANNEL_MUZZLE, pos+1);
      }
      break;
    case C_CHUNMUZZLE:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Message", "please specify at least a user");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	if (in_channel()) {
	  strcpy(t2, global.current_room);
	  strcat(t2, " ");
	  strcat(t2, pos+1);
	  send_command(CMD_CLIENT_CHANNEL_UNMUZZLE, t2);
	} else
	  client_message("Message", "please specify a channel or switch to a channel");
      } else {
	send_command(CMD_CLIENT_CHANNEL_UNMUZZLE, pos+1);
      }
      break;
    case C_CHVOICE:
      if ((!pos) || (pos[1] == 0)) {
	if (in_channel())
	  send_command(CMD_CLIENT_CHANNEL_VOICE, global.current_room);
	else
	  client_message("Error", "please specify a channel or a user switch to a channel");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	if (pos2) pos2[0] = 0;
	if (user_search(pos+1)) {
	  strcpy(t2, global.current_room);
	  strcat(t2, " ");
	  strcat(t2, pos+1);
	  send_command(CMD_CLIENT_CHANNEL_VOICE, t2);
	} else
	  send_command(CMD_CLIENT_CHANNEL_VOICE, pos+1);
      } else {
	send_command(CMD_CLIENT_CHANNEL_VOICE, pos+1);
      }
      break;
    case C_CHUNVOICE:
      if ((!pos) || (pos[1] == 0)) {
	client_message("Error", "Wrong format in command /deop");
	break;
      }
      pos2 = strchr(pos+1, ' ');
      if ((!pos2) || (pos2[1] == 0)) {
	if (in_channel()) {
	  strcpy(t2, global.current_room);
	  strcat(t2, " ");
	  strcat(t2, pos+1);
	  send_command(CMD_CLIENT_CHANNEL_UNVOICE, t2);
	} else 
	  client_message("Error", "please specify a channel or switch to a channel");
      } else {
	pos2[0] = 0;
	strcpy(t2, pos+1);
	strcat(t2, " ");
	strcat(t2, pos2+1);
	send_command(CMD_CLIENT_CHANNEL_UNVOICE, t2);
      }
      break;
    case C_CLIENTINFO:
      if ((!pos) || (pos[1] == 0)) {
      } else {
	if (global.clientinfo) free(global.clientinfo);
	if (!strcasecmp(pos+1, "NONE")) global.clientinfo = NULL;
	else global.clientinfo = strdup(pos+1);
      }
      if (global.clientinfo) {
	client_message("Client Info", "%s", global.clientinfo);
      } else {
	client_message("Client Info", "No alternative client info");
      }
      break;
    case C_GUSER:
      if (global.user_win)
	gtk_widget_show(global.user_win);
      if ((!pos) || (pos[1] == 0)) {
	show_global_users("", TRUE);
      } else {
	show_global_users(pos, TRUE);
      }
      break;
    case C_HELP:
      if ((!pos) || (pos[1] == 0)) {
	chat_print("message", "[Available Commands]\n");
	for (i2 = 0; i2 < NO_FOLDERS; i2++) {
	  if ((i2 == F_MOD) && 
	      ((int)global.user.level < (int)L_MOD)) continue;
	  chat_print("message", " ");
	  strcpy(t, FolderNames[i2]);
	  chat_print("error", t);
	  cnt = 0;
	  for (i1 = 0; i1 < g_list_length(global.commands); i1++) {
	    command = (command_t*)(g_list_nth(global.commands, i1)->data);
	    if ((command->level > L_USER) &&
		((int)command->level > (int)global.user.level))
	      continue;
	    if (command->folder == i2) {
	      if (cnt%8==0) chat_print("NULL", "\n      ");
	      else chat_print("NULL", ", ");
	      chat_print("message", "/");
	      chat_print("message", command->name);
	      cnt++;
	    }
	  }
	  chat_print("NULL", "\n");
	}
	chat_print("text", "Type ");
	chat_print("message", "/help <command>");
	chat_print("text", " for more information!\n");
      } else {
	pos++;
	pos2 = strchr(pos, ' ');
	if (pos2) pos2[0] = 0;
	command = command_is_command(pos);
	if (command) {
	  chat_print("text", "Syntax: ");
	  chat_print_ln("message", command->syntax);
	  chat_print_ln("text", command->description);
	} else {
	  client_message("Error", "This is not a command!");
	}
      }
      break;
    default:
      client_message("Error", "Command declared but not implemented yet");
      //      printf("tried to do command [%d][%s][%s]\n",
      //	     command->id, command->name, command->description);
      break;
    }
    gtk_entry_set_text(entry, "");
    return;
  } else {
    if (is_string_in_list(global.private_users, global.current_room)) {
      send_private(global.current_room, text2);
    } else {
      if (!strcmp(global.current_room, "Console")) return;
      while ((pos = strchr(text2, '\n')) != 0) {
	pos[0] = 0;
	strcpy(t2, global.current_room);
	strcat(t2, " ");
	strcat(t2, text2);
	send_command(CMD_CLIENT_PUBLIC, t2);
	text2 = pos+1;
      }
      if (strlen(text2) > 0) {
	strcpy(t2, global.current_room);
	strcat(t2, " ");
	strcat(t2, text2);
	send_command(CMD_CLIENT_PUBLIC, t2);
      }
    }
    gtk_entry_set_text(entry, "");
  }
  free(text);
}


void
on_notebook3_switch_page               (GtkNotebook     *notebook,
                                        GtkNotebookPage *page,
                                        gint             page_num,
                                        gpointer         user_data)
{
  GtkWidget* label;
  
  label = page->tab_label;
  strcpy(global.current_room, GTK_LABEL(label)->label);
  gtk_widget_set_style(label, global.style[0]);  

}


gboolean
on_search_list_button_press_event      (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkWidget* pop;
  int row, column;

  if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
				    (int)event->x, (int)event->y,
				    &row, &column))
    row = -1;
  global.popup_row = row;
  global.popup_list = widget;
  gtk_clist_select_row (GTK_CLIST(widget), row, 0);

  if (event->type == GDK_2BUTTON_PRESS) {
    on_download_activate(NULL, NULL);
    return FALSE;
  }
  if (event->button == global.popup_create) {
    //    if (row == -1) return FALSE;
    pop = create_search_list_popup((row!=-1));
    gtk_menu_popup(GTK_MENU(pop), NULL, NULL, NULL, NULL,
		   event->button, event->time);
  }

  return FALSE;
}

//////////////////////////////////////////
// whois stuff
void
on_who_is_user_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  file_t* file;

  list = GTK_CLIST(global.popup_list);
  
  file = (file_t*)gtk_clist_get_row_data(list, global.popup_row);
  if (file) send_command(CMD_CLIENT_WHOIS, file->user);
}

// end whois stuff
//////////////////////////////////////////


//////////////////////////////////////////
// download stuff
void
on_download_selected_activate         (GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
  GtkCList* clist;
  GList* row_list;
  file_t* file;
  int row;
  
  if (!directory_exists(global.path.download)) {
    download_dialog("Download");
    return;
  }
  if (!directory_exists(global.path.incomplete)) {
    download_dialog("Incomplete");
    return;
  }

  clist = GTK_CLIST(global.popup_list);
  row_list = clist->selection;

  while (row_list) {
    row = (int)row_list->data;
    file = (file_t*)gtk_clist_get_row_data(clist, row);
    download_file(file);
    row_list = row_list->next;
  }
  
}

void
on_download_activate            (GtkMenuItem     *menuitem,
				 gpointer         user_data)
{
  GtkCList* list;
  file_t* file;

  if (!directory_exists(global.path.download)) {
    download_dialog("Download");
    return;
  }
  if (!directory_exists(global.path.incomplete)) {
    download_dialog("Incomplete");
    return;
  }

  list = GTK_CLIST(global.popup_list);
  file = (file_t*)gtk_clist_get_row_data(list, global.popup_row);
  download_file(file);
}


// end download stuff
/////////////////////////////////

//////////////////////////////////////////
// hotlist stuff

void
on_button32_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  char user[200];
  void* pointer;

  temp = lookup_widget(GTK_WIDGET(button), "entry16");
  strcpy(user, gtk_entry_get_text(GTK_ENTRY(temp)));

  if (strlen(user) == 0) return;

  temp = lookup_widget(GTK_WIDGET(button), "add_user_win");
  pointer = gtk_object_get_data(GTK_OBJECT(temp), "ref");
  if (pointer == (void*)1) {
    add_to_hotlist(user);
  } else {
    send_command(CMD_CLIENT_WHOIS, user);
  }
  gtk_widget_hide(temp);
  gtk_widget_destroy(temp);
}

void
on_add_to_hot_list_activate       (GtkMenuItem     *menuitem,
				   gpointer         user_data)
{
  GtkWidget* win;

  win = create_add_user_win();
  gtk_object_set_data(GTK_OBJECT(win), "ref", (void*)1);
  gtk_window_set_title (GTK_WINDOW (win), "Add User To Hotlist");
  gtk_widget_show(win);
}



void
on_add_user_to_hotlist_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  file_t* file;

  list = GTK_CLIST(global.popup_list);
  file = (file_t*)gtk_clist_get_row_data(list, global.popup_row);
  if (!file) return;
  
  add_to_hotlist(file->user);
}


void
on_add_selected_users_to_hotlist_activate
                                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  GList* row_list;
  file_t* file;
  int row;
  int insert;

  list = GTK_CLIST(global.popup_list);
  row_list = list->selection;

  while (row_list) {
    row = (int)row_list->data;
    file = (file_t*)gtk_clist_get_row_data(list, row);
    insert = 1;

    if (file) {
      add_to_hotlist(file->user);
    }
    row_list = row_list->next;
  }


}

gboolean
on_online_button_press_event         (GtkWidget       *widget,
				      GdkEventButton  *event,
				      gpointer         user_data)
{
  GtkWidget* pop;
  GtkWidget* temp;
  int row, column;
  char *text;
  char t[500];
  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;
  guint8 space;

  if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
				    (int)event->x, (int)event->y,
				    &row, &column))
    row = -1;
  if (row == -1) return FALSE;
  global.popup_row = row;
  global.popup_list = widget;

  //  gtk_clist_select_row (GTK_CLIST(widget), row, 0);

  if (event->type == GDK_2BUTTON_PRESS) {
    on_whois_user_activate(NULL, NULL);
    return FALSE;
  }

  if (event->button == global.popup_create) {
    gtk_clist_get_pixtext (GTK_CLIST(widget), row, 0,
			   &text, &space,
			   &pixmap, &bitmap);
    pop = create_online_popup1();
    strcpy(t, text);

    if (is_string_in_list(global.ignored_users, t)) {
      temp = lookup_widget(pop, "private_mode");
      gtk_widget_set_sensitive(temp, FALSE);
    }
    temp = lookup_widget(pop, "ignore_user");
    if (is_string_in_list(global.ignored_users, t)) {
      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (temp),
				      TRUE);
    }
    gtk_signal_connect (GTK_OBJECT (temp), "activate",
			GTK_SIGNAL_FUNC (on_ignore_user_activate),
			NULL);
    temp = lookup_widget(pop, "friend1");
    if (is_string_in_list(global.frienduser, t)) {
      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (temp),
				      TRUE);
    }
    gtk_signal_connect (GTK_OBJECT (temp), "activate",
			GTK_SIGNAL_FUNC (on_friend1_activate),
			NULL);
    temp = lookup_widget(pop, "enemy1");
    if (is_string_in_list(global.enemyuser, t)) {
      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (temp),
				      TRUE);
    }
    gtk_signal_connect (GTK_OBJECT (temp), "activate",
			GTK_SIGNAL_FUNC (on_enemy1_activate),
			NULL);
    gtk_menu_popup(GTK_MENU(pop), NULL, NULL, NULL, NULL,
		   event->button, event->time);
  }

  return FALSE;
}

// end hotlist stuff
/////////////////////////////////

/////////////////////////////////
// list stuff
void
on_list_click_column                   (GtkCList        *clist,
                                        gint             column,
                                        gpointer         user_data)
{
  if (column == clist->sort_column) {
    if (clist->sort_type == GTK_SORT_ASCENDING)
      gtk_clist_set_sort_type(clist, GTK_SORT_DESCENDING);
    else
      gtk_clist_set_sort_type(clist, GTK_SORT_ASCENDING);
  } else {
    gtk_clist_set_sort_column(clist, column);
    gtk_clist_set_sort_type(clist, GTK_SORT_DESCENDING);
  }

  gtk_clist_freeze (clist);
  gtk_clist_sort(clist);
  gtk_clist_thaw(clist);

}
// end list stuff
/////////////////////////////////


gboolean
on_transfer_down_button_press_event    (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkWidget* pop;
  int row, column;
  transfer_t* transfer;
  GtkCList* clist;

  if (event->button == global.popup_create) {
    if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
				     (int)event->x, (int)event->y,
				     &row, &column))
      row = -1;
      
    //    if (row == -1) return FALSE;
    global.popup_row = row;
    global.popup_list = widget;
    gtk_clist_select_row (GTK_CLIST(widget), row, 0);

    // test
    clist = GTK_CLIST(global.popup_list);
    transfer = (transfer_t*)gtk_clist_get_row_data(clist, global.popup_row);
    pop = create_download_popup(transfer);
    // test end

    //    pop = create_download_popup();

    gtk_menu_popup(GTK_MENU(pop), NULL, NULL, NULL, NULL, 
		   event->button, event->time);
  }
  return FALSE;
}


void
on_cancel_transfer_activate (GtkMenuItem     *menuitem,
			     gpointer         user_data)
{
  GtkCList* clist;
  GList* row_list;
  int row;
  transfer_t* transfer;

  clist = GTK_CLIST(global.popup_list);
  while ((row_list = clist->selection) != NULL) {
    row = (int)row_list->data;
    transfer = (transfer_t*)gtk_clist_get_row_data(clist, row);
    if (transfer) {
      transfer_end(transfer, S_CANCELED);
      gtk_clist_remove(clist, row);
    }
  }

}

void
on_delete_transfer_activate (GtkMenuItem     *menuitem,
			     gpointer         user_data)
{
  GtkCList* clist;
  GList* row_list;
  int row;
  transfer_t* transfer;

  clist = GTK_CLIST(global.popup_list);
  while ((row_list = clist->selection) != NULL) {
    row = (int)row_list->data;
    transfer = (transfer_t*)gtk_clist_get_row_data(clist, row);
    if (transfer) {
      transfer_end(transfer, S_DELETE);
      gtk_clist_remove(clist, row);
    }
  }

}

void
on_retry_download_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  transfer_t* transfer;

  list = GTK_CLIST(global.popup_list);
  transfer = (transfer_t*)gtk_clist_get_row_data(list, global.popup_row);
  download_start(transfer, FALSE);
}

void
on_force_download_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  transfer_t* transfer;

  list = GTK_CLIST(global.popup_list);
  transfer = (transfer_t*)gtk_clist_get_row_data(list, global.popup_row);
  download_start(transfer, TRUE);
}

void
on_who_is_user_activate2               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  transfer_t* transfer;

  list = GTK_CLIST(global.popup_list);
  
  transfer = (transfer_t*)gtk_clist_get_row_data(list, global.popup_row);
  if (transfer) send_command(CMD_CLIENT_WHOIS, transfer->user);
}


void
on_button64_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkCList* list;
  int i1;
  transfer_t* transfer;

  list = GTK_CLIST(lookup_widget(global.win, "transfer_down"));
  i1 = 0;
  while (1) {
    if (i1 >= list->rows) break;
    transfer = (transfer_t*)gtk_clist_get_row_data(list, i1);
    if (transfer->status == S_FINISHED) {
      gtk_clist_remove(list, i1);
    } else {
      i1++;
    }
  }
  list = GTK_CLIST(lookup_widget(global.win, "transfer_up"));
  i1 = 0;
  while (1) {
    if (i1 >= list->rows) break;
    transfer = (transfer_t*)gtk_clist_get_row_data(list, i1);
    if (transfer->status == S_FINISHED) {
      gtk_clist_remove(list, i1);
    } else {
      i1++;
    }
  }
}


void
on_button80_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  GtkWidget* temp2;
  char* text;
  void* mode;

  temp2 = lookup_widget(GTK_WIDGET(button), "entry39");

  mode = gtk_object_get_data(GTK_OBJECT(global.dir_select_win), "mode");
  if (mode == (void*)1) {
    temp = lookup_widget(global.options_win, "clist13");
    text = gtk_entry_get_text(GTK_ENTRY(temp2));
    if (strlen(text) > 0) {
      strcpy(tstr[0], text);
      gtk_clist_append(GTK_CLIST(temp), list);
    }
  } else if (mode == (void*)2) {
    temp = lookup_widget(global.options_win, "entry14");
    gtk_entry_set_text(GTK_ENTRY(temp),
		       gtk_entry_get_text(GTK_ENTRY(temp2)));
  } else if (mode == (void*)3) {
    temp = lookup_widget(global.options_win, "entry75");
    gtk_entry_set_text(GTK_ENTRY(temp),
		       gtk_entry_get_text(GTK_ENTRY(temp2)));
  }

  gtk_widget_hide(global.dir_select_win);
}


void
on_button81_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(global.dir_select_win);
}


void
on_dir_select_win_destroy              (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_widget_destroy(global.dir_select_win);
  global.dir_select_win = NULL;
}


void
on_search_artist_activate              (GtkEditable     *editable,
                                        gpointer         user_data)
{
  gtk_signal_emit_stop_by_name(GTK_OBJECT(editable), "activate");
  send_search_request(FALSE);
}


void
on_search_results_activate             (GtkEditable     *editable,
                                        gpointer         user_data)
{
  send_search_request(FALSE);
}



gboolean
on_chat_entry_key_press_event          (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  GtkEntry* entry;
  char* text;
  char* text2;
  char comm[1024];
  char t[1024];
  GList* c_list;
  int i1;
  command_t* command;
  static int cnt = 0;

  entry = GTK_ENTRY(widget);
  if (event->type == GDK_KEY_PRESS) {
    if (event->keyval == 65289) {     // tab
      cnt++;
      gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");

      text = strdup(gtk_entry_get_text(entry));

      if (strlen(text) <= 0) return TRUE;
      text2 = strrchr(text, ' ');
      if ((text[0] != '/') || (text2)) {
	if (text2) text2++;
	else text2 = text;

	if (text2[0] == '@') text2++;
	c_list = user_search(text2);
	
	switch (g_list_length(c_list)) {
	case 0:
	  //	  client_message("Error", "no fitting user found");
	  cnt = 0;
	  break;
	case 1:
	  text2[0] = 0;
	  strcpy(t, text);
	  strcat(t, c_list->data);
	  gtk_entry_set_text(entry, t);
	  cnt = 0;
	  break;
	default:
	  i1 = calc_list_prefix(c_list);
	  text2[0] = 0;
	  strcpy(t, text);
	  strncat(t, c_list->data, i1);
	  gtk_entry_set_text(entry, t);

	  if (cnt > 1) {
	    strcpy(t, "******** Available Users:\n");
	    for (i1 = 0; i1 < g_list_length(c_list); i1++) {
	      if (i1 > 0) strcat(t, ", ");
	      strcat(t, g_list_nth(c_list, i1)->data);
	    }
	    chat_print_ln("message", t);
	    cnt = 0;
	  }
	  break;
	}
      } else {
	text2 = strdup(text+1);
	c_list = command_search(text2);

	switch (g_list_length(c_list)) {
	case 0:
	  //	  client_message("Error", "no fitting command found");
	  cnt = 0;
	  break;
	case 1:
	  command = (command_t*)(c_list->data);
	  sprintf(comm,"/%s ", command->name);
	  gtk_entry_set_text(entry, comm);
	  cnt = 0;
	  break;
	default:
	  i1 = calc_command_prefix(c_list);
	  command = (command_t*)(c_list->data);
	  comm[0] = '/';
	  comm[1] = 0;
	  strncat(comm, command->name, i1);
	  gtk_entry_set_text(entry, comm);

	  if (cnt > 1) {
	    strcpy(t, "******** Available Commands:\n");
	    for (i1 = 0; i1 < g_list_length(c_list); i1++) {
	      if (i1 > 0) strcat(t, ", /");
	      else strcat(t, "/");
	      command = (command_t*)(g_list_nth(c_list, i1)->data);
	      strcat(t, command->name);
	    }
	    chat_print_ln("message", t);
	    cnt = 0;
	  }
	  break;
	}
	free(text2);
      }
    } else if ((event->keyval == 65362) || (event->keyval == 65364)) {
      cnt = 0;
      if (event->keyval == 65362) {     // Up
	if (global.history_pos > 0) global.history_pos--;
	else global.history_pos = 0;
      }
      if (event->keyval == 65364) {     // Down
	if (global.history_pos < g_list_length(global.chat_history))
	  global.history_pos++;
	else global.history_pos = g_list_length(global.chat_history);
      }
      c_list = g_list_nth(global.chat_history, global.history_pos);
      if (c_list) {
	text = c_list->data;
	if (text) gtk_entry_set_text(GTK_ENTRY(widget), text);
      } else {
	gtk_entry_set_text(GTK_ENTRY(widget), "");
      }
      gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
    } else {
      cnt = 0;
    }
  }

  text = gtk_entry_get_text(entry);
  if (!strcasecmp(text, "/reply") && (event->keyval == 32)) {
    if (global.reply)
      sprintf(t, "/whisper %s", global.reply);
    else
      sprintf(t, "/whisper");
    gtk_entry_set_text(GTK_ENTRY(widget), t);
  }
  if (!strcasecmp(text, "/reply ")) {
    if (global.reply)
      sprintf(t, "/whisper %s ", global.reply);
    else
      sprintf(t, "/whisper ");
    gtk_entry_set_text(GTK_ENTRY(widget), t);
  }
  
  return 1;
}


void
on_whois_win_destroy                   (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_widget_destroy(global.whois_win);
  global.whois_win = NULL;
}


void
on_button82_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(global.whois_win);
}


void
on_button83_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  char* nick;
  GtkWidget* temp;

  temp = lookup_widget(global.whois_win, "entry56");
  nick = gtk_entry_get_text(GTK_ENTRY(temp));
  
  add_to_hotlist(nick);
}


gboolean
on_transfer_up_button_press_event      (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkWidget* pop;
  int row, column;
  GtkCList* clist;
  transfer_t* transfer;

  if (event->button == global.popup_create) {
    if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
				     (int)event->x, (int)event->y,
				     &row, &column))
      row = -1;
      
    //    if (row == -1) return FALSE;
    global.popup_row = row;
    global.popup_list = widget;
    gtk_clist_select_row (GTK_CLIST(widget), row, 0);

    clist = GTK_CLIST(global.popup_list);
    transfer = (transfer_t*)gtk_clist_get_row_data(clist, global.popup_row);
    pop = create_upload_popup(transfer);
    gtk_menu_popup(GTK_MENU(pop), NULL, NULL, NULL, NULL, 
		   event->button, event->time);
  }

  return FALSE;
}

void
on_entry34_activate                    (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  char user[200];
  
  temp = lookup_widget(GTK_WIDGET(editable), "entry34");
  strcpy(user, gtk_entry_get_text(GTK_ENTRY(temp)));
  send_command(CMD_CLIENT_WHOIS, user);
}


void
on_entry16_activate                    (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  char user[200];
  void* pointer;

  temp = lookup_widget(GTK_WIDGET(editable), "entry16");
  strcpy(user, gtk_entry_get_text(GTK_ENTRY(temp)));

  if (strlen(user) == 0) return;

  temp = lookup_widget(GTK_WIDGET(editable), "add_user_win");
  pointer = gtk_object_get_data(GTK_OBJECT(temp), "ref");
  if (pointer == (void*)1) {
    add_to_hotlist(user);
  } else {
    send_command(CMD_CLIENT_WHOIS, user);
  }
  gtk_widget_hide(temp);
  gtk_widget_destroy(temp);
}


void
on_togglebutton2_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  GtkWidget* temp2;

  temp = lookup_widget(GTK_WIDGET(togglebutton), "label125");
  temp2 = lookup_widget(GTK_WIDGET(togglebutton), "entry35");

  if (gtk_toggle_button_get_active(togglebutton)) {
    gtk_widget_set_sensitive(temp, TRUE);
    gtk_widget_set_sensitive(temp2, TRUE);
  } else {
    gtk_widget_set_sensitive(temp, FALSE);
    gtk_widget_set_sensitive(temp2, FALSE);
  }
  
}


void
on_button84_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  global.connect_button = toggle_button_state(GTK_WIDGET(button));
}


void
on_button90_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{

  gtk_widget_destroy(global.ban_win);
  global.ban_win = NULL;

}


void
on_add_to_hotlist_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  char *text;
  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;
  guint8 space;

  list = GTK_CLIST(global.popup_list);
  
  gtk_clist_get_pixtext (list, global.popup_row, 0,
			 &text, &space,
			 &pixmap, &bitmap);
  add_to_hotlist(text);
  
}


void
on_whois_user_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  char comm[500];
  char *text;
  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;
  guint8 space;

  list = GTK_CLIST(global.popup_list);

  gtk_clist_get_pixtext (list, global.popup_row, 0,
			 &text, &space,
			 &pixmap, &bitmap);

  //  gtk_clist_get_text(list, global.popup_row, 0, &text);
  strcpy(comm, text);

  //  sprintf(comm, "%s", text);
  send_command(CMD_CLIENT_WHOIS, text);

}


void
on_ignore_user_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  char *text;
  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;
  guint8 space;

  list = GTK_CLIST(global.popup_list);
  
  gtk_clist_get_pixtext (list, global.popup_row, 0,
			 &text, &space,
			 &pixmap, &bitmap);
  if (is_string_in_list(global.ignored_users, text)) {
    send_command(CMD_CLIENT_UNIGNORE_USER, text);
  } else {
    send_command(CMD_CLIENT_IGNORE_USER, text);
  }
}


void
on_show_ignore_list_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (global.ignore_win) {
    gtk_widget_show(global.ignore_win);
  } else {
    global.ignore_win = create_ignore_win();
    gtk_widget_show(global.ignore_win);
    send_command(CMD_CLIENT_IGNORE_LIST, "");
  }
}


void
on_ignore_win_destroy                  (GtkObject       *object,
                                        gpointer         user_data)
{
  
  gtk_widget_hide(global.ignore_win);
  gtk_widget_destroy(global.ignore_win);
  global.ignore_win = NULL;
}


void
on_button91_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  int i1;
  GtkCList* clist;
  char* text;

  // clear ignore list
  clist = GTK_CLIST(lookup_widget(global.ignore_win, "ignore_list"));
  for (i1 = 0; i1 < clist->rows; i1++) {
    gtk_clist_get_text(clist, i1, 0, &text);
    send_command(CMD_CLIENT_UNIGNORE_USER, text);
  }
  
}


void
on_button93_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(global.ignore_win);
}


void
on_refresh_users_files_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  GtkCList* clist;
  GtkCTree* ctree;
  GtkCTreeNode* node;
  GtkCTreeNode* node2;
  char *text;
  char *t;
  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;
  guint8 space;
  hot_t* data;
  hot_t* data2;

  clist = GTK_CLIST(global.popup_list);
  ctree = GTK_CTREE(global.popup_list);
  
  gtk_clist_get_pixtext (clist, global.popup_row, 0,
			 &text, &space,
			 &pixmap, &bitmap);
  t = strdup(text);

  node2 = gtk_ctree_node_nth(ctree, global.popup_row);

  data = gtk_ctree_node_get_row_data(ctree, node2);
  temp = lookup_widget(global.win, "hot_tree");
  data2 = (hot_t*)gtk_object_get_data(GTK_OBJECT(temp), "stats");
  data2->files -= data->files;
  data2->bytes -= data->bytes;
  data->files = 0;
  data->bytes = 0;
  hotlist_update_stats();

  node = GTK_CTREE_ROW(node2)->children;
  while(node) {
    gtk_ctree_remove_node(ctree, node);
    node = GTK_CTREE_ROW(node2)->children;
  }

  if (opennap_version(0, 34)) {
    send_command(CMD_CLIENT_BROWSE_NEW, text);
  } else {
    send_command(CMD_CLIENT_BROWSE, text);
  }
  free(t);
}


void
on_remove_user_from_hotlist_activate   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  char *text;
  char *t;

  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;
  guint8 space;

  list = GTK_CLIST(global.popup_list);
  
  gtk_clist_get_pixtext (list, global.popup_row, 0,
			 &text, &space,
			 &pixmap, &bitmap);
  t = strdup(text);
  remove_from_hotlist(t);
  free(t);
}


void
on_download2_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

  GtkCTree* ctree;
  GtkCTreeNode* node;
  file_t* file;

  if (!directory_exists(global.path.download)) {
    download_dialog("Download");
    return;
  }
  if (!directory_exists(global.path.incomplete)) {
    download_dialog("Incomplete");
    return;
  }

  ctree = GTK_CTREE(global.popup_list);
  
  node = gtk_ctree_node_nth(ctree, global.popup_row);
  file = (file_t*)gtk_ctree_node_get_row_data(ctree, node);
  
  download_file(file);
}


gboolean
on_hot_tree_button_press_event         (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkWidget* pop;
  int row, column;
  char *text;
  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;
  guint8 space;
  
  if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
				    (int)event->x, (int)event->y,
				    &row, &column))
    row = -1;
  if (row == -1) return FALSE;
  global.popup_row = row;
  global.popup_list = widget;

  gtk_clist_get_pixtext (GTK_CLIST(widget), row, 0,
			 &text, &space,
			 &pixmap, &bitmap);
  if (pixmap == global.pix.user1) row = 1;
  else if (pixmap == global.pix.user2) row = 1;
  else if (pixmap == global.pix.dummy) row = 1;
  else if (pixmap == NULL) {
    gtk_clist_select_row (GTK_CLIST(widget), row, 0);
    row = 2;
  } else return FALSE;
  
  if (event->type == GDK_2BUTTON_PRESS) {
    if (row == 2) on_download2_activate(NULL, NULL);
    else on_refresh_users_files_activate(NULL, NULL);
    return FALSE;
  }
  if (event->button == global.popup_create) {
    pop = create_hotlist_popup(row);
    gtk_menu_popup(GTK_MENU(pop), NULL, NULL, NULL, NULL,
		   event->button, event->time);
  }

  return FALSE;
}



void
on_button60_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkCList* list;
  GtkCTree* ctree;
  GList* row_list;
  file_t* file;
  GtkCTreeNode* node;

  if (!directory_exists(global.path.download)) {
    download_dialog("Download");
    return;
  }
  if (!directory_exists(global.path.incomplete)) {
    download_dialog("Incomplete");
    return;
  }

  list = GTK_CLIST(lookup_widget(global.win, "hot_tree"));
  ctree = GTK_CTREE(lookup_widget(global.win, "hot_tree"));
  row_list = list->selection;

  while (row_list) {
    node = (GtkCTreeNode*)row_list->data;
    file = (file_t*)gtk_ctree_node_get_row_data(ctree, node);
    download_file(file);
    row_list = row_list->next;
  }
}


void
on_combo_entry14_changed               (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkEntry* entry;
  GtkEntry* entry2;
  GtkText* des;
  GtkEntry* alias;
  char* text;
  command_t* command;
  char t[2048];

  entry = GTK_ENTRY(lookup_widget(global.options_win, "combo_entry14"));
  entry2 = GTK_ENTRY(lookup_widget(global.options_win, "entry40"));
  des = GTK_TEXT(lookup_widget(global.options_win, "text9"));
  alias = GTK_ENTRY(lookup_widget(global.options_win, "entry41"));
  
  text = gtk_entry_get_text(entry);
  command = command_is_command(text);
  if (!command) {
    g_warning("could not get command, CRITICAL!");
    return;
  }
  
  // syntax
  gtk_entry_set_text(entry2, command->syntax);

  // description
  gtk_text_backward_delete(des, gtk_text_get_length(des));
  sprintf(t, "%s\n", command->description);
  gtk_text_insert(des, NULL, NULL, NULL, t, strlen(t));

  // alias
  text = command_get_alias_string(command->id);
  gtk_entry_set_text(alias, text);
}



void
on_button101_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkEntry* entry1;
  GtkEntry* entry2;
  char* text;
  char* pos;

  command_t* command;

  entry1 = 
    GTK_ENTRY(lookup_widget(global.options_win, "combo_entry14"));
  entry2 = 
    GTK_ENTRY(lookup_widget(global.options_win, "entry41"));

  text = gtk_entry_get_text(entry1);
  command = command_is_command(text);
  if (!command) {
    g_warning("chat command does not exists");
    return;
  }
  command_delete_alias_for(command->id);
  
  text = strdup(gtk_entry_get_text(entry2));
  
  pos = strtok(text, " \t");
  while (pos) {
    if (strlen(pos) > 0)
      command_make_alias(command->name, pos);
    pos = strtok(NULL, " \t");
  }
  free(text);
  
  text = command_get_alias_string(command->id);
  gtk_entry_set_text(entry2, text);
}



void
on_button96_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  apply_preferences();
}


void destroy_temp_colors(gpointer data) {
  GList* temp_colors;
  
  temp_colors = (GList*)data;
  g_list_free(temp_colors);
}

void draw_temps(GList* temp_colors) {
  GtkWidget* temp;
  GtkWidget* win;
  style_t* style;
  char str[2048];

  win = global.options_win;

  temp = lookup_widget(win, "text8");
  gtk_text_backward_delete(GTK_TEXT(temp), 
			   gtk_text_get_length(GTK_TEXT(temp)));
  
  style = style_get(temp_colors, "join");
  strcpy(str, "[Testuser1 joined channel]\n");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "user");
  strcpy(str, "<");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "yourself");
  strcpy(str, "Yourself");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "user");
  strcpy(str, "> ");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "text");
  strcpy(str, "Hello!\n");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "part");
  strcpy(str, "[Testuser2 left channel]\n");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "user");
  strcpy(str, "<Testuser1 emote message>\n");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "error");
  strcpy(str, "(Error) ");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "text");
  strcpy(str, "This is a server message \n");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "whisper");
  strcpy(str, "<from: Testuser2> This is a private message\n");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "user");
  strcpy(str, "<");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "friend");
  strcpy(str, "Friend_User");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "user");
  strcpy(str, "> ");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "text");

  style = style_get(temp_colors, "text");
  strcpy(str, "This is a message with ");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "highlight");
  strcpy(str, "highlighted text\n");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "message");
  strcpy(str, "(Message) ");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  style = style_get(temp_colors, "text");
  strcpy(str, "This is an internal client message.\n");
  gtk_text_insert(GTK_TEXT(temp), style->font, 
		  style->fore, style->back, 
		  str, strlen(str));
  
}

void
on_combo_entry15_changed               (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget* win;
  GtkWidget* temp;
  GtkWidget* temp2;
  GList* temp_colors = NULL;
  char* text;

  win = global.options_win;
  temp = lookup_widget(win, "combo_entry15");
  
  text = gtk_entry_get_text(GTK_ENTRY(temp));

  temp = lookup_widget(global.options_win, "button98");
  temp2 = lookup_widget(global.options_win, "button99");
  if (scheme_is_valid(text)) {
    //    temp_colors = gtk_object_get_data(GTK_OBJECT(win), "temp_colors");
    temp_colors = scheme_load(temp_colors, text);
    gtk_object_set_data_full(GTK_OBJECT(win), "temp_colors", temp_colors,
			     (GtkDestroyNotify)destroy_temp_colors);
    draw_temps(temp_colors);
    gtk_widget_set_sensitive(temp, TRUE);
    gtk_widget_set_sensitive(temp2, TRUE);
  } else {
    gtk_widget_set_sensitive(temp, FALSE);
    gtk_widget_set_sensitive(temp2, FALSE);
  }
}

void
on_resume_download_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {  
  GtkCList* list;
  transfer_t* transfer;

  list = GTK_CLIST(global.popup_list);
  
  transfer = (transfer_t*)gtk_clist_get_row_data(list, global.popup_row);
  download_start(transfer, FALSE);
}

void
on_stop_download_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {
  GtkCList* list;
  transfer_t* transfer;

  list = GTK_CLIST(global.popup_list);
  
  transfer = (transfer_t*)gtk_clist_get_row_data(list, global.popup_row);
  
  if (transfer) transfer_end(transfer, S_STOPPED);
}

void
on_options_win_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_widget_hide(global.options_win);
  gtk_widget_destroy(global.options_win);
  global.options_win = NULL;
}


void
on_fileselection1_destroy              (GtkObject       *object,
                                        gpointer         user_data)
{

}


void
on_ban_win_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_widget_hide(global.ban_win);
  gtk_widget_destroy(global.ban_win);
  global.ban_win = NULL;

}


int hex_to_int(char ch) {
  if ((ch <= '9') && (ch >= '0')) 
    return ch-'0';
  else if ((ch <= 'f') && (ch >= 'a'))
    return ch-'a'+10;
  else if ((ch <= 'F') && (ch >= 'A'))
    return ch-'A'+10;
  else return 0;
}

void convert_color(char* text, gdouble* color) {
  int temp;

  temp = hex_to_int(text[1])*16 + hex_to_int(text[2]);
  color[0] = (gdouble)temp/255.0;
  temp = hex_to_int(text[3])*16 + hex_to_int(text[4]);
  color[1] = (gdouble)temp/255.0;
  temp = hex_to_int(text[5])*16 + hex_to_int(text[6]);
  color[2] = (gdouble)temp/255.0;
}

void
on_button94_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* win;
  GtkWidget* colorsel;
  GtkWidget* temp;
  gdouble color[3];
  char* text;
  GList* temp_colors;
  style_t* style;

  win = create_colorselectiondialog1();
  gtk_object_set_data_full(GTK_OBJECT(win),
			   "foreground", (gpointer)1, NULL);

  temp = lookup_widget(win, "NULL_button2");
  temp = GTK_BIN(temp)->child;
  gtk_label_set_text(GTK_LABEL(temp), "Reset");
  temp = lookup_widget(win, "cancel_button2");
  temp = GTK_BIN(temp)->child;
  gtk_label_set_text(GTK_LABEL(temp), "Cancel");

  // setting up current color
  colorsel = GTK_COLOR_SELECTION_DIALOG(win)->colorsel;

  temp = lookup_widget(global.options_win, "combo_entry13");
  text = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));

  temp_colors = 
    gtk_object_get_data(GTK_OBJECT(global.options_win), "temp_colors");
  if (temp_colors == NULL) {
    g_warning("no temp colors found");
    return;
  }
  style = style_search(temp_colors, text);
  if (!style) {
    g_warning("style not found");
    return;
  }
  convert_color(style->n_fore, color);
  
  gtk_color_selection_set_color(GTK_COLOR_SELECTION(colorsel), color);

  gtk_widget_show(win);
}


void
on_button106_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* win;
  GtkWidget* colorsel;
  GtkWidget* temp;
  gdouble color[3];
  char* text;
  GList* temp_colors;
  style_t* style;

  win = create_colorselectiondialog1();
  gtk_object_set_data_full(GTK_OBJECT(win),
			   "foreground", (gpointer)2, NULL);

  temp = lookup_widget(win, "NULL_button2");
  temp = GTK_BIN(temp)->child;
  gtk_label_set_text(GTK_LABEL(temp), "Reset");
  temp = lookup_widget(win, "cancel_button2");
  temp = GTK_BIN(temp)->child;
  gtk_label_set_text(GTK_LABEL(temp), "Cancel");

  // setting up current color
  colorsel = GTK_COLOR_SELECTION_DIALOG(win)->colorsel;

  temp = lookup_widget(global.options_win, "combo_entry13");
  text = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));

  temp_colors = 
    gtk_object_get_data(GTK_OBJECT(global.options_win), "temp_colors");
  if (temp_colors == NULL) {
    g_warning("no temp colors found");
    return;
  }
  style = style_search(temp_colors, text);
  if (!style) {
    g_warning("style not found");
    return;
  }
  convert_color(style->n_back, color);
  
  gtk_color_selection_set_color(GTK_COLOR_SELECTION(colorsel), color);

  gtk_widget_show(win);
}


void
on_button107_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* win;
  GtkWidget* fontsel;
  GtkWidget* temp;
  char* text;
  GList* temp_colors;
  style_t* style;
  
  win = create_fontselectiondialog1();

  temp = lookup_widget(win, "NULL_button1");
  temp = GTK_BIN(temp)->child;
  gtk_label_set_text(GTK_LABEL(temp), "Reset");
  
  temp = lookup_widget(win, "cancel_button3");
  temp = GTK_BIN(temp)->child;
  gtk_label_set_text(GTK_LABEL(temp), "Cancel");
  
  // setting up font
  fontsel = GTK_FONT_SELECTION_DIALOG(win)->fontsel;
  temp = lookup_widget(global.options_win, "combo_entry13");
  text = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));

  temp_colors = 
    gtk_object_get_data(GTK_OBJECT(global.options_win), "temp_colors");
  if (temp_colors == NULL) {
    g_warning("no temp colors found");
    return;
  }
  style = style_search(temp_colors, text);
  if (!style) {
    g_warning("style not found");
    return;
  }
  
  gtk_font_selection_set_font_name(GTK_FONT_SELECTION(fontsel), style->n_font);

  gtk_widget_show(win);
}


void
on_colorselectiondialog1_destroy       (GtkObject       *object,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  temp = GTK_WIDGET(object);
  gtk_widget_destroy(temp);
}


void
on_fontselectiondialog1_destroy        (GtkObject       *object,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  temp = GTK_WIDGET(object);
  gtk_widget_destroy(temp);

}


void
on_cancel_button3_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  temp = lookup_widget(GTK_WIDGET(button), "fontselectiondialog1");
  gtk_widget_destroy(temp);

}


void set_color(gdouble* color, int fore) {
  GtkWidget* temp;
  char* text;
  GList* temp_colors;
  char color_str[20];
  char hex[17] = "0123456789abcdef";
  int int_val;

  temp = lookup_widget(global.options_win, "combo_entry13");
  text = gtk_entry_get_text(GTK_ENTRY(temp));

  temp_colors = 
    gtk_object_get_data(GTK_OBJECT(global.options_win), "temp_colors");
  if (temp_colors == NULL) {
    g_warning("no temp colors found");
    return;
  }

  if (color == NULL) {
    if (fore)
      temp_colors = 
	style_new(temp_colors, text, "NULL", NULL, NULL, 1);
    else
      temp_colors = 
	style_new(temp_colors, text, NULL, "NULL", NULL, 1);
    
    draw_temps(temp_colors);
    return;
  }

  color_str[0] = '#';
  int_val = (int)(color[0]*255);
  color_str[1] = hex[int_val/16];
  color_str[2] = hex[int_val%16];
  int_val = (int)(color[1]*255);
  color_str[3] = hex[int_val/16];
  color_str[4] = hex[int_val%16];
  int_val = (int)(color[2]*255);
  color_str[5] = hex[int_val/16];
  color_str[6] = hex[int_val%16];
  color_str[7] = 0;
  if (fore)
    temp_colors = 
      style_new(temp_colors, text, color_str, NULL, NULL, 1);
  else
    temp_colors = 
      style_new(temp_colors, text, NULL, color_str, NULL, 1);

  draw_temps(temp_colors);
}

void
on_ok_button2_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* colorsel;
  GtkWidget* temp;
  static gdouble color[3];
  gpointer* pointer;

  // setting fore or background
  temp = lookup_widget(GTK_WIDGET(button), "colorselectiondialog1");
  colorsel = GTK_COLOR_SELECTION_DIALOG(temp)->colorsel;
  pointer = gtk_object_get_data(GTK_OBJECT(temp), "foreground");
  if (pointer == NULL) {
    g_warning("shit color");
    return;
  }

  gtk_color_selection_get_color(GTK_COLOR_SELECTION(colorsel), color);

  gtk_widget_destroy(temp);
  
  switch ((int)pointer) {
  case 1:
    set_color(color, 1);
    break;
  case 2:
    set_color(color, 0);
    break;
  default:
    g_warning("unknwon data");
    break;
  }
}


void
on_button100_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  char* text;
  GList* temp_colors;

  temp = lookup_widget(global.options_win, "combo_entry15");
  
  temp_colors = 
    gtk_object_get_data(GTK_OBJECT(global.options_win), "temp_colors");
  if (temp_colors == NULL) {
    g_warning("no temp colors found!");
    return;
  }

  text = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
  scheme_save(temp_colors, text);
}

void set_font(char* font) {
  GtkWidget* temp;
  char* text;
  GList* temp_colors;

  temp = lookup_widget(global.options_win, "combo_entry13");
  text = gtk_entry_get_text(GTK_ENTRY(temp));

  temp_colors = 
    gtk_object_get_data(GTK_OBJECT(global.options_win), "temp_colors");
  if (temp_colors == NULL) {
    g_warning("no temp colors found");
    return;
  }

  if (font) temp_colors = style_new(temp_colors, text, NULL, NULL, font, 1);
  else temp_colors = style_new(temp_colors, text, NULL, NULL, "NULL", 1);

  draw_temps(temp_colors);
}


void
on_ok_button3_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  GtkWidget* fontsel;
  char* font;

  temp = lookup_widget(GTK_WIDGET(button), "fontselectiondialog1");
  fontsel = GTK_FONT_SELECTION_DIALOG(temp)->fontsel;

  if (gtk_font_selection_get_font_name(GTK_FONT_SELECTION(fontsel))) {
    font = strdup(gtk_font_selection_get_font_name(GTK_FONT_SELECTION(fontsel)));
    set_font(font);
  }
  gtk_widget_destroy(temp);
}

void
on_button111_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  char* text;

  temp = lookup_widget(GTK_WIDGET(button), "clist4");
  gtk_clist_clear(GTK_CLIST(temp));

  temp = lookup_widget(GTK_WIDGET(button), "button111");
  gtk_widget_set_sensitive(temp, FALSE);

  temp = lookup_widget(GTK_WIDGET(button), "entry43");
  text = gtk_entry_get_text(GTK_ENTRY(temp));

  send_command(CMD_CLIENT_NAMES_LIST, text);
}


void
on_button114_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  toggle_button_state(GTK_WIDGET(button));
}


void
on_entry45_activate                    (GtkEditable     *editable,
                                        gpointer         user_data)
{
  on_button122_clicked(NULL, NULL);
}


void
on_entry46_activate                    (GtkEditable     *editable,
                                        gpointer         user_data)
{
  on_button123_clicked(NULL, NULL);
}


void
on_checkbutton7_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  temp = lookup_widget(global.win, "checkbutton7");
  global.show_joins = 
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));

  setup_preferences(P_JOIN);
}


void
on_button115_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  toggle_button_state(GTK_WIDGET(button));
}

// next tow for seach in hotlist
void
on_button117_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkCList* clist;
  GtkWidget* temp;
  char* text;

  clist = GTK_CLIST(lookup_widget(global.win, "clist5"));
  
  gtk_clist_clear(clist);
  temp = lookup_widget(global.win, "entry48");

  text = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));

  hotlist_search(text);
  
  free(text);
}


void
on_entry48_activate                    (GtkEditable     *editable,
                                        gpointer         user_data)
{
  on_button117_clicked(NULL, NULL);
}


void
on_spinbutton5_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  temp = GTK_WIDGET(editable);
  global.network.max_downloads = 
    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp));

  if (global.options_win) {
    temp = lookup_widget(global.options_win, "spinbutton1");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
			      global.network.max_downloads);
  }
}


void
on_spinbutton6_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  temp = GTK_WIDGET(editable);
  global.network.max_uploads = 
    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp));
  if (global.options_win) {
    temp = lookup_widget(global.options_win, "spinbutton3");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
			      global.network.max_uploads);
  }
}

void
on_spinbutton8_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  temp = GTK_WIDGET(editable);
  global.network.max_transfers = 
    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp));
  if (global.options_win) {
    temp = lookup_widget(global.options_win, "spinbutton7");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
			      global.network.max_transfers);
  }
}

void
on_button120_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  lib_refresh();
}


void
on_button98_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  char* text;
  GList* temp_colors = NULL;

  temp = lookup_widget(global.options_win, "combo_entry15");
  
  text = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
  if (scheme_is_valid(text)) {
    temp_colors = scheme_load(temp_colors, text);
    gtk_object_set_data_full(GTK_OBJECT(global.options_win),
			     "temp_colors", temp_colors,
			     (GtkDestroyNotify)destroy_temp_colors);
    draw_temps(temp_colors);
  }
}


void
on_button99_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  char* text;

  temp = lookup_widget(global.options_win, "combo_entry15");
  text = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
  scheme_delete(text);
  free(text);
}


void
on_button122_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  char* pos;
  GtkEntry* temp;
  GtkWidget* temp2;

  temp = GTK_ENTRY(lookup_widget(global.win, "entry45"));
  pos = strdup(gtk_entry_get_text(temp));
  make_list_from_string(&global.highlight, pos, "\"");
  free(pos);
  pos = make_string_from_list(global.highlight, "\"");
  gtk_entry_set_text(temp, pos);
  temp2 = lookup_widget(global.win, "button122");
  gtk_widget_set_sensitive(temp2, FALSE);

  setup_preferences(P_HIGHLIGHT);
}

void
on_button123_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  char* pos;
  GtkEntry* temp;
  GtkWidget* temp2;

  temp = GTK_ENTRY(lookup_widget(global.win, "entry46"));
  pos = strdup(gtk_entry_get_text(temp));
  delete_user_list(&global.frienduser);
  make_list_from_string(&global.frienduser, pos, " ");
  update_users(global.frienduser);
  free(pos);
  pos = make_string_from_list(global.frienduser, " ");
  gtk_entry_set_text(temp, pos);
  temp2 = lookup_widget(global.win, "button123");
  gtk_widget_set_sensitive(temp2, FALSE);

  setup_preferences(P_FRIEND);
}


void
on_checkbutton9_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  char* text;
  char str[1024];
  char str2[1024];
  time_t tim;
  
  time(&tim);

  temp = lookup_widget(global.win, "entry50");
  if (gtk_toggle_button_get_active(togglebutton)) {
    temp = lookup_widget(global.win, "button125");
    gtk_widget_set_sensitive(temp, FALSE);
    temp = lookup_widget(global.win, "entry50");
    gtk_widget_set_sensitive(temp, TRUE);
    text = gtk_entry_get_text(GTK_ENTRY(temp));

    sprintf(str2, ctime(&tim));
    str2[strlen(str2)-1] = 0;
    sprintf(str, "%s [%s]", text, str2);
    global.afk = strdup(str);
  } else {
    temp = lookup_widget(global.win, "entry50");
    gtk_widget_set_sensitive(temp, FALSE);
    temp = lookup_widget(global.win, "button125");
    gtk_widget_set_sensitive(temp, FALSE);
    if (global.afk) {
      free(global.afk);
      global.afk = NULL;
    }
  }

}


void
on_checkbutton10_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  temp = lookup_widget(global.win, "checkbutton10");
  global.ignore_public_ignore = 
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));

  setup_preferences(P_IGNORE);
}


void
on_entry50_changed                     (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  
  temp = lookup_widget(global.win, "button125");
  gtk_widget_set_sensitive(temp, TRUE);
}


void
on_play_file2_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCTree* ctree;
  file_t* file = NULL;
  GtkCTreeNode* node;
  char* dir = NULL;

  ctree = GTK_CTREE(global.popup_list);
  node = gtk_ctree_node_nth(ctree, global.popup_row);
  if (!node) return;

  if (GTK_CTREE_ROW (node)->is_leaf) {
    file = (file_t*)gtk_ctree_node_get_row_data(ctree, node);
  } else {
    dir = (char*)gtk_ctree_node_get_row_data(ctree, node);
  }

  if (file) {
    play_file(file->longname);
  } else {
    play_file(dir);
  }
}


void
on_play_file_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  transfer_t* transfer;

  list = GTK_CLIST(global.popup_list);
  
  transfer = (transfer_t*)gtk_clist_get_row_data(list, global.popup_row);
  
  if (transfer) {
    play_file(transfer->localname);
  }
}


void
on_cancel_button2_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  temp = lookup_widget(GTK_WIDGET(button), "colorselectiondialog1");
  gtk_widget_destroy(temp);
}

void
on_NULL_button2_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  gpointer* pointer;

  // setting fore or background
  temp = lookup_widget(GTK_WIDGET(button), "colorselectiondialog1");
  pointer = gtk_object_get_data(GTK_OBJECT(temp), "foreground");
  if (pointer == NULL) {
    g_warning("shit color");
    return;
  }

  // color = NULL
  gtk_widget_destroy(temp);
  
  switch ((int)pointer) {
  case 1:
    set_color(NULL, 1);
    break;
  case 2:
    set_color(NULL, 0);
    break;
  default:
    g_warning("unknwon data");
    break;
  }

}


void
on_NULL_button1_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  temp = lookup_widget(GTK_WIDGET(button), "fontselectiondialog1");
  set_font(NULL);

  gtk_widget_destroy(temp);

}


void
on_button124_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  char* pos;
  GtkEntry* temp;
  GtkWidget* temp2;

  temp = GTK_ENTRY(lookup_widget(global.win, "entry51"));
  pos = strdup(gtk_entry_get_text(temp));
  delete_user_list(&global.enemyuser);
  make_list_from_string(&global.enemyuser, pos, " ");
  update_users(global.enemyuser);
  free(pos);
  pos = make_string_from_list(global.enemyuser, " ");
  gtk_entry_set_text(temp, pos);

  temp2 = lookup_widget(global.win, "button124");
  gtk_widget_set_sensitive(temp2, FALSE);

  setup_preferences(P_ENEMY);
}


void
on_button125_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  char* text;
  GtkWidget* temp;

  temp = lookup_widget(global.win, "button125");
  if (!GTK_WIDGET_IS_SENSITIVE(temp)) {
    return;
  }
  temp = lookup_widget(global.win, "entry50");
  text = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
  set_afk(text);
  free(text);
}


void
on_entry51_activate                    (GtkEditable     *editable,
                                        gpointer         user_data)
{
  on_button124_clicked(NULL, NULL);
}


void
on_entry50_activate                    (GtkEditable     *editable,
                                        gpointer         user_data)
{
  on_button125_clicked(NULL, NULL);
}


gboolean
on_entry49_key_press_event             (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  on_entry46_key_press_event(widget, event, user_data);
  return 1;
}



void
on_entry45_changed                     (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  
  temp = lookup_widget(global.win, "button122");
  gtk_widget_set_sensitive(temp, TRUE);
}


void
on_entry46_changed                     (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  
  temp = lookup_widget(global.win, "button123");
  gtk_widget_set_sensitive(temp, TRUE);
}


void
on_entry51_changed                     (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  
  temp = lookup_widget(global.win, "button124");
  gtk_widget_set_sensitive(temp, TRUE);
}


gboolean
on_entry46_key_press_event             (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  GtkEntry* entry;
  char* text;
  char* text2;
  char t[1024];
  GList* c_list;
  int i1;
  static int cnt = 0;

  if (is_string_in_list(global.private_users, global.current_room)) return FALSE;

  //  printf("key: [%d][%d]\n", event->keyval, event->state);
  if ((event->type == GDK_KEY_PRESS) && (event->keyval == 65289)) {     // tab
    entry = GTK_ENTRY(widget);
    gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
    text = strdup(gtk_entry_get_text(entry));

    if (strlen(text) <= 0) return TRUE;
    text2 = strrchr(text, ' ');
    if (text2) text2++;
    else text2 = text;
    c_list = user_search(text2);
	
    switch (g_list_length(c_list)) {
    case 0:
      //	  client_message("Error", "no fitting user found");
      break;
    case 1:
      text2[0] = 0;
      strcpy(t, text);
      strcat(t, c_list->data);
      gtk_entry_set_text(entry, t);
      break;
    default:
      cnt++;
      i1 = calc_list_prefix(c_list);
      text2[0] = 0;
      strcpy(t, text);
      strncat(t, c_list->data, i1);
      gtk_entry_set_text(entry, t);
      
      if (cnt > 1) {
	strcpy(t, "******** Available Users:\n");
	for (i1 = 0; i1 < g_list_length(c_list); i1++) {
	  if (i1 > 0) strcat(t, ", ");
	  strcat(t, g_list_nth(c_list, i1)->data);
	}
	chat_print_ln("message", t);
	cnt = 0;
      }
      break;
    }
  }

  return 1;
}


gboolean
on_entry51_key_press_event             (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  on_entry46_key_press_event(widget, event, user_data);
  return 1;
}


void
on_button126_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  
  temp = lookup_widget(global.win, "search_list");
  gtk_clist_clear(GTK_CLIST(temp));
  update_search_stats();
}

void
on_delete_file_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget* win;
  GtkWidget* temp;

  win = create_delete_win();
  temp = lookup_widget(win, "button170");

  gtk_signal_connect (GTK_OBJECT (temp), "clicked",
                      GTK_SIGNAL_FUNC (on_button170_clicked),
                      (gpointer)2);
  gtk_widget_show(win);
}

void
on_private_mode_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  char *text;
  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;
  guint8 space;

  list = GTK_CLIST(global.popup_list);
  gtk_clist_get_pixtext (list, global.popup_row, 0,
			 &text, &space,
			 &pixmap, &bitmap);

  if (is_string_in_list(global.private_users, text)) {
    client_message("Message", "User is already in private list");
    return;
  }

  global.private_users = 
    g_list_append(global.private_users, strdup(text));

  create_private_page(text);
}


void
on_checkbutton14_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  temp = lookup_widget(global.win, "checkbutton14");
  global.send_ignore = 
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));

  setup_preferences(P_SENDIGNORE);
}


void
on_button127_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  char* text;
  int row;

  temp = lookup_widget(global.options_win, "combo_entry16");
  text = gtk_entry_get_text(GTK_ENTRY(temp));

  if (strlen(text) <= 0) return;
  strcpy(tstr[0], text);
  gtk_entry_set_text(GTK_ENTRY(temp), "");

  temp = lookup_widget(global.options_win, "clist6");
  row = gtk_clist_append(GTK_CLIST(temp), list);
}


void
on_button128_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkCList* list;
  GList* row_list;
  int row;

  list = GTK_CLIST(lookup_widget(GTK_WIDGET(button), "clist6"));

  while ((row_list = list->selection) != NULL) {
    row = (int)row_list->data;
    gtk_clist_remove(list, row);
  }
}


void
on_button129_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  on_hidden1_activate(NULL, NULL);
}


void
on_button131_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp1;
  GtkWidget* temp2;
  char* user;
  char* text;
  int len;

  temp1 = lookup_widget(global.win, "entry55");
  temp2 = lookup_widget(global.win, "text12");

  len = gtk_text_get_length(GTK_TEXT(temp2));
  text = (char*)malloc(sizeof(char)*len+1);
  strncpy(text, GTK_TEXT(temp2)->text.ch, len);
  text[len] = 0;

  user = gtk_entry_get_text(GTK_ENTRY(temp1));

  if (add_mail(user, text)) {
    //    gtk_entry_set_text(GTK_ENTRY(temp1), "");
    gtk_text_set_point(GTK_TEXT(temp2), 0);
    gtk_text_forward_delete(GTK_TEXT(temp2),
			    gtk_text_get_length(GTK_TEXT(temp2)));
  }

  free(text);
}


void
on_button132_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkCList* list;
  GList* row_list;
  int row;

  list = GTK_CLIST(lookup_widget(GTK_WIDGET(button), "clist7"));

  while ((row_list = list->selection) != NULL) {
    row = (int)row_list->data;
    gtk_clist_remove(list, row);
  }
}


void
on_button130_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  toggle_button_state(GTK_WIDGET(button));
}

void
on_button133_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkCList* list;
  int i1;
  transfer_t* transfer;

  list = GTK_CLIST(lookup_widget(global.win, "transfer_down"));
  i1 = 0;
  while (1) {
    if (i1 >= list->rows) break;
    transfer = (transfer_t*)gtk_clist_get_row_data(list, i1);
    if (!transfer_is_active(transfer)) {
      gtk_clist_remove(list, i1);
    } else {
      i1++;
    }
  }
  list = GTK_CLIST(lookup_widget(global.win, "transfer_up"));
  i1 = 0;
  while (1) {
    if (i1 >= list->rows) break;
    transfer = (transfer_t*)gtk_clist_get_row_data(list, i1);
    if ((!transfer_is_active(transfer)) || (transfer->status == S_QUEUED)) {
      gtk_clist_remove(list, i1);
    } else {
      i1++;
    }
  }
}


void
on_hot_tree_tree_select_row            (GtkCTree        *ctree,
                                        GtkCTreeNode    *node,
                                        gint             column,
                                        gpointer         user_data)
{
  char *text;
  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;
  guint8 space;
  hot_t* data;
  GtkWidget* temp;
  char str[1024];

  gtk_ctree_node_get_pixtext (ctree, node, 0, &text, &space,
			      &pixmap, &bitmap);
    
  if ((pixmap != global.pix.user1) &&
      (pixmap != global.pix.user2) &&
      (pixmap != global.pix.dummy)) return;
  
  data = gtk_ctree_node_get_row_data(ctree, node);
  temp = lookup_widget(global.win, "label221");
  gtk_label_set_text(GTK_LABEL(temp), text);
  temp = lookup_widget(global.win, "label223");
  sprintf(str, "%d", data->files);
  gtk_label_set_text(GTK_LABEL(temp), str);
  temp = lookup_widget(global.win, "label225");
  sprintf(str, "%lu", data->bytes/1024/1024);
  gtk_label_set_text(GTK_LABEL(temp), str);
}


void
on_friend1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* clist;
  char *text;
  char t[500];
  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;
  guint8 space;

  clist = GTK_CLIST(global.popup_list);
  gtk_clist_get_pixtext (clist, global.popup_row, 0,
			 &text, &space,
			 &pixmap, &bitmap);
  strcpy(t, text);
  toggle_friend(t);
}


void
on_enemy1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  char *text;
  char t[500];
  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;
  guint8 space;

  list = GTK_CLIST(global.popup_list);
  
  gtk_clist_get_pixtext (list, global.popup_row, 0,
			 &text, &space,
			 &pixmap, &bitmap);
  strcpy(t, text);
  toggle_enemy(t);
}


void rec_save (GtkCTree *ctree, GtkCTreeNode *node,
	       char* data) {
  file_t* file;
  FILE* fd = (FILE*)data;

  if (!GTK_CTREE_ROW (node)->is_leaf) return;

  file = gtk_ctree_node_get_row_data(ctree, node);
  fprintf(fd, "\"%s\" %s %ld %d %d %d\n", 
	  file->longname, file->md5, file->size,
	  file->bitrate, file->frequency, file->length);
}

void
on_button134_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  char filename[1024];
  FILE* fd;
  GtkCTree* ctree;

  sprintf(filename, "%s/shared", global.lopster_home);

  if ((fd = fopen(filename, "w")) == NULL) {
    g_warning("could not save shared files");
    return;
  }
  ctree = GTK_CTREE(lookup_widget(global.win, "lib_tree"));
  gtk_ctree_post_recursive(ctree, NULL, (GtkCTreeFunc)rec_save, fd);
  fclose(fd);
}


gboolean
on_entry55_key_press_event             (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  on_entry46_key_press_event(widget, event, user_data);
  return FALSE;
}


void
on_remove_subtree1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCTree* ctree;
  GtkCTreeNode* node;

  if (global.popup_row == -1) return;

  ctree = GTK_CTREE(global.popup_list);
  
  node = gtk_ctree_node_nth(ctree, global.popup_row);
  lib_del_flag(FLAG_IN_LIB, node);

  lib_update();
  lib_commit();
}


void
on_delete_subtree1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget* win;
  GtkWidget* temp;

  win = create_delete_win();
  temp = lookup_widget(win, "button170");

  gtk_signal_connect (GTK_OBJECT (temp), "clicked",
                      GTK_SIGNAL_FUNC (on_button170_clicked),
                      (gpointer)2);
  gtk_widget_show(win);
}


gboolean
on_lib_tree_button_press_event         (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkWidget* pop;
  int row, column;
  char *text;
  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;
  guint8 space;

  if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
				      (int)event->x, (int)event->y,
				      &row, &column))
    row = -1;
  
  global.popup_row = row;
  global.popup_list = widget;
  
  gtk_clist_get_pixtext (GTK_CLIST(widget), row, 0,
			 &text, &space,
			 &pixmap, &bitmap);
  
  if (event->type == GDK_2BUTTON_PRESS) {
    on_play_file2_activate(NULL, NULL);
    return FALSE;
  }
  if (event->button == global.popup_create) {
    if (pixmap == NULL) {
      pop = create_lib_popup2();
      gtk_clist_select_row (GTK_CLIST(widget), row, 0);
    } else {
      pop = create_lib_popup1();
      gtk_clist_select_row (GTK_CLIST(widget), row, 0);
    }  
    gtk_menu_popup(GTK_MENU(pop), NULL, NULL, NULL, NULL,
		   event->button, event->time);
  }
  return FALSE;
}


void
on_remove_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* clist;
  file_t* file;

  if (global.popup_row == -1) return;

  clist = GTK_CLIST(global.popup_list);
  
  file = (file_t*)gtk_clist_get_row_data(clist, global.popup_row);
  file->flags &= (0xff^FLAG_IN_LIB);

  lib_update();
  lib_commit();
}


gboolean
on_clist5_button_press_event           (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkWidget* pop;
  int row, column;
  
  if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
				    (int)event->x, (int)event->y,
				    &row, &column))
    row = -1;
  if (row == -1) return FALSE;
  global.popup_row = row;
  global.popup_list = widget;
  gtk_clist_select_row (GTK_CLIST(widget), row, 0);

  if (event->type == GDK_2BUTTON_PRESS) {
    on_download_activate(NULL, NULL);
    return FALSE;
  }
  if (event->button == global.popup_create) {
    pop = create_hotlist_popup3();
    gtk_menu_popup(GTK_MENU(pop), NULL, NULL, NULL, NULL,
		   event->button, event->time);
  }

  return FALSE;
}

void
on_join_channel1_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* clist;
  char* text;

  if (global.popup_row == -1) return;

  clist = GTK_CLIST(global.popup_list);
  gtk_clist_get_text(clist, global.popup_row, 0, &text);
  join_channel(text);
}

void
on_normal1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  
  temp = lookup_widget(global.win, "channel_list");
  gtk_clist_clear(GTK_CLIST(temp));
  
  //  gtk_clist_set_column_visibility (GTK_CLIST(temp), 2, FALSE);
  //  gtk_clist_set_column_visibility (GTK_CLIST(temp), 3, FALSE);
  send_command(CMD_CLIENT_LIST_CHANNELS, "");
}

void
on_hidden1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  
  temp = lookup_widget(global.win, "channel_list");
  gtk_clist_clear(GTK_CLIST(temp));
  
  //  gtk_clist_set_column_visibility (GTK_CLIST(temp), 2, TRUE);
  //  gtk_clist_set_column_visibility (GTK_CLIST(temp), 3, TRUE);
  send_command(CMD_CLIENT_FULL_CHANNEL_LIST, "");
}


gboolean
on_channel_list_button_press_event     (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkWidget* pop;
  int row, column;

  if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
				    (int)event->x, (int)event->y,
				    &row, &column))
    row = -1;
  global.popup_row = row;
  global.popup_list = widget;
  gtk_clist_select_row (GTK_CLIST(widget), row, 0);

  if (event->type == GDK_2BUTTON_PRESS) {
    on_join_channel1_activate(NULL, NULL);
    return FALSE;
  }
  if (event->button == global.popup_create) {
    pop = create_channel_popup();
    gtk_menu_popup(GTK_MENU(pop), NULL, NULL, NULL, NULL,
		   event->button, event->time);
  }

  return FALSE;
}


void
on_checkbutton21_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  temp = lookup_widget(global.win, "checkbutton21");
  global.parse_color = 
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp));

  setup_preferences(P_PARSE);
}


gboolean
on_window_key_press_event              (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  //  printf("key: [%d][%d]\n", event->keyval, event->state);
  if (event->type == GDK_KEY_PRESS) {
    if ((event->keyval >= 49) &&      // 1-9
	(event->keyval <= 57) &&
	(event->state == 8)) {        // ALT
      temp = lookup_widget(global.win, "notebook3");
      gtk_notebook_set_page(GTK_NOTEBOOK(temp), (event->keyval-49));
    } else if ((event->keyval >= 49) &&      // 1-9
	       (event->keyval <= 57) &&
	       (event->state == 4)) {        // STRG
      temp = lookup_widget(global.win, "notebook1");
      gtk_notebook_set_page(GTK_NOTEBOOK(temp), (event->keyval-49));
    } else if ((event->keyval == 97) &&      // a
	       (event->state == 8)) {        // ALT
      temp = lookup_widget(global.win, "notebook3");
      gtk_notebook_prev_page(GTK_NOTEBOOK(temp));
    } else if ((event->keyval == 113) &&     // q
	       (event->state == 8)) {        // ALT
      temp = lookup_widget(global.win, "notebook1");
      gtk_notebook_prev_page(GTK_NOTEBOOK(temp));
    } else if ((event->keyval == 115) &&     // s
	       (event->state == 8)) {        // ALT
      temp = lookup_widget(global.win, "notebook3");
      gtk_notebook_next_page(GTK_NOTEBOOK(temp));
    } else if ((event->keyval == 119) &&     // w
	       (event->state == 8)) {        // ALT
      temp = lookup_widget(global.win, "notebook1");
      gtk_notebook_next_page(GTK_NOTEBOOK(temp));
    }
  }
  return 1;
}

void
on_about_destroy                       (GtkObject       *object,
                                        gpointer         user_data)
{
  GtkWidget* win;

  win = GTK_WIDGET(object);
  gtk_widget_hide(win);
  gtk_widget_destroy(win);
}


void
on_button145_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* win;

  win = lookup_widget(GTK_WIDGET(button), "about_win");
  gtk_widget_hide(win);
  gtk_widget_destroy(win);
}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget* win;
  GtkText* text_field;
  GdkColor color1;
  GdkColor color2;
  GdkFont* font;
  char text[1024];

  win = create_about_win();
  text_field = GTK_TEXT(lookup_widget(win, "text13"));
  gtk_text_set_word_wrap(text_field, 1);
  gtk_widget_set_style(GTK_WIDGET(text_field), global.style[2]);  

  gdk_color_parse("#FF0000", &color1);
  font = gdk_font_load("-adobe-times-bold-r-normal-*-*-260-*-*-p-*-iso8859-1");
  sprintf(text, "Lopster %s\n", VERSION);
  gtk_text_insert(text_field, font, &color1, NULL, text, strlen(text));
  font = gdk_font_load("-adobe-times-bold-r-normal-*-*-140-*-*-p-*-iso8859-1");
  font = gdk_font_load("-adobe-helvetica-medium-r-normal-*-*-140-*-*-p-*-iso8859-1");

  gdk_color_parse("#FFFFFF", &color1);
  gdk_color_parse("#0066ee", &color2);
  gtk_text_insert(text_field, font, &color1, NULL, 
		  "\nCopyright (C) 2000 ", strlen("\nCopyright (C) 2000 "));
  gtk_text_insert(text_field, font, &color2, NULL, 
		  "Markus Lausser\n", strlen("Markus Lausser\n"));
  gtk_text_insert(text_field, font, &color1, NULL, 
		  "Author ", strlen("Author "));
  gtk_text_insert(text_field, font, &color2, NULL, 
		  "Markus Lausser (Sgop)\n", strlen("Markus Lausser (Sgop)\n"));
  gtk_text_insert(text_field, font, &color1, NULL, 
		  "E-mail ", strlen("E-mail "));
  gtk_text_insert(text_field, font, &color2, NULL, 
		  "Sgop@users.sourceforge.net\n", strlen("Sgop@users.sourceforge.net\n"));
  gtk_text_insert(text_field, font, &color1, NULL, 
		  "Home ", strlen("Home "));
  gtk_text_insert(text_field, font, &color2, NULL, 
		  "lopster.sourceforge.net\n", strlen("lopster.sourceforge.net\n"));
  gtk_text_insert(text_field, font, &color1, NULL, 
		  "Project ", strlen("Project "));
  gtk_text_insert(text_field, font, &color2, NULL, 
		  "sourceforge.net/projects/lopster\n", strlen("sourceforge.net/projects/lopster\n"));
  
  gtk_widget_show(win);
}


void
on_transfer_resize_column              (GtkCList        *clist,
                                        gint             column,
                                        gint             width,
                                        gpointer         user_data)
{
  int i1;
  transfer_t* transfer;

  i1 = 0;
  while (1) {
    if (i1 >= clist->rows) break;
    transfer = (transfer_t*)gtk_clist_get_row_data(clist, i1);
    transfer_update(transfer);
    i1++;
  }

}


void
on_button146_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  send_search_request(TRUE);
}


void
on_spinbutton13_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkSpinButton* spin1;
  GtkSpinButton* spin2;
  int val;

  spin1 = GTK_SPIN_BUTTON(lookup_widget(global.win, "spinbutton13"));
  val = gtk_spin_button_get_value_as_int(spin1);
  if (val < 0) {
    spin2 = GTK_SPIN_BUTTON(lookup_widget(global.win, "spinbutton12"));
    val = gtk_spin_button_get_value_as_int(spin2);
    if (val > 0) {
      gtk_spin_button_set_value(spin1, (gfloat)59);
      gtk_spin_button_set_value(spin2, (gfloat)(val-1));
    } else {
      gtk_spin_button_set_value(spin1, (gfloat)0);
    }
  } else if (val > 59) {
    gtk_spin_button_set_value(spin1, (gfloat)0);
    spin2 = GTK_SPIN_BUTTON(lookup_widget(global.win, "spinbutton12"));
    val = gtk_spin_button_get_value_as_int(spin2);
    gtk_spin_button_set_value(spin2, (gfloat)(val+1));
  }
}


void
on_spinbutton11_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkSpinButton* spin1;
  GtkSpinButton* spin2;
  int val;

  spin1 = GTK_SPIN_BUTTON(lookup_widget(global.win, "spinbutton11"));
  val = gtk_spin_button_get_value_as_int(spin1);
  if (val < 0) {
    spin2 = GTK_SPIN_BUTTON(lookup_widget(global.win, "spinbutton10"));
    val = gtk_spin_button_get_value_as_int(spin2);
    if (val > 0) {
      gtk_spin_button_set_value(spin1, (gfloat)1023);
      gtk_spin_button_set_value(spin2, (gfloat)(val-1));
    } else {
      gtk_spin_button_set_value(spin1, (gfloat)0);
    }
  } else if (val > 1023) {
    gtk_spin_button_set_value(spin1, (gfloat)0);
    spin2 = GTK_SPIN_BUTTON(lookup_widget(global.win, "spinbutton10"));
    val = gtk_spin_button_get_value_as_int(spin2);
    gtk_spin_button_set_value(spin2, (gfloat)(val+1));
  }
}


void
on_clear_selected_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* clist;
  GList* row_list;
  int row;

  clist = GTK_CLIST(lookup_widget(global.win, "search_list"));
  while ((row_list = clist->selection) != NULL) {
    row = (int)row_list->data;
    gtk_clist_remove(clist, row);
  }
  update_search_stats();
}


void
on_clear_users_files_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* clist;
  file_t* file;
  char* user;
  int i1;

  clist = GTK_CLIST(lookup_widget(global.win, "search_list"));
  file = (file_t*)gtk_clist_get_row_data(clist, global.popup_row);
  user = strdup(file->user);

  i1 = 0;
  while (i1 < clist->rows) {
    file = (file_t*)gtk_clist_get_row_data(clist, i1);
    if (!strcmp(user, file->user)) {
      gtk_clist_remove(clist, i1);
    } else {
      i1++;
    }
  }
  update_search_stats();
}


void
on_clear_all_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  on_button126_clicked(NULL, NULL);
}



void
on_ctree1_tree_select_row              (GtkCTree        *ctree,
                                        GtkCTreeNode    *node,
                                        gint             column,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  char* longname;

  temp = lookup_widget(global.dir_select_win, "entry39");
  longname = gtk_ctree_node_get_row_data(ctree, node);
  gtk_entry_set_text(GTK_ENTRY(temp), longname);
}


void
on_usermode_win_destroy                (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_widget_hide(global.usermode_win);
  gtk_widget_destroy(global.usermode_win);
  global.usermode_win = NULL;
}


void
on_button147_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  int no;
  char data[1024];
  char wid[100];
  GtkToggleButton* but;

  send_command(CMD_CLIENT_USER_MODE, "NONE");
  
  data[0] = 0;
  for (no = 0; no < NO_MODES; no++) {
    sprintf(wid, "cb%d", no);
    but = GTK_TOGGLE_BUTTON(lookup_widget(global.usermode_win, wid));
    if (gtk_toggle_button_get_active(but)) {
      strcat(data, Mode[no]);
      strcat(data, " ");
    };
  }

  if (global.usermode) free(global.usermode);

  if (strlen(data) > 0) {
    data[strlen(data)-1] = 0;
    send_command(CMD_CLIENT_USER_MODE, data);
    global.usermode = strdup(data);
  } else {
    global.usermode = strdup("NONE");
  }
  client_message("Message", "User mode was set to [%s]", global.usermode);
  gtk_widget_hide(global.usermode_win);
}


void
on_button148_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  if (global.usermode_win)
    gtk_widget_hide(global.usermode_win);
}


void
on_client_win_destroy                  (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_widget_hide(global.client_win);
  gtk_widget_destroy(global.client_win);
  global.client_win = NULL;
}


void
on_button149_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  
  temp = lookup_widget(global.client_win, "clist8");
  gtk_clist_freeze (GTK_CLIST(temp));
  gtk_clist_clear(GTK_CLIST(temp));
  gtk_clist_thaw(GTK_CLIST(temp));
  send_command(CMD_CLIENT_VERSION_STATS, "");
}


void
on_button150_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  if (global.client_win)
    gtk_widget_hide(global.client_win);
}


void
on_customize_list_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* clist;
  GtkWidget* win;

  clist = GTK_CLIST(global.popup_list);
  win = create_customize_win(clist);
  gtk_widget_show(win);
}


void
on_customize_win_destroy               (GtkObject       *object,
                                        gpointer         user_data)
{
  GtkWidget* win;

  win = GTK_WIDGET(object);
  gtk_widget_hide(win);
  gtk_widget_destroy(win);
}


void
on_button1x_clicked                   (GtkButton       *button,
				       gpointer         user_data)
{
  int i1;
  GtkCList* clist;
  GtkWidget* win;
  GtkToggleButton* tbutton;
  char text[1024];

  win = lookup_widget(GTK_WIDGET(button), "customize_win");
  clist = GTK_CLIST(gtk_object_get_data(GTK_OBJECT(win), "ref_clist"));

  for (i1 = 0; i1 < clist->columns; i1++) {
    sprintf(text, "check%d", i1);
    tbutton = GTK_TOGGLE_BUTTON(lookup_widget(win, text));
    if (gtk_toggle_button_get_active(tbutton))
      gtk_clist_set_column_visibility(clist, i1, TRUE);
    else
      gtk_clist_set_column_visibility(clist, i1, FALSE);
  }
  gtk_widget_hide(win);
  gtk_widget_destroy(win);
}


void
on_button2x_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* win;

  win = lookup_widget(GTK_WIDGET(button), "customize_win");
  gtk_widget_hide(win);
  gtk_widget_destroy(win);
}

void
on_banned_users_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  show_bans(NULL);
}


void
on_linked_servers_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  send_command(CMD_SERVER_LINKS, "");
}


void
on_client_statistic_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  show_clients();
}


void
on_whois_user1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget* win;

  win = create_add_user_win();
  gtk_window_set_title (GTK_WINDOW (win), "Whois User");
  gtk_object_set_data(GTK_OBJECT(win), "ref", (void*)2);
  gtk_widget_show(win);
}


void
on_resume_files_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GList* dlist;
  resume_t* resume;

  if (global.resume_win) {
    gtk_widget_show(global.resume_win);
  } else {
    global.resume_win = create_resume_win();
    gtk_widget_show(global.resume_win);

    dlist = g_list_first(global.incomplete);
    while (dlist) {
      resume = (resume_t*)(dlist->data);
      resume_clist_add(resume);
      dlist = g_list_next(dlist);
    }
  }
}


void
on_resume_win_destroy                  (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_widget_destroy(global.resume_win);
  global.resume_win = NULL;
}

void
on_button153_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(global.resume_win);
}


void
on_server1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  temp = lookup_widget(global.win, "notebook1");
  gtk_notebook_set_page(GTK_NOTEBOOK(temp), 0);
  temp = lookup_widget(global.win, "notebook3");
  gtk_notebook_set_page(GTK_NOTEBOOK(temp), 1);
  temp = lookup_widget(global.win, "server_list");
  
  if (GTK_CLIST(temp)->rows == 0) {
    napigator_get_list();
  }
}

gboolean
on_server_list_button_press_event      (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkWidget* pop;
  int row, column;

  if (event->button == global.popup_create) {
    if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
				      (int)event->x, (int)event->y,
				      &row, &column))
      row = -1;
    
    global.popup_row = row;
    global.popup_list = widget;

    if (row != -1)
      gtk_clist_select_row (GTK_CLIST(widget), row, 0);
    pop = create_napigator_popup(row);
    gtk_menu_popup(GTK_MENU(pop), NULL, NULL, NULL, NULL,
		   event->button, event->time);
  }

  return FALSE;
}

void
on_refresh_activate                    (GtkMenuItem     *menuitem,
					 gpointer         user_data)
{
  GtkWidget* temp;

  temp = lookup_widget(global.win, "server_list");
  gtk_clist_clear(GTK_CLIST(temp));
  
  napigator_get_list();
}

void
on_add_servers_activate                    (GtkMenuItem     *menuitem,
					    gpointer         user_data)
{
  GtkCList* clist;
  GList* row_list;
  GtkWidget* temp1;
  int row;
  char* text;
  server_t* server;
  char t[1024];

  clist = GTK_CLIST(lookup_widget(global.win, "server_list"));
  row_list = clist->selection;

  while (row_list) {
    row = (int)row_list->data;
    server = (server_t*)malloc(sizeof(server_t));

    gtk_clist_get_text(clist, row, 0, &text);
    server->socket.ip = strdup(text);

    gtk_clist_get_text(clist, row, 1, &text);
    server->socket.ip_long = inet_addr(text);

    gtk_clist_get_text(clist, row, 2, &text);
    server->socket.port = atoi(text);

    gtk_clist_get_text(clist, row, 0, &text);
    server->socket.description = strdup(text);
    server->meta = 0;

    if (global.user.username)
      server->nick = strdup(global.user.username);
    else server->nick = strdup("");
    if (global.user.password)
      server->passwd = strdup(global.user.password);
    else server->passwd = strdup("");

    if (add_server_to_list(server) == 0) {
      // already in list, do not insert
      sprintf(t, "Server [%s] [%s] already in list\n",
	      server->socket.ip, server->socket.description);
      temp1 = lookup_widget(global.win, "text14");
      gtk_text_insert(GTK_TEXT(temp1), NULL, NULL, NULL, 
		      t, strlen(t));
      free(server);
    }
    
    row_list = row_list->next;
  }
}

void
on_button158_clicked                    (GtkButton       *button,
					 gpointer         user_data)
{
  add_server(0);
}

void
on_connect2_activate(GtkMenuItem     *menuitem,
		     gpointer         user_data)
{
  GtkWidget* temp;
  GtkCList* clist;
  server_t* server;
  char t[1024];

  on_connect_activate(NULL, NULL);
  
  clist = GTK_CLIST(global.popup_list);
  server = (server_t*)gtk_clist_get_row_data(clist, global.popup_row);
  
  temp = lookup_widget(global.connect_win, "combo_entry12");
  sprintf(t, "%s %d", server->socket.description, 
	  server->socket.port);
  gtk_entry_set_text(GTK_ENTRY(temp), t);

  on_connection_ok_clicked(NULL, NULL);
}

void
on_delete_selected_activate(GtkMenuItem     *menuitem,
			    gpointer         user_data)
{
  GtkCList* clist;
  GList* row_list;
  int row;
  server_t* server;

  clist = GTK_CLIST(lookup_widget(global.win, "clist11"));

  while ((row_list = clist->selection) != NULL) {
    row = (int)row_list->data;
    server = (server_t*)gtk_clist_get_row_data(clist, row);
    gtk_clist_remove(clist, row);
    global.servers = g_list_remove(global.servers, server);
    // delete server, missing
  }
}

gboolean
on_clist11_button_press_event          (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkWidget* pop;
  int row, column;
  
  if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
				    (int)event->x, (int)event->y,
				    &row, &column))
    row = -1;
  
  global.popup_row = row;
  global.popup_list = widget;
  if (row != -1)
    gtk_clist_select_row (GTK_CLIST(widget), row, 0);

  if (event->type == GDK_2BUTTON_PRESS) {
    if (row != -1) {
      on_connect2_activate(NULL, NULL);
      return FALSE;
    }
  }
  if (event->button == global.popup_create) {
    pop = create_server_popup(row);
    gtk_menu_popup(GTK_MENU(pop), NULL, NULL, NULL, NULL,
		   event->button, event->time);
  }

  return FALSE;
}



void
on_button159_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  add_server(1);
}


void
on_combo_entry12_changed               (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  char* text;
  server_t* server;

  temp = lookup_widget(global.connect_win, "combo_entry12");
  text = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
  
  server = search_server(text);
  free(text);

  if (!server || !server->nick || !server->passwd || 
      strlen(server->nick) < 1 || strlen(server->passwd) < 1) {
    temp = lookup_widget(global.connect_win, "connection_name");
    gtk_entry_set_text(GTK_ENTRY(temp), global.user.username);
    temp = lookup_widget(global.connect_win, "connection_pass");
    gtk_entry_set_text(GTK_ENTRY(temp), global.user.password);
  } else {
    temp = lookup_widget(global.connect_win, "connection_name");
    gtk_entry_set_text(GTK_ENTRY(temp), server->nick);
    temp = lookup_widget(global.connect_win, "connection_pass");
    gtk_entry_set_text(GTK_ENTRY(temp), server->passwd);
  }
}


void
on_button162_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  
  temp = lookup_widget(GTK_WIDGET(button), "information_win");
  gtk_widget_destroy(temp);
}


gboolean
on_resume_tree_button_press_event      (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkWidget* pop;
  int row, column;

  if (event->button == global.popup_create) {
    if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
				      (int)event->x, (int)event->y,
				      &row, &column))
      row = -1;
    
    global.popup_row = row;
    global.popup_list = widget;

    if (row != -1)
      gtk_clist_select_row (GTK_CLIST(widget), row, 0);
    pop = create_resume_popup();
    gtk_menu_popup(GTK_MENU(pop), NULL, NULL, NULL, NULL,
		   event->button, event->time);
  }

  return TRUE;
}

void
on_resume_search_activate(GtkMenuItem     *menuitem,
			  gpointer         user_data)
{
  GtkCTree* ctree;
  GtkCTreeNode* node;
  GtkCTreeNode* node2;
  resume_t* resume;
  char* pos;
  file_t* file;
  char t2[2048];
  char t[1024];

  ctree = GTK_CTREE(global.popup_list);
  
  node = gtk_ctree_node_nth(ctree, global.popup_row);
  resume = (resume_t*)gtk_ctree_node_get_row_data(ctree, node);

  while ((node2 = GTK_CTREE_ROW(node)->children) != NULL) {
    file = (file_t*)gtk_ctree_node_get_row_data(ctree, node2);
    destroy_file_row(file);
    gtk_ctree_remove_node(ctree, node2);
  }
  
  if (resume->status < R_RESUME) resume->status = R_SEARCH;
  resume_clist_update(resume);

  // now doing the search
  pos = strrchr(resume->filename, '/');
  if (!pos) pos = resume->filename;
  else pos++;

  strcpy(t2, "FILENAME CONTAINS \"");
  strcat(t2, pos);
  strcat(t2, "\"");
  if (opennap_version(0, 37)) {
    strcat(t2, " SIZE \"EQUAL TO\" \"");
    sprintf(t, "%ld\"", resume->size);
    strcat(t2, t);
  }
  send_command(CMD_CLIENT_SEARCH, t2);
}

void
on_resume_delete_activate(GtkMenuItem     *menuitem,
			  gpointer         user_data)
{
  GtkCTree* ctree;
  GtkCTreeNode* node;
  resume_t* resume;

  ctree = GTK_CTREE(global.popup_list);
  
  node = gtk_ctree_node_nth(ctree, global.popup_row);
  resume = (resume_t*)gtk_ctree_node_get_row_data(ctree, node);
  
  resume_list_remove(resume);
}

void
on_refresh_list_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  send_command(830, global.current_room);
}


gboolean
on_clist12_button_press_event          (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkWidget* pop;
  int row, column;

  if (event->button == global.popup_create) {
    if (!gtk_clist_get_selection_info(GTK_CLIST(widget),
				      (int)event->x, (int)event->y,
				      &row, &column))
      row = -1;
    if (row == -1) return FALSE;
    
    global.popup_row = row;
    global.popup_list = widget;
    gtk_clist_select_row (GTK_CLIST(widget), row, 0);
    pop = create_user_popup();
    gtk_menu_popup(GTK_MENU(pop), NULL, NULL, NULL, NULL,
		   event->button, event->time);
  }
  
  return FALSE;
}

void
on_global_users_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (global.user_win)
    gtk_widget_show(global.user_win);
  show_global_users(NULL, FALSE);
}


void
on_user_win_destroy                    (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_widget_destroy(global.user_win);
  global.user_win = NULL;
}


void
on_button164_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  char args[1024] = "";
  GtkWidget* temp;
  char* text;
  int first = 1;

  temp = lookup_widget(global.user_win, "entry74");
  text = gtk_entry_get_text(GTK_ENTRY(temp));
  if (strlen(text) > 0) {
    sprintf(args, "%s", text);
    
    temp = lookup_widget(global.user_win, "checkbutton23");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
      if (first) {strcat(args, " "); first = 0;}
      strcat(args, "e");
    }
    temp = lookup_widget(global.user_win, "checkbutton24");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
      if (first) {strcat(args, " "); first = 0;}
      strcat(args, "a");
    }
    temp = lookup_widget(global.user_win, "checkbutton25");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
      if (first) {strcat(args, " "); first = 0;}
      strcat(args, "m");
    }
    temp = lookup_widget(global.user_win, "checkbutton26");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
      if (first) {strcat(args, " "); first = 0;}
      strcat(args, "l");
    }
    temp = lookup_widget(global.user_win, "checkbutton27");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
      if (first) {strcat(args, " "); first = 0;}
      strcat(args, "z");
    }
    temp = lookup_widget(global.user_win, "checkbutton28");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
      if (first) {strcat(args, " "); first = 0;}
      strcat(args, "c");
    }
  }
  show_global_users(args, TRUE);
}


void
on_button165_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(global.user_win);
}


void
on_entry74_changed                     (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  char* text;
  int val;

  temp = lookup_widget(global.user_win, "entry74");
  text = gtk_entry_get_text(GTK_ENTRY(temp));
  if (strlen(text) > 0) val = TRUE;
  else val = FALSE;
  
  temp = lookup_widget(global.user_win, "checkbutton23");
  gtk_widget_set_sensitive(temp, val);
  temp = lookup_widget(global.user_win, "checkbutton24");
  gtk_widget_set_sensitive(temp, val);
  temp = lookup_widget(global.user_win, "checkbutton25");
  gtk_widget_set_sensitive(temp, val);
  temp = lookup_widget(global.user_win, "checkbutton26");
  gtk_widget_set_sensitive(temp, val);
  temp = lookup_widget(global.user_win, "checkbutton27");
  gtk_widget_set_sensitive(temp, val);
  temp = lookup_widget(global.user_win, "checkbutton28");
  gtk_widget_set_sensitive(temp, val);
}


void
on_private_mode2_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  char *text;

  list = GTK_CLIST(global.popup_list);
  gtk_clist_get_text(list, global.popup_row, 0, &text);

  if (is_string_in_list(global.private_users, text)) {
    client_message("Message", "User is already in private list");
    return;
  }

  global.private_users = 
    g_list_append(global.private_users, strdup(text));

  create_private_page(text);
}


void
on_add_to_hotlist2_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  char *text;

  list = GTK_CLIST(global.popup_list);
  
  gtk_clist_get_text(list, global.popup_row, 0, &text);
  add_to_hotlist(text);
}


void
on_whois_user2_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCList* list;
  char *text;

  list = GTK_CLIST(global.popup_list);

  gtk_clist_get_text(list, global.popup_row, 0, &text);
  send_command(CMD_CLIENT_WHOIS, text);
}


void
on_transfer_down_select_row            (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  transfer_t* transfer;
  GtkWidget* temp;

  transfer = (transfer_t*)gtk_clist_get_row_data(clist, row);
  if (transfer->user_info) {
    temp = lookup_widget(global.win, "label422");
    gtk_label_set_text(GTK_LABEL(temp), transfer->user);
    temp = lookup_widget(global.win, "spinbutton15");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
			      (gfloat)(transfer->user_info->max));
  }
}


void
on_spinbutton15_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  char* user;
  int val;

  temp = lookup_widget(global.win, "label422");
  user = GTK_LABEL(temp)->label;
  temp = lookup_widget(global.win, "spinbutton15");
  val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp));
  change_user_info(user, val);
}

void
on_online_save_clicked (GtkButton       *button,
			gpointer         user_data) {
  GtkCList* clist;
  GtkPaned* paned;
  int i1;
  char str[1024];

  sprintf(str, "%s_online", global.current_room);
  clist = GTK_CLIST(lookup_widget(global.win, str));
  for (i1 = 0; i1 < 3; i1++) {
    global.online_width[i1] = clist->column[i1].width;
    global.online_show[i1] = clist->column[i1].visible;
  }

  sprintf(str, "%s_channel", global.current_room);
  paned = GTK_PANED(lookup_widget(global.win, str));
  global.paned_pos = paned->child1_size;
}

void
on_ctree2_tree_select_row              (GtkCTree        *ctree,
                                        GtkCTreeNode    *node,
                                        gint             column,
                                        gpointer         user_data)
{
  int row;
  GtkNotebook* notebook;

  if (!global.options_win) return;

  notebook = GTK_NOTEBOOK(lookup_widget(global.options_win, "notebook2"));
  row = (int)gtk_ctree_node_get_row_data(ctree, node)-1;
  if (gtk_notebook_get_current_page(notebook) != row) {
    gtk_notebook_set_page(notebook, row);
  }
}


void
on_notebook2_switch_page               (GtkNotebook     *notebook,
                                        GtkNotebookPage *page,
                                        gint             page_num,
                                        gpointer         user_data)
{
  GtkCTree* ctree;
  GtkCTreeNode* node;

  return;
  if (!global.options_win) return;

  page_num++;
  ctree = GTK_CTREE(lookup_widget(global.options_win, "ctree2"));
  node = gtk_ctree_find_by_row_data(ctree, NULL, (void*)(page_num));
  gtk_ctree_select(ctree, node);
}


void
on_button167_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  if (!global.dir_select_win) {
    global.dir_select_win = create_dir_select_win();
    temp = lookup_widget(global.dir_select_win, "ctree1");
    gtk_signal_connect (GTK_OBJECT (temp), "tree_expand",
			GTK_SIGNAL_FUNC (on_ctree1_tree_expand),
			NULL);
    gtk_clist_set_compare_func(GTK_CLIST(temp), (GtkCListCompareFunc)dir_compare);
    update_dir_tree();
  }
  gtk_object_set_data(GTK_OBJECT(global.dir_select_win),
		      "mode", (void*)1);
  gtk_widget_show(global.dir_select_win);
}


void
on_button168_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkCList* clist;

  clist = GTK_CLIST(lookup_widget(global.options_win, "clist13"));
  gtk_clist_clear(clist);
}


void
on_button169_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkCList* clist;
  GList* row_list;
  int row;

  clist = GTK_CLIST(lookup_widget(global.options_win, "clist13"));
  while ((row_list = clist->selection) != NULL) {
    row = (int)row_list->data;
    gtk_clist_remove(clist, row);
  }
}


void
on_button166_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;

  if (!global.dir_select_win) {
    global.dir_select_win = create_dir_select_win();
    temp = lookup_widget(global.dir_select_win, "ctree1");
    gtk_signal_connect (GTK_OBJECT (temp), "tree_expand",
			GTK_SIGNAL_FUNC (on_ctree1_tree_expand),
			NULL);
    gtk_clist_set_compare_func(GTK_CLIST(temp), (GtkCListCompareFunc)dir_compare);
    update_dir_tree();
  }
  gtk_object_set_data(GTK_OBJECT(global.dir_select_win),
		      "mode", (void*)3);
  gtk_widget_show(global.dir_select_win);
}


void
on_button140_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  char command[1024];
  int i;
  gchar *argv[4];
  gint pid;

  sprintf(command, "netscape -remote 'openURL(http://lopster.sourceforge.net, new-window)'");
  pid = fork();
  if(pid == 0) {
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = command;
    argv[3] = 0;
    for (i = 3; i < 256; i++) close(i);
    setsid();
    execve("/bin/sh", argv, environ);
    exit(-1);
  }
}


void
on_button139_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  char command[1024];
  int i;
  gchar *argv[4];
  gint pid;

  sprintf(command, "netscape -remote 'openURL(http://sourceforge.net/project/showfiles.php?group_id=8472&release_id=14589, new-window)'");
  pid = fork();
  if(pid == 0) {
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = command;
    argv[3] = 0;
    for (i = 3; i < 256; i++) close(i);
    setsid();
    execve("/bin/sh", argv, environ);
    exit(-1);
  }
}


void
on_button143_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  char command[1024];
  int i;
  gchar *argv[4];
  gint pid;

  sprintf(command, "netscape -remote 'openURL(http://sourceforge.net/mail/?group_id=8472, new-window)'");
  pid = fork();
  if(pid == 0) {
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = command;
    argv[3] = 0;
    for (i = 3; i < 256; i++) close(i);
    setsid();
    execve("/bin/sh", argv, environ);
    exit(-1);
  }
}


void
on_button141_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  char command[1024];
  int i;
  gchar *argv[4];
  gint pid;

  sprintf(command, "netscape -remote 'openURL(http://sourceforge.net/projects/lopster, new-window)'");
  pid = fork();
  if(pid == 0) {
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = command;
    argv[3] = 0;
    for (i = 3; i < 256; i++) close(i);
    setsid();
    execve("/bin/sh", argv, environ);
    exit(-1);
  }
}


void
on_button142_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  char command[1024];
  int i;
  gchar *argv[4];
  gint pid;

  sprintf(command, "netscape -remote 'openURL(http://sourceforge.net/forum/?group_id=8472, new-window)'");
  pid = fork();
  if(pid == 0) {
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = command;
    argv[3] = 0;
    for (i = 3; i < 256; i++) close(i);
    setsid();
    execve("/bin/sh", argv, environ);
    exit(-1);
  }
}


void
on_button144_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  char command[1024];
  int i;
  gchar *argv[4];
  gint pid;

  sprintf(command, "netscape -remote 'openURL(http://sourceforge.net/bugs/?group_id=8472, new-window)'");
  pid = fork();
  if(pid == 0) {
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = command;
    argv[3] = 0;
    for (i = 3; i < 256; i++) close(i);
    setsid();
    execve("/bin/sh", argv, environ);
    exit(-1);
  }
}


void
on_delete_win_destroy                  (GtkObject       *object,
                                        gpointer         user_data)
{
}


void delete_file() {
  GtkCList* clist;
  file_t* file;

  if (global.popup_row == -1) return;

  clist = GTK_CLIST(global.popup_list);
  
  file = (file_t*)gtk_clist_get_row_data(clist, global.popup_row);
  file->flags &= (0xff^FLAG_IN_LIB);
  file->flags |= FLAG_DELETE;

  lib_update();
  lib_commit();
}


void delete_subtree() {
  GtkCTree* ctree;
  GtkCTreeNode* node;

  if (global.popup_row == -1) return;

  ctree = GTK_CTREE(global.popup_list);
  
  node = gtk_ctree_node_nth(ctree, global.popup_row);
  lib_del_flag(FLAG_IN_LIB, node);
  lib_set_flag(FLAG_DELETE, node);

  lib_update();
  lib_commit();
}

void
on_button170_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  on_button171_clicked(button, user_data);

  if (user_data == (gpointer)1) delete_file();
  else delete_subtree();
}


void
on_button171_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* temp;
  
  temp = lookup_widget(GTK_WIDGET(button), "delete_win");
  gtk_widget_hide(temp);
  gtk_widget_destroy(temp);
}

