#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/time.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#include "support.h"
#include "connection.h"
#include "global.h"
#include "transfer.h"
#include "handler.h"
#include "log.h"
#include "share.h"
#include "server.h"
#include "search.h"
#include "preferences.h"
#include "chat.h"

void connection_close(socket_t* socket, char* reason) {
  GtkStatusbar* status;
  GtkWidget* temp;
  char* t;

  if (global.status.connecting) {
    status = GTK_STATUSBAR(lookup_widget(global.connect_win, "connect_status"));
    gtk_statusbar_push(status, 1, reason);
    temp = lookup_widget(global.connect_win, "connection_ok");
    gtk_widget_set_sensitive(temp, TRUE);
  }

  if (reason)
    client_message("Message", reason);

  // delete private users
  if (global.private_users) {
    g_list_free(global.private_users);
    global.private_users = NULL;
  }

  // delete ignored users
  if (global.ignored_users) {
    g_list_free(global.ignored_users);
    global.ignored_users = NULL;

    if (global.ignore_win) {
      temp = lookup_widget(global.ignore_win, "ignore_list");
      gtk_clist_clear(GTK_CLIST(temp));
    }
  }
  
  // delete search hotlist
  temp = lookup_widget(global.win, "clist5");
  gtk_clist_clear(GTK_CLIST(temp));

  {
    GtkCTree* ctree;
    GtkCTreeNode *node;
    
    ctree = GTK_CTREE(lookup_widget(global.win, "hot_tree"));
    
    node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
    while(node) {
      t = strdup(GTK_CELL_PIXTEXT(GTK_CTREE_ROW (node)->row.cell[ctree->tree_column])->text);
      gtk_ctree_set_node_info(ctree, node, t, 5,
			      global.pix.dummy, 
			      global.pix.dummyb,
			      global.pix.dummy, 
			      global.pix.dummyb,
			      0, 0);
      node = GTK_CTREE_ROW(node)->sibling;
      free(t);
    }
  }

  temp = lookup_widget(global.win, "menu_connect");
  temp = GTK_BIN(temp)->child;
  gtk_label_set_text(GTK_LABEL(temp), "Connect");

  temp = lookup_widget(global.win, "global_users");
  gtk_widget_set_sensitive(temp, FALSE);

  // delete search
  temp = lookup_widget(global.win, "search_list");
  gtk_clist_clear(GTK_CLIST(temp));

  // clear queues
  g_list_free(global.queue);
  global.queue = NULL;
  g_list_free(global.share_queue);
  global.share_queue = NULL;

  // closing socket
  if (socket->input >= 0) {
    gdk_input_remove(socket->input);
    socket->input = -1;
  }
  if (socket->timeout >= 0) {
    gtk_timeout_remove(socket->timeout);
    socket->timeout = -1;
  }
  global.status.connecting = 0;
  global.status.connected = 0;
  global.status.searching = 0;
  global.status.whois_hide = 0;

  global.user.status = STATUS_INACTIVE;
  global.napster.network = N_UNKNOWN;
  update_user_stats();
  update_search_stats();
  
  lib_del_flag(FLAG_SHARED, NULL);

  close(socket->fd);
}

int connection_timeout(gpointer data) {
  socket_t* socket;
  fd_set fds;
  struct timeval t = { 0, 0 };
  int i1;
  napster_command_t* command;

  socket = (socket_t*)data;

  if (socket->cnt>= socket->max_cnt) {
    connection_close(socket, "Timed out!");
    return 1;
  }
  socket->cnt++;

  i1 = 0;
  while (i1 < 10) {
    if (!global.queue) break;
    FD_ZERO(&fds);
    FD_SET(socket->fd, &fds);
    if (select(socket->fd+1, NULL, &fds, NULL, &t) == 1) {
      command = (napster_command_t*)g_list_nth_data(global.queue, 0);
      send_command_real(command);
      global.queue = g_list_remove(global.queue, command);
    } else {
      //      printf("cannot write to socket\n");
      return 1;
    }
    i1++;
  }
  while (i1 < 10) {
    if (!global.share_queue) break;
    FD_ZERO(&fds);
    FD_SET(socket->fd, &fds);
    if (select(socket->fd+1, NULL, &fds, NULL, &t) == 1) {
      command = (napster_command_t*)g_list_nth_data(global.share_queue, 0);
      send_command_real(command);
      global.share_queue = g_list_remove(global.share_queue, command);
    } else {
      //      printf("cannot write to socket\n");
      return 1;
    }
    i1++;
    if (!global.share_queue) refresh_channels();
  }
  return 1;
}

int connect_napster() {
  GtkStatusbar* status;
  struct hostent* h = NULL;
  server_t* server;
  char t[1024];

  status = GTK_STATUSBAR(lookup_widget(global.connect_win, "connect_status"));

  if (global.status.connected || global.status.connecting) {
    return -1;
  }
  global.status.connecting = 1;

  server = search_server();
  if (!server) {
    gtk_statusbar_push(status, 1, " Could find server in list");
    return -1;
  }

  h = resolv_address(server->socket.ip);
  if (h) {
    sprintf(t, "%u.%u.%u.%u", 
	    h->h_addr[0] & 0xff,
	    h->h_addr[1] & 0xff,
	    h->h_addr[2] & 0xff,
	    h->h_addr[3] & 0xff);
    server->socket.ip_long = inet_addr(t);
  } else {
    gtk_statusbar_push(status, 1, " Could not resolve address");
    return -1;
  }

  if (global.napster.socket.ip) free(global.napster.socket.ip);
  global.napster.socket.ip = strdup(server->socket.ip);
  global.napster.socket.ip_long = server->socket.ip_long;
  global.napster.socket.port = server->socket.port;
  if (global.napster.socket.description) free(global.napster.socket.description);
  global.napster.socket.description = strdup(server->socket.description);
  if (global.napster.nick) free(global.napster.nick);
  global.napster.nick = strdup(server->nick);
  if (global.napster.passwd) free(global.napster.passwd);
  global.napster.passwd = strdup(server->passwd);

  if (server->meta) {
    return get_best_server(server);
  } else {
    return connect_to_server();
  }

  return 0;
}

int get_best_server(server_t* server) {
  int res;
  GtkStatusbar* status;

  status = GTK_STATUSBAR(lookup_widget(global.connect_win, "connect_status"));
  gtk_statusbar_push(status, 1, " Get best server");

  res = connect_socket(&global.napster.socket);

  if ((res != 0) && (res != EINPROGRESS)) {
    gtk_statusbar_push(status, 1, strerror(errno));
    return -1;
  }
  // receiving best server
  global.napster.socket.input =
    gdk_input_add(global.napster.socket.fd, GDK_INPUT_READ, 
		  GTK_SIGNAL_FUNC(get_best_server_read), &global.napster);
  
  global.napster.socket.max_cnt = 20;
  global.napster.socket.timeout =
    gtk_timeout_add(1000, connection_timeout, &global.napster.socket);

  return 0;
}

int get_best_server_read(gpointer data, 
			 gint source, 
			 GdkInputCondition condition) {
  int length;
  socket_t* socket;
  unsigned char buffer[100+1];
  char* pos;

  socket = (socket_t*)data;

  gdk_input_remove(socket->input);
  socket->input = -1;
  gtk_timeout_remove(socket->timeout);
  socket->timeout = -1;

  while ((length = recv(socket->fd, buffer, 100, 0)) <= 0) {
    if (length <= 0) {
      connection_close(socket, "Could not get best server");
      return 1;
    }
  }  

  buffer[length] = '\0';

  pos = strtok(buffer, ":");
  global.napster.socket.ip = strdup(pos);
  global.napster.socket.ip_long = inet_addr(pos);
  global.napster.socket.description = strdup("No description");
  pos = strtok(NULL, " ");
  global.napster.socket.port = atoi(pos);

#ifdef CONECTION_DEBUG
  printf("got server [%s][%d]\n", global.napster.socket.ip,
	 global.napster.socket.port);
#endif

  if (strcmp(global.napster.socket.ip, "127.0.0.1")) {
    connection_close(socket, "Got best Server");
    connect_to_server();
  } else {
    connection_close(socket, "All Servers busy");
    return -1;
  }

  return 1;
}


int connect_to_server() {
  GtkStatusbar* status;
  GtkWidget* temp;
  int res;

  global.status.connecting = 1;

  status = GTK_STATUSBAR(lookup_widget(global.connect_win, "connect_status"));
  gtk_statusbar_push(status, 1, " Connecting to server...");

  res = connect_socket(&(global.napster.socket));

  if ((res != 0) && (res != EINPROGRESS)) {
    temp = lookup_widget(global.connect_win, "connection_ok");
    gtk_widget_set_sensitive(temp, TRUE);
    gtk_statusbar_push(status, 1, strerror(errno));
    global.status.connecting = 0;
    return -1;
  }

  global.napster.socket.input =
    gdk_input_add(global.napster.socket.fd, GDK_INPUT_WRITE, 
		  GTK_SIGNAL_FUNC(server_login),
		  &(global.napster));  
  global.napster.socket.max_cnt = 10000;
  global.napster.socket.timeout =
    gtk_timeout_add(100, connection_timeout, &(global.napster.socket));

  return 0;
}

int server_login(gpointer data, gint source, GdkInputCondition condition) {
  GtkStatusbar* status;
  GtkWidget* toggle;
  int port;
  GtkWidget* temp;
  char* text;
  char t2[2048];
  char t[1024];

  gdk_input_remove(global.napster.socket.input);

  status = GTK_STATUSBAR(lookup_widget(global.connect_win, "connect_status"));
  gtk_statusbar_push(status, 1, " Logging in...");
  toggle = lookup_widget(global.connect_win, "new_user");

  if (global.network.firewall) port = 0;
  else port = global.network.port;

  strcpy(t2, global.napster.nick);
  strcat(t2, " ");
  strcat(t2, global.napster.passwd);
  strcat(t2, " ");
  sprintf(t, "%d \"", port);
  strcat(t2, t);
  if (global.clientinfo) {
    strcat(t2, global.clientinfo);
  } else {
    strcat(t2, "Lopster ");
    strcat(t2, VERSION);
  }
  sprintf(t, "\" %d", global.network.linespeed);
  strcat(t2, t);

  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle))) {
    temp = lookup_widget(global.connect_win, "entry35");
    text = gtk_entry_get_text(GTK_ENTRY(temp));
    if (strcmp(text, global.napster.passwd)) {
      connection_close(&global.napster.socket, " Password Retype Error");
      return -1;
    }
    if (send_command(CMD_CLIENT_REGISTER, global.napster.nick)) {
      connection_close(&global.napster.socket, " Could not register nick");
      return -1;
    }
    strcat(t2, " ");
    if (strlen(global.user.email) > 0) {
      strcat(t2, global.user.email);
    } else {
      strcat(t2, "dummy@mail");
    }
    if (send_command(CMD_CLIENT_LOGIN_REGISTER, t2)) {
      connection_close(&global.napster.socket, " Could not login");
      return -1;
    }
  } else {
    if (send_command(CMD_CLIENT_LOGIN, t2)) {
      connection_close(&global.napster.socket, "Could not login...");
      return -1;
    }
  }

  global.napster.socket.input =
    gdk_input_add(global.napster.socket.fd, GDK_INPUT_READ, 
		  GTK_SIGNAL_FUNC(server_get_input),
		  &(global.napster));  
  return 0;
}

int server_get_input(gpointer data, gint source, GdkInputCondition condition) {
  static char mess[2048];
  gint16 type;
  socket_t * socket;

  socket = (socket_t*)data;

  type = receive_command(global.napster.socket.fd, mess);

  if (type < 0) {
    return 1;
  }

  log("protocol", "R: %5d L: %4d [%s]\n", type, strlen(mess), mess);
  socket->cnt = 0;
  handle_command(type, mess);
  
  return 1;
}


int connect_socket(socket_t* s) {
  int protocol;
  struct sockaddr_in server;
  int sock;
  int res;

  protocol = getprotobyname("TCP")->p_proto;
  sock = socket(AF_INET, SOCK_STREAM, protocol);

  if (sock < 0) {
    return -1;
  }
  s->fd = sock;
  s->cnt = 0;

  server.sin_family = AF_INET;
  server.sin_addr.s_addr = s->ip_long;
  server.sin_port = htons(s->port);
  
  if (server.sin_addr.s_addr == -1) {
    return -1;
  }
  
  fcntl(sock, F_SETFL, O_NONBLOCK);
  res = connect(sock, (struct sockaddr *) &server, sizeof(server));

  return errno;
}

int send_command(gint16 type, char* data) {
  napster_command_t* command;

  if (!(global.status.connected || global.status.connecting)) return -1;
  if ((global.napster.network == N_NAPSTER) &&
      (type >= 1000)) {
    g_warning("do not send command >= 1000 on napster server");
    return 1;
  }

  command = (napster_command_t*)malloc(sizeof(napster_command_t));
  command->type = type;
  command->data = strdup(data);

  if ((type == CMD_CLIENT_ADD_FILE) || type == CMD_CLIENT_REMOVE_FILE)
    global.share_queue = g_list_append(global.share_queue, command);
  else
    global.queue = g_list_append(global.queue, command);
  return 0;
}

gint send_command_real(napster_command_t* command) {
  unsigned char *msg;
  gint sendt = 0;
  gint16 length;
  gint16 length2;
  int res;

  if (!(global.status.connected || global.status.connecting)) return -1;

  length = strlen(command->data);
  msg = (gchar *)malloc((length + 5)*sizeof(gchar));
  log("protocol", "S: %5d L: %4d [%s]\n", command->type, 
      strlen(command->data), command->data);

  length2 = BSWAP16(length);
  command->type = BSWAP16(command->type);

  memcpy(msg, &length2, 2);
  memcpy(&msg[2], &command->type, 2);
  memcpy(&msg[4], command->data, length);
  msg[length+4] = 0;

  global.queue = g_list_remove(global.queue, command);

  while(sendt < (length+4)) {
    res = send(global.napster.socket.fd, msg+sendt, length+4-sendt, 0);
    if (res == -1) {
      printf("share send error %d %s\n", errno, strerror(errno));
      if (errno != EAGAIN) {
	g_warning("send error, errno:%d", errno);
	free(msg);
	return -1;
      }
    } else if (res == 0) {
      g_warning("0 char sent");
    } else {
      sendt += res;
    }
  }
  free(msg);
  
  return 0;
}

gint16 receive_command(int sock, char* data) {
  int res;
  static int progress = 0;
  static unsigned char buffer[2048];
  static int length = 0;
  static gint16 c_type;
  static gint16 c_length;

  if (progress == 0) {
    if (length < 4) {
      res = recv(sock, buffer+length, 4-length, 0);
      if (res > 0) {
	length += res;
      } else if (res == 0) {
	length = progress = 0;
	connection_close(&global.napster.socket, " Remote Close");
      } else {
	g_warning("1 recv error\n");
	length = progress = 0;
	connection_close(&global.napster.socket, strerror(errno));
      }      
    }
    if (length == 4) {
      length = 0;
      progress++;
      memcpy(&c_length, buffer, 2);
      memcpy(&c_type, &buffer[2], 2);

      c_length = BSWAP16(c_length);
      c_type = BSWAP16(c_type);
    }
    return -1;
  } 

  if (progress == 1) {
    if (length < c_length) {
      res = recv(sock, buffer+length, c_length-length, 0);
	
      if (res > 0) {
	length += res;
      } else if (res == 0) {
	length = progress = 0;
	connection_close(&global.napster.socket, strerror(errno));
      } else {
	g_warning("2 recv error\n");
      }
    }
    if (length == c_length) {
      buffer[length] = '\0';
      strcpy(data, buffer);
      progress = 0;
      length = 0;
      return c_type;
    }
  }

  return -1;
}

int handle_upload_request(gpointer data, gint source, 
			  GdkInputCondition condition) {
  
  struct sockaddr_in from;
  int len;
  transfer_t* transfer;

  transfer = transfer_new();
  transfer->status = S_CONNECTING;

  len = sizeof(from);
  transfer->sock.fd =
    accept(global.upload_socket.fd, (struct sockaddr *)&from, &len);
  if (transfer->sock.fd < 0) {
    free(transfer);
    return 1;
  }
  fcntl(transfer->sock.fd, F_SETFL, O_NONBLOCK);
  
  send(transfer->sock.fd, "1", 1, 0);

  transfer->sock.input = 
    gdk_input_add(transfer->sock.fd, GDK_INPUT_READ, 
		  GTK_SIGNAL_FUNC(incoming_header), transfer);
  
  return 1;
}

void create_upload_port(int send) {
  int len = 1;
  int sock;
  int protocol;
  struct sockaddr_in server;
  char str[2048];

  if (global.upload_socket.fd >= 0) {
    if (global.upload_socket.input > 0)
      gdk_input_remove(global.upload_socket.input);
    close(global.upload_socket.fd);
    global.upload_socket.fd = -1;
  }
  if (send) {
    if ((global.network.port == 0) || global.network.firewall) {
      if (global.status.connected) {
	send_command(CMD_CLIENT_CHANGE_DATA_PORT, "0");
      }
      return;
    }
  }

  protocol = getprotobyname("IP")->p_proto;
  sock = socket(AF_INET, SOCK_STREAM, protocol);

  if (sock < 0) {
    g_warning("could not create socket");
    return;
  }

  global.upload_socket.ip = strdup("127.0.0.1");
  global.upload_socket.ip_long = htonl(INADDR_ANY);
  global.upload_socket.port = global.network.port;
  global.upload_socket.fd = sock;
  global.upload_socket.cnt = 0;

  server.sin_family = AF_INET;
  server.sin_addr.s_addr = global.upload_socket.ip_long;
  server.sin_port = htons(global.upload_socket.port);

  setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &len, sizeof(len));
  setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, (char *)&len, sizeof(len));

  if (bind(sock, (struct sockaddr *)&server, sizeof(server)) < 0) {
    close(sock);
    // set port to 0
    port_dialog(global.network.port);
    global.network.port = 0;
    global.upload_socket.fd = -1;
    return;
  } else {
    client_message("Information", "Data Port was set to %d", 
		   global.network.port);
    client_message("Information", "At most %d transfers are allowed at a time.", 
		   10 + global.network.max_transfers);
  }
  listen(sock, 10 + global.network.max_transfers);

  log("protocol", "Listen for incoming connections on port [%d]\n", global.upload_socket.port);
  setup_preferences(P_PORT);

  if (send && global.status.connected) {
    sprintf(str, "%d", global.upload_socket.port);
    send_command(CMD_CLIENT_CHANGE_DATA_PORT, str);
  }

  global.upload_socket.input = 
    gdk_input_add(sock, GDK_INPUT_READ,
		  GTK_SIGNAL_FUNC(handle_upload_request), NULL);

  return;
}
