#include <dirent.h>
#include <ctype.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <netdb.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>

#include <gtk/gtk.h>

#include "support.h"
#include "md5.h"
#include "connection.h"
#include "commands.h"
#include "interface.h"
#include "chat.h"
#include "callbacks.h"
#include "scheme.h"
#include "handler.h"
#include "log.h"
#include "global.h"
#include "share.h"
#include "transfer.h"
#include "hotlist.h"
#include "resume.h"
#include "server.h"

#include "../pixmaps/speedgreen.xpm"
#include "../pixmaps/speedyellow.xpm"
#include "../pixmaps/speedred.xpm"
#include "../pixmaps/speedgray.xpm"
#include "../pixmaps/folder.xpm"
#include "../pixmaps/folder_open.xpm"
#include "../pixmaps/user1.xpm"
#include "../pixmaps/user2.xpm"
#include "../pixmaps/ignore.xpm"
#include "../pixmaps/enemy.xpm"
#include "../pixmaps/enemy2.xpm"
#include "../pixmaps/friend.xpm"
#include "../pixmaps/d16x16.xpm"

const char Level[5][20] = {
  "Leech",
  "User",
  "Moderator",
  "Admin",
  "Elite"
};
const char UserStatus[5][20] = {
  "Incative",
  "Active",
  "Remote",
  "Cloaked",
  "Leeched"
};
const char LineSpeed[11][20] = {
  "unknown",
  "14.4 kbps",
  "28.8 kbps",
  "33.6 kbps",
  "56.7 kbps",
  "64K ISDN",
  "128K ISDN",
  "Cable",
  "DSL",
  "T1",
  "T3+"
};
const char LineSpeedShort[11][20] = {
  "?",
  "14.4",
  "28.8",
  "33.6",
  "56.7",
  "64K",
  "128K",
  "Cable",
  "DSL",
  "T1",
  "T3+"
};

char Network[3][20] = {
  "Unknown",
  "Opennap",
  "Napster"
};

const int ColorTable[10][10] = {
  {  0,  1,  2,  3,  4,  5,  6,  7,  8,  9},
  { 10, 11, 12, 13, 14, 15,  0,  1,  1,  1},
  { -2, -2, -2, -2, -2, -2, -2, -2, -2, -2},
  {  1,  4,  3,  5,  2,  6, 10, 15, -3, -3},
  {  1,  1,  1,  1,  1,  1,  1,  1, -4, -4},
  { 14,  7,  9,  8, 12, 13, 11,  0, 15,  5},
  { -6, -6, -6, -6, -6, -6, -6, -6, -6, -6},
  { -7, -7, -7, -7, -7, -7, -7, -7, -7, -7},
  { -8, -8, -8, -8, -8, -8, -8, -8, -8, -8},
  { -9, -9, -9, -9, -9, -9, -9, -9, -9, -9}
};
const int ColorTable2[8] = {
   1,  4,  3,  5,  2,  6, 10,  15
};

global_t global;
char tstr[10][512];
char* list[10] = {
  tstr[0], tstr[1],
  tstr[2], tstr[3],
  tstr[4], tstr[5],
  tstr[6], tstr[7],
  tstr[8], tstr[9]
};

int directory_exists(char* dir) {
  struct stat st;

  if (stat(dir, &st) < 0) return 0; 
  else return 1;
}

void create_dir(char* dir) {
  char* pos;
  char* slash;

  if (!dir) return;

  slash = dir;
  while ((pos = strchr(slash+1, '/')) != NULL) {
    pos[0] = 0;
    if (!directory_exists(dir)) {
      mkdir(dir, 7*64);
    }
    pos[0] = '/';
    slash = pos;
  }
  if (!directory_exists(dir)) {
    mkdir(dir, 7*64);
  }
}

int get_color(char chr) {
  switch (chr) {
  case 'r': return 4;
  case 'b': return 2;
  case 'g': return 3;
  case 'c': return 10;
  case 'y': return 8;
  case 'Y': return 5;
  case 'k': return 1;
  default: return -1;
  }
} 

void text_print_text(GtkText* widget, char* fmt, ...) {
  char* pos;
  char text2[2048];
  char* text;
  GdkColor* color = NULL;
  int temp;
  va_list ap;

  va_start (ap, fmt);
  vsprintf (text2, fmt, ap);
  va_end (ap);

  text = text2;
  while ((pos = strchr(text, '$')) != NULL) {
    gtk_text_insert(widget, NULL, color, NULL, text, pos-text);
    text = pos;
    temp = get_color(pos[1]);
    if (temp >= 0) {
      color = &global.color_table[temp];
      text += 2;
    } else {
      gtk_text_insert(widget, NULL, color, NULL, text, 1);
      text++;
    }
  }
  gtk_text_insert(widget, NULL, color, NULL, text, strlen(text));
}

void port_dialog(int port) {
  GtkWidget* win;
  GtkText* text;

  win = create_information_win();
  gtk_widget_show(win);
  gtk_window_set_title(GTK_WINDOW(win), "Bind Error");
  text = GTK_TEXT(lookup_widget(win, "text15"));
  gtk_text_set_word_wrap(text, 1);
  
  text_print_text(text, "$r-------------------------\n Information\n-------------------------\n\n");
  text_print_text(text, "$kCould not set up data port $b%d$k.\n", port);
  text_print_text(text, "$kMaybe another Napster client or program uses this port.\n");
  text_print_text(text, "$kAt the moment the data port was set to $b0$k, so you cannot download from user who set their port also to 0\n");
  text_print_text(text, "$kIf you do not want this, choose another port at $bPreferences->Network->Port for Uploading Files $k and deactivate the $bFirewall $kbutton!\n");
}

void download_dialog(char* title) {
  GtkWidget* win;
  GtkText* text;
  
  win = create_information_win();
  gtk_widget_show(win);
  gtk_window_set_title(GTK_WINDOW(win), "Information");
  text = GTK_TEXT(lookup_widget(win, "text15"));
  gtk_text_set_word_wrap(text, 1);
  
  text_print_text(text, "$r-------------------------\n Warning\n-------------------------\n\n");
  text_print_text(text, "$kNo valid $b%s path $kis specified.\n", title);
  text_print_text(text, "$kPlease select a valid directory in $bNapster->Preferences->Global->%s Folder $k!\n", title);
}

void changes_dialog(char* title) {
  GtkWidget* win;
  GtkText* text;
  
  win = create_information_win();
  gtk_window_set_title(GTK_WINDOW(win), title);
  text = GTK_TEXT(lookup_widget(win, "text15"));
  gtk_text_set_word_wrap(text, 1);
  
  text_print_text(text, "$r-------------------------\nChanges since version 0.9.5\n-------------------------\n\n");
  
  text_print_text(text, "$rHave Fun!!!  Sgop\n");
  gtk_widget_show(win);
}

void server_dialog(char* title) {
  GtkWidget* win;
  GtkText* text;
  
  win = create_information_win();
  gtk_window_set_title(GTK_WINDOW(win), title);
  text = GTK_TEXT(lookup_widget(win, "text15"));
  gtk_text_set_word_wrap(text, 1);
  
  text_print_text(text, "$r-------------------------\nServer entries have changes\n-------------------------\n\n");
  text_print_text(text, "$kThe server entries in the rc-file have a different format. They were converted automatically.\n");
  text_print_text(text, "$kNow the $bofficial Napster Meta Server $k is no longer hard coded but is also stored in the rcfile, so you have to add it $bmanually$k, if you want to use it.\n");
  text_print_text(text, "$kFor this do the following:\n");
  text_print_text(text, "$kFirst connect to the Internet. Switch to the $bServer $kpage and look at the field $bEnter server manually$k. Enter $bserver.napster.com$k as the address, specify a description if you want and press the button $bAdd Meta Server$k !\n\n");
  text_print_text(text, "$cThat's all!\n");
  gtk_widget_show(win);
}

void geo_error(int cnt, char* line) {
  printf("unparsable line %d in geometry file: [%s]\n", cnt, line);
}

char* is_clist(char* line) {
  if (!strncasecmp(line, "DOWNLOAD_COLUMN", 15)) {
    return "transfer_down";
  } else if (!strncasecmp(line, "UPLOAD_COLUMN", 13)) {
    return "transfer_up";
  } else if (!strncasecmp(line, "HOTTREE_COLUMN", 14)) {
    return "hot_tree";
  } else if (!strncasecmp(line, "HOTSEARCH_COLUMN", 16)) {
    return "clist5";
  } else if (!strncasecmp(line, "SEARCH_COLUMN", 13)) {
    return "search_list";
  } else if (!strncasecmp(line, "LIB_COLUMN", 10)) {
    return "lib_tree";
  } else if (!strncasecmp(line, "CHANNEL_COLUMN", 14)) {
    return "channel_list";
  } else if (!strncasecmp(line, "SERVER_COLUMN", 13)) {
    return "server_list";
  } else if (!strncasecmp(line, "SELECTED_COLUMN", 15)) {
    return "clist11";
  } else if (!strncasecmp(line, "ONLINE_COLUMN", 13)) {
    return (char*)1;
  } else {
    return NULL;
  }
}

void read_geometry() {
  char File[1024];
  FILE* file;
  char line[500];
  char* pos1;
  char* pos2;
  char* pos3;
  int line_cnt;
  GtkCList* clist;
  int online;
  GtkWidget* temp;

  strcpy(File, global.lopster_home);
  strcat(File, "/geometry");

  if ((file = fopen(File, "r")) == NULL) return;

  line_cnt = 0;
  while (fgets(line, 400, file)) {
    line_cnt++;
    (strchr(line, '\n'))[0] = 0;
    if ((line[0] == '#') || 
	(line[0] == ' ') ||
	(line[0] == 0)) {
    } else if (!strncasecmp(line, "POSITION", 8)) {
      pos1 = strchr(line, '=');
      if (!pos1) {
	geo_error(line_cnt, line);
	continue;
      }
      pos2 = strrchr(pos1+1, ' ');
      if (!pos2) {
	geo_error(line_cnt, line);
	continue;
      }
      pos2[0] = 0;
      /*
      gtk_window_reposition(GTK_WINDOW(global.win),
			    atoi(pos1+1), atoi(pos2+1));
      */
    } else if (!strncasecmp(line, "SIZE", 4)) {
      pos1 = strchr(line, '=');
      if (!pos1) {
	geo_error(line_cnt, line);
	continue;
      }
      pos2 = strrchr(pos1+1, ' ');
      if (!pos2) {
	geo_error(line_cnt, line);
	continue;
      }
      pos2[0] = 0;
      gtk_window_set_default_size(GTK_WINDOW (global.win),
				  atoi(pos1+1), atoi(pos2+1));
    } else if (!strncasecmp(line, "ONLINE_PANED_POS", 16)) {
      pos1 = strchr(line, '=');
      if (!pos1) {
	geo_error(line_cnt, line);
	continue;
      }
      global.paned_pos = atoi(pos1+1);
    } else if (!strncasecmp(line, "SERVER_PANED_POS", 16)) {
      pos1 = strchr(line, '=');
      if (!pos1) {
	geo_error(line_cnt, line);
	continue;
      }
      temp = lookup_widget(global.win, "vpaned2");
      gtk_paned_set_position (GTK_PANED(temp), atoi(pos1+1));
    } else if (!strncasecmp(line, "HOT_PANED_POS", 13)) {
      pos1 = strchr(line, '=');
      if (!pos1) {
	geo_error(line_cnt, line);
	continue;
      }
      temp = lookup_widget(global.win, "vpaned1");
      gtk_paned_set_position (GTK_PANED(temp), atoi(pos1+1));
    } else if ((pos1 = is_clist(line)) != NULL) {
      if (pos1 == (char*)1) online = 1;
      else {
	online = 0;
	clist = GTK_CLIST(lookup_widget(global.win, pos1));
      }
      pos1 = strchr(line, '=');
      if (!pos1) {
	geo_error(line_cnt, line);
	continue;
      }
      pos2 = strchr(pos1+1, ',');
      if (!pos2) {
	geo_error(line_cnt, line);
	continue;
      }
      if (online) {
	global.online_width[atoi(pos1+1)] = atoi(pos2+1);
      } else {
	gtk_clist_set_column_width (clist, atoi(pos1+1), atoi(pos2+1));
      }
      pos3 = strchr(pos2+1, ',');
      if (pos3) {
	if (online) {
	  global.online_show[atoi(pos1+1)] = atoi(pos3+1);
	} else {
	  gtk_clist_set_column_visibility(clist, atoi(pos1+1), atoi(pos3+1));
	}
      }
    } else {
      geo_error(line_cnt, line);
    }
  }
}

void write_geometry() {
  char filename[1024];
  FILE* fd;
  GtkCList* clist;
  GtkPaned* paned;
  int i1;

  sprintf(filename, "%s/geometry", global.lopster_home);

  if ((fd = fopen(filename, "w")) == NULL) {
    g_warning("could not save geometry");
    return;
  }

  fprintf(fd, "SIZE=%d %d\n", global.win->allocation.width, 
	  global.win->allocation.height);
  clist = GTK_CLIST(lookup_widget(global.win, "transfer_down"));
  for (i1 = 0; i1 < clist->columns; i1++) {
    fprintf(fd, "DOWNLOAD_COLUMN=%d,%d,%d\n", i1, 
	    clist->column[i1].width, clist->column[i1].visible);
  }
  clist = GTK_CLIST(lookup_widget(global.win, "transfer_up"));
  for (i1 = 0; i1 < clist->columns; i1++) {
    fprintf(fd, "UPLOAD_COLUMN=%d,%d,%d\n", i1, 
	    clist->column[i1].width, clist->column[i1].visible);
  }
  clist = GTK_CLIST(lookup_widget(global.win, "hot_tree"));
  for (i1 = 0; i1 < clist->columns; i1++) {
    fprintf(fd, "HOTTREE_COLUMN=%d,%d,%d\n", i1, 
	    clist->column[i1].width, clist->column[i1].visible);
  }
  clist = GTK_CLIST(lookup_widget(global.win, "clist5"));
  for (i1 = 0; i1 < clist->columns; i1++) {
    fprintf(fd, "HOTSEARCH_COLUMN=%d,%d,%d\n", i1, 
	    clist->column[i1].width, clist->column[i1].visible);
  }
  clist = GTK_CLIST(lookup_widget(global.win, "search_list"));
  for (i1 = 0; i1 < clist->columns; i1++) {
    fprintf(fd, "SEARCH_COLUMN=%d,%d,%d\n", i1, 
	    clist->column[i1].width, clist->column[i1].visible);
  }
  clist = GTK_CLIST(lookup_widget(global.win, "lib_tree"));
  for (i1 = 0; i1 < clist->columns; i1++) {
    fprintf(fd, "LIB_COLUMN=%d,%d,%d\n", i1, 
	    clist->column[i1].width, clist->column[i1].visible);
  }
  clist = GTK_CLIST(lookup_widget(global.win, "channel_list"));
  for (i1 = 0; i1 < clist->columns; i1++) {
    fprintf(fd, "CHANNEL_COLUMN=%d,%d,%d\n", i1, 
	    clist->column[i1].width, clist->column[i1].visible);
  }
  clist = GTK_CLIST(lookup_widget(global.win, "server_list"));
  for (i1 = 0; i1 < clist->columns; i1++) {
    fprintf(fd, "SERVER_COLUMN=%d,%d,%d\n", i1, 
	    clist->column[i1].width, clist->column[i1].visible);
  }
  clist = GTK_CLIST(lookup_widget(global.win, "clist11"));
  for (i1 = 0; i1 < clist->columns; i1++) {
    fprintf(fd, "SELECTED_COLUMN=%d,%d,%d\n", i1, 
	    clist->column[i1].width, clist->column[i1].visible);
  }
  for (i1 = 0; i1 < 3; i1++) {
    fprintf(fd, "ONLINE_COLUMN=%d,%d,%d\n", i1, 
	    global.online_width[i1], global.online_show[i1]);
  }
  fprintf(fd, "ONLINE_PANED_POS=%d\n", global.paned_pos);

  paned = GTK_PANED(lookup_widget(global.win, "vpaned2"));
  fprintf(fd, "SERVER_PANED_POS=%d\n", paned->child1_size);

  paned = GTK_PANED(lookup_widget(global.win, "vpaned1"));
  fprintf(fd, "HOT_PANED_POS=%d\n", paned->child1_size);
  
  fclose(fd);
}

void sigpipe_handler(int val) {
  printf("sigpipe: Broken pipe: %d\n", val);
}

struct hostent* resolv_address(char* address) {
  struct hostent* h;
  int i1 = 0, i2 = 0, i3 = 0, i4 = 0;
  char n[4];

  h = gethostbyname(address);
  if (h == NULL) {
    sscanf(address, "%d.%d.%d.%d", &i1, &i2, &i3, &i4);
    n[0] = i1;
    n[1] = i2;
    n[2] = i3;
    n[3] = i4;
    h = gethostbyaddr(n, 4, AF_INET);
  }
  return h;
}

void update_user_stats() {
  GtkWidget* temp;
  char t[1024];

  // user
  temp = lookup_widget(global.win, "label271");
  if (global.napster.nick)
    gtk_label_set_text(GTK_LABEL(temp), global.napster.nick);
  else
    gtk_label_set_text(GTK_LABEL(temp), global.user.username);
  temp = lookup_widget(global.win, "label272");
  gtk_label_set_text(GTK_LABEL(temp), Level[global.user.level]);
  temp = lookup_widget(global.win, "label273");
  gtk_label_set_text(GTK_LABEL(temp), UserStatus[global.user.status]);
  temp = lookup_widget(global.win, "label274");
  if (global.reply)
    sprintf(t, "Reply: %s", global.reply);
  else
    sprintf(t, "Reply: <None>");
  gtk_label_set_text(GTK_LABEL(temp), t);

  // server
  if (global.status.connected) {
    temp = lookup_widget(global.win, "label276");
    gtk_label_set_text(GTK_LABEL(temp), 
		       global.napster.socket.description?global.napster.socket.description:"???");
    temp = lookup_widget(global.win, "label277");
    sprintf(t, "%s:%d", global.napster.socket.ip,
	    global.napster.socket.port);
    gtk_label_set_text(GTK_LABEL(temp), t);
    temp = lookup_widget(global.win, "label278");
    gtk_label_set_text(GTK_LABEL(temp), 
		       Network[global.napster.network]);
  } else {
    temp = lookup_widget(global.win, "label276");
    gtk_label_set_text(GTK_LABEL(temp), "Not connected");
    temp = lookup_widget(global.win, "label277");
    gtk_label_set_text(GTK_LABEL(temp), "????");
    temp = lookup_widget(global.win, "label278");
    gtk_label_set_text(GTK_LABEL(temp), 
		       Network[global.napster.network]);
  }
}

char* strcasestr(char* str, const char* sub) {
  int len = strlen(sub);
  int max_pos = strlen(str)-len;
  int str_pos;
  int sub_pos;

  str_pos = 0;
  sub_pos = 0;
  while (str_pos <= max_pos) {
    while (toupper(str[str_pos+sub_pos]) == 
	   toupper(sub[sub_pos])) {
      sub_pos++;
      if (sub_pos == len) return &(str[str_pos]);
    }
    str_pos++;
    sub_pos = 0;
  }
  
  return NULL;
}

int speed2int(char* linespeed) {
  int i1;

  for (i1 = 0; i1 < 11; i1++) {
    if (!strcmp(linespeed, LineSpeed[i1])) return i1;
  }

  return 0;
}

int level2int(char* level) {
  int i1;

  if (level[0] == '\"') {
    level++;
    level[strlen(level)-1] = 0;
  }
  for (i1 = 0; i1 < 5; i1++) {
    if (!strcasecmp(level, Level[i1])) return i1;
  }

  return -1;
}

int status2int(char* status) {
  int i1;

  if (status[0] == '\"') {
    status++;
    status[strlen(status)-1] = 0;
  }
  for (i1 = 0; i1 < 5; i1++) {
    if (!strcmp(status, UserStatus[i1])) return i1;
  }

  g_warning("status not found");
  return 0;
}

int idle_function(gpointer data) {
  GtkWidget* temp;

  if (global.auto_connect) {
    on_connect_activate(NULL, NULL);
    temp = lookup_widget(global.connect_win, "combo_entry12");
    gtk_entry_set_text(GTK_ENTRY(temp), "Last Server");
    on_connection_ok_clicked(NULL, NULL);
  }

  return 0;
}

void read_rc() {
  char RCFile[1024];
  FILE* file;
  char line[500];
  char* pos1;
  char* pos2;
  int section = 0;
  server_t* server;
  GtkWidget* temp;
  int changes = 1;
  int server_changes = 0;

  strcpy(RCFile, global.lopster_home);
  strcat(RCFile, "/lopsterrc");

  if ((file = fopen(RCFile, "r")) == NULL) {
    g_warning("could not load rc-file");
    global.channels = 
      g_list_append(global.channels, strdup("Lopster"));

    server = (server_t*)malloc(sizeof(server_t));
    server->socket.ip = strdup("bitchx.dimension6.com");
    server->socket.ip_long = 0;
    server->socket.port = 8888;
    server->socket.description = strdup("Bitchx");
    server->meta = 0;
    server->nick = strdup("");
    server->passwd = strdup("");
    global.servers = g_list_append(global.servers, server);

    server = (server_t*)malloc(sizeof(server_t));
    server->socket.ip = strdup("mp3.chemlab.org");
    server->socket.ip_long = 0;
    server->socket.port = 8888;
    server->socket.description = strdup("Chemlab");
    server->meta = 0;
    server->nick = strdup("");
    server->passwd = strdup("");
    global.servers = g_list_append(global.servers, server);

    server = (server_t*)malloc(sizeof(server_t));
    server->socket.ip = strdup("server.napster.com");
    server->socket.ip_long = 0;
    server->socket.port = 8875;
    server->socket.description = strdup("Official Meta Server");
    server->meta = 1;
    server->nick = strdup("");
    server->passwd = strdup("");
    global.servers = g_list_append(global.servers, server);

    on_preferences_activate(NULL, NULL);
  } else {
    while (fgets(line, 400, file)) {
      (strchr(line, '\n'))[0] = 0;
      if ((line[0] == '#') || 
	  (line[0] == ' ') ||
	  (line[0] == 0)) {
      } else if (!strncasecmp(line, "Section", 7)) {
	pos1 = strchr(line, '[')+1;
	pos2 = strrchr(line, ']');
	pos2[0] = 0;
	if (!strcasecmp(pos1, "User"))
	  section = 1;
	else if (!strcasecmp(pos1, "Path"))
	  section = 2;
	else if (!strcasecmp(pos1, "Network"))
	  section = 3;
	else if (!strcasecmp(pos1, "Hotlist"))
	  section = 4;
	else if (!strcasecmp(pos1, "Server"))
	  section = 5;
	else if (!strcasecmp(pos1, "Chat"))
	  section = 6;
	else if (!strcasecmp(pos1, "Buttons"))
	  section = 7;
	else if (!strcasecmp(pos1, "Global"))
	  section = 8;
	else if (!strcasecmp(pos1, "Library"))
	  section = 9;
	else if (!strcasecmp(pos1, "Search"))
	  section = 10;
	else {
	  g_warning("unknown section in rc-file [%s]", pos1);
	}
      } else if (section == 1) {
	if (!strncasecmp(line, "NAME", 4)) {
	  strcpy(global.user.username, &line[5]);
	} else if (!strncasecmp(line, "PASSWORD", 8)) {
	  strcpy(global.user.password, &line[9]);
	} else if (!strncasecmp(line, "EMAIL", 5)) {
	  strcpy(global.user.email, &line[6]);
	} else if (!strncasecmp(line, "MODE", 4)) {
	  global.usermode = strdup(line+5);
	} else {
	  g_warning("unknown tag in section USER [%s]", line);
	}
      } else if (section == 2) {
	if (!strncasecmp(line, "DOWNLOAD", 8)) {
	  global.path.download = strdup(&line[9]);
	} else if (!strncasecmp(line, "INCOMPLETE", 10)) {
	  global.path.incomplete = strdup(&line[11]);
	} else if (!strncasecmp(line, "SHARED", 6)) {
	  pos1 = strchr(line, '=')+1;
	  global.path.shared = 
	    g_list_append(global.path.shared, strdup(pos1));
	} else {
	  g_warning("unknown tag in section PATH [%s]", line);
	}
      } else if (section == 3) {
	if (!strncasecmp(line, "LINESPEED", 9)) {
	  sscanf(line+10, "%d", &(global.network.linespeed));
	} else if (!strncasecmp(line, "PORT", 4)) {
	  sscanf(line+5, "%d", &(global.network.port));
	} else if (!strncasecmp(line, "MAX_DOWNLOADS", 13)) {
	  sscanf(line+14, "%d", &(global.network.max_downloads));
	} else if (!strncasecmp(line, "MAX_UPLOADS", 11)) {
	  sscanf(line+12, "%d", &(global.network.max_uploads));
	} else if (!strncasecmp(line, "MAX_TRANSFERS", 13)) {
	  sscanf(line+14, "%d", &(global.network.max_transfers));
	} else if (!strncasecmp(line, "FIREWALL", 8)) {
	  sscanf(line+9, "%d", &(global.network.firewall));
	} else if (!strncasecmp(line, "AutoConnect", 11)) {
	  sscanf(line+12, "%d", &(global.auto_connect));
	} else {
	  g_warning("unknown tag in section NETWORK [%s]", line);
	}
      } else if (section == 4) {
	if (!strncasecmp(line, "USER", 4)) {
	  pos1 = strchr(line, '=') + 1;
	  add_to_hotlist(pos1);
	} else if (!strncasecmp(line, "ShortNames", 10)) {
	  pos1 = strchr(line, '=') + 1;
	  temp = lookup_widget(global.win, "checkbutton18");
	  if (atoi(pos1))
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), TRUE);
	  else
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), FALSE);
	} else {
	  g_warning("unknown tag in section HOTLIST [%s]", line);
	}
      } else if (section == 5) {
	if (!strncasecmp(line, "Server", 6)) {
	  server = (server_t*)malloc(sizeof(server_t));
	  pos1 = arg(line, '=');
	  pos1 = arg(NULL, ':');
	  server->socket.ip = strdup(pos1);
	  server->socket.ip_long = 0;
	  pos1 = arg(NULL, ':');
	  sscanf(pos1, "%d", &(server->socket.port));
	  pos1 = arg(NULL, ':');
	  server->socket.description = strdup(pos1);
	  pos1 = arg(NULL, ':');
	  sscanf(pos1, "%d", &(server->meta));
	  pos1 = arg(NULL, ':');
	  if (!pos1) {
	    server_changes = 1;
	    server->meta = 0;
	    if (global.user.username)
	      server->nick = strdup(global.user.username);
	    else server->nick = strdup("");
	    if (global.user.password)
	      server->passwd = strdup(global.user.password);
	    else server->passwd = strdup("");
	  } else {
	    server->nick = strdup(pos1);
	    pos1 = arg(NULL, ':');
	    if (pos1) server->passwd = strdup(pos1);
	    else server->passwd = strdup(global.user.password);
	  }
	  global.servers = g_list_append(global.servers, server);
	} else if (!strncasecmp(line, "Last", 4)) {
	  server = (server_t*)malloc(sizeof(server_t));
	  pos1 = arg(line, '=');
	  pos1 = arg(NULL, ':');
	  server->socket.ip = strdup(pos1);
	  pos1 = arg(NULL, ':');
	  sscanf(pos1, "%d", &(server->socket.port));
	  pos1 = arg(NULL, ':');
	  server->socket.description = strdup(pos1);
	  pos1 = arg(NULL, ':');
	  sscanf(pos1, "%d", &(server->meta));
	  pos1 = arg(NULL, ':');
	  if (!pos1) {
	    free(server->socket.description);
	    free(server);
	  } else {
	    server->nick = strdup(pos1);
	    pos1 = arg(NULL, ':');
	    if (pos1) server->passwd = strdup(pos1);
	    else server->passwd = strdup(global.user.password);
	    global.last_server = server;
	  }
	} else {
	  g_warning("unknown tag in section SERVER [%s]", line);
	}
      } else if (section == 6) {
	if (!strncasecmp(line, "Alias", 5)) {
	  pos1 = strchr(line, '=')+1;
	  pos2 = strchr(line, ':');
	  pos2[0] = 0;
	  pos2++;
	  command_make_alias(pos2, pos1);
	} else if (!strncasecmp(line, "Highlight", 9)) {
	  pos1 = strchr(line, '=')+1;
	  global.highlight = 
	    g_list_append(global.highlight, strdup(pos1));
	} else if (!strncasecmp(line, "Friend", 6)) {
	  pos1 = strchr(line, '=')+1;
	  global.frienduser = 
	    g_list_append(global.frienduser, strdup(pos1));
	} else if (!strncasecmp(line, "Enemy", 5)) {
	  pos1 = strchr(line, '=')+1;
	  global.enemyuser = 
	    g_list_append(global.enemyuser, strdup(pos1));
	} else if (!strncasecmp(line, "Scheme", 6)) {
	  pos1 = strchr(line, '=')+1;
	  global.scheme = strdup(pos1);
	} else if (!strncasecmp(line, "ShowJoins", 9)) {
	  pos1 = strchr(line, '=')+1;
	  global.show_joins = atoi(pos1);
	} else if (!strncasecmp(line, "PublicIgnore", 12)) {
	  pos1 = strchr(line, '=')+1;
	  global.ignore_public_ignore = atoi(pos1);
	} else if (!strncasecmp(line, "SendIgnore", 10)) {
	  pos1 = strchr(line, '=')+1;
	  global.send_ignore = atoi(pos1);
	} else if (!strncasecmp(line, "Channel", 7)) {
	  pos1 = strchr(line, '=')+1;
	  global.channels = 
	    g_list_append(global.channels, strdup(pos1));
	} else if (!strncasecmp(line, "Logging", 7)) {
	  pos1 = strchr(line, '=')+1;
	  global.logging = atoi(pos1);
	} else if (!strncasecmp(line, "ColorParse", 10)) {
	  pos1 = strchr(line, '=')+1;
	  global.parse_color = atoi(pos1);
	} else {
	  g_warning("unknown tag in section CHAT [%s]", line);
	}
      } else if (section == 7) {
	if (!strncasecmp(line, "Chat", 4)) {
	  pos1 = strchr(line, '=')+1;
	  temp = lookup_widget(global.win, "button114");
	  set_button_state(temp, atoi(pos1));
	} else if (!strncasecmp(line, "Connect", 7)) {
	  pos1 = strchr(line, '=')+1;
	  global.connect_button = atoi(pos1);
	} else if (!strncasecmp(line, "Search", 6)) {
	  pos1 = strchr(line, '=')+1;
	  temp = lookup_widget(global.win, "button115");
	  set_button_state(temp, atoi(pos1));
	} else if (!strncasecmp(line, "Mail", 4)) {
	  pos1 = strchr(line, '=')+1;
	  temp = lookup_widget(global.win, "button130");
	  set_button_state(temp, atoi(pos1));
	} else {
	  g_warning("unknown tag in section BUTTONS [%s]", line);
	}
      } else if (section == 8) {
	if (!strncasecmp(line, "Player", 6)) {
	  pos1 = strchr(line, '=') + 1;
	  global.player = strdup(pos1);
	} else if (!strncasecmp(line, "Version", 7)) {
	  pos1 = strchr(line, '=') + 1;
	  if (!strcmp(pos1, VERSION)) changes = 0;
	} else if (!strncasecmp(line, "PopupCreate", 11)) {
	  pos1 = strchr(line, '=') + 1;
	  global.popup_create = atoi(pos1);
	} else if (!strncasecmp(line, "ClientInfo", 10)) {
	  pos1 = strchr(line, '=') + 1;
	  if (global.clientinfo) free(global.clientinfo);
	  global.clientinfo = strdup(pos1);
	} else {
	  g_warning("unknown tag in section PLAYER [%s]", line);
	}
      } else if (section == 9) {
	if (!strncasecmp(line, "ShortNames", 10)) {
	  pos1 = strchr(line, '=') + 1;
	  temp = lookup_widget(global.win, "checkbutton19");
	  if (atoi(pos1))
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), TRUE);
	  else
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), FALSE);
	} else if (!strncasecmp(line, "AutoSave", 8)) {
	  pos1 = strchr(line, '=') + 1;
	  temp = lookup_widget(global.win, "checkbutton20");
	  if (atoi(pos1))
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), TRUE);
	  else
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp), FALSE);
	} else {
	  g_warning("unknown tag in section LIBRARY [%s]", line);
	}
      } else if (section == 10) {
	if (!strncasecmp(line, "MaxResults", 10)) {
	  pos1 = strchr(line, '=') + 1;
	  temp = lookup_widget(global.win, "search_results");
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), atoi(pos1));
	} else if (!strncasecmp(line, "Speed", 5)) {
	  pos1 = strchr(line, '=') + 1;
	  pos2 = strchr(pos1, ':');
	  pos2[0] = 0;
	  temp = lookup_widget(global.win, "combo_speed");
	  gtk_entry_set_text(GTK_ENTRY(temp), pos1);
	  temp = lookup_widget(global.win, "combo_speed2");
	  gtk_entry_set_text(GTK_ENTRY(temp), pos2+1);
	} else if (!strncasecmp(line, "Bitrate", 7)) {
	  pos1 = strchr(line, '=') + 1;
	  pos2 = strchr(pos1, ':');
	  pos2[0] = 0;
	  temp = lookup_widget(global.win, "combo_bitrate");
	  gtk_entry_set_text(GTK_ENTRY(temp), pos1);
	  temp = lookup_widget(global.win, "combo_bitrate2");
	  gtk_entry_set_text(GTK_ENTRY(temp), pos2+1);
	} else if (!strncasecmp(line, "Frequency", 9)) {
	  pos1 = strchr(line, '=') + 1;
	  pos2 = strchr(pos1, ':');
	  pos2[0] = 0;
	  temp = lookup_widget(global.win, "combo_freq");
	  gtk_entry_set_text(GTK_ENTRY(temp), pos1);
	  temp = lookup_widget(global.win, "combo_freq2");
	  gtk_entry_set_text(GTK_ENTRY(temp), pos2+1);
	} else if (!strncasecmp(line, "Duration", 8)) {
	  pos1 = strchr(line, '=') + 1;
	  pos2 = strchr(pos1, ':');
	  pos2[0] = 0;
	  temp = lookup_widget(global.win, "entry71");
	  gtk_entry_set_text(GTK_ENTRY(temp), pos1);
	  pos1 = pos2+1;
	  pos2 = strchr(pos1, ':');
	  pos2[0] = 0;
	  temp = lookup_widget(global.win, "spinbutton12");
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), atoi(pos1));
	  temp = lookup_widget(global.win, "spinbutton13");
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), atoi(pos2+1));
	} else if (!strncasecmp(line, "Size", 4)) {
	  pos1 = strchr(line, '=') + 1;
	  pos2 = strchr(pos1, ':');
	  pos2[0] = 0;
	  temp = lookup_widget(global.win, "entry70");
	  gtk_entry_set_text(GTK_ENTRY(temp), pos1);
	  pos1 = pos2+1;
	  pos2 = strchr(pos1, ':');
	  pos2[0] = 0;
	  temp = lookup_widget(global.win, "spinbutton10");
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), atoi(pos1));
	  temp = lookup_widget(global.win, "spinbutton11");
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp), atoi(pos2+1));
	} else {
	  g_warning("unknown tag in section SEARCH [%s]", line);
	}
      } else {
	g_warning("unparsable line [%s]", line);
      }
    }
    fclose(file);
  }

  /*
  if (changes == 1)
    changes_dialog("Changes since Version 0.9.5");
  */
  if (server_changes == 1) {
    server_dialog("Changes (Server)");
  }
}

void write_rc() {
  char RCFile[1024];
  FILE* file;
  GList* dlist;
  server_t* server;
  command_t* comm;
  int i1;
  char* str;
  GtkWidget* temp;

  strcpy(RCFile, global.lopster_home);
  strcat(RCFile, "/lopsterrc");

  if ((file = fopen(RCFile, "w")) != NULL) {
    fprintf(file, "# DANGER! Only edit if you know what you are doing\n");
    fprintf(file, "###################################\n");
    fprintf(file, "\nSection [USER]\n");
    fprintf(file, "Name=%s\n", global.user.username);
    fprintf(file, "Password=%s\n", global.user.password);
    fprintf(file, "EMail=%s\n", global.user.email);
    if (global.usermode)
      fprintf(file, "Mode=%s\n", global.usermode);

    fprintf(file, "\nSection [PATH]\n");
    if (global.path.download)
      fprintf(file, "Download=%s\n", global.path.download);
    if (global.path.shared) {
      for (i1 = 0; i1 < g_list_length(global.path.shared); i1++) {
	str = (char*)(g_list_nth(global.path.shared, i1)->data);
	fprintf(file, "Shared=%s\n", str);
      }
    }
    if (global.path.incomplete)
      fprintf(file, "Incomplete=%s\n", global.path.incomplete);

    fprintf(file, "\nSection [GLOBAL]\n");
    fprintf(file, "Version=%s\n", VERSION);
    if (global.player)
      fprintf(file, "Player=%s\n", global.player);
    fprintf(file, "PopupCreate=%d\n", global.popup_create);
    if (global.clientinfo)
      fprintf(file, "ClientInfo=%s\n", global.clientinfo);

    fprintf(file, "\nSection [Network]\n");
    fprintf(file, "Linespeed=%d\n", global.network.linespeed);
    fprintf(file, "Port=%d\n", global.network.port);
    fprintf(file, "Max_Downloads=%d\n", global.network.max_downloads);
    fprintf(file, "Max_Uploads=%d\n", global.network.max_uploads);
    fprintf(file, "Max_Transfers=%d\n", global.network.max_transfers);
    fprintf(file, "Firewall=%d\n", global.network.firewall);
    fprintf(file, "AutoConnect=%d\n", global.auto_connect);
    
    fprintf(file, "\nSection [Hotlist]\n");
    
    if (global.hotlist) {
      dlist = g_list_first(global.hotlist);
      while (dlist) {
	str = (char*)(dlist->data);
	fprintf(file, "User=%s\n", str);
	dlist = g_list_next(dlist);
      }
    }
    temp = lookup_widget(global.win, "checkbutton18");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
      fprintf(file, "ShortNames=1\n");
    } else {
      fprintf(file, "ShortNames=0\n");
    }

    fprintf(file, "\nSection [Server]\n");
    
    temp = lookup_widget(global.win, "clist11");
    i1 = 0;
    while (1) {
      if (i1 >= GTK_CLIST(temp)->rows) break;
      server = (server_t*)gtk_clist_get_row_data(GTK_CLIST(temp), i1);
      i1++;
      fprintf(file, "Server=%s:%d:%s:%d:%s:%s\n", server->socket.ip, 
	      server->socket.port, server->socket.description, server->meta, 
	      server->nick?server->nick:"",
	      server->passwd?server->passwd:"");
    }

    if (global.last_server) {
      fprintf(file, "Last=%s:%d:%s:%d:%s:%s\n", 
	      global.last_server->socket.ip, 
	      global.last_server->socket.port, 
	      global.last_server->socket.description, 
	      global.last_server->meta, 
	      global.last_server->nick?global.last_server->nick:"",
	      global.last_server->passwd?global.last_server->passwd:"");
    }

    // chat
    fprintf(file, "\nSection [Chat]\n");
    for (i1 = 0; i1 < g_list_length(global.commands); i1++) {
      comm = 
	(command_t*)(g_list_nth(global.commands, i1)->data);
      if (comm->is_alias) {
	fprintf(file, "Alias=%s:%s\n", comm->name, &(comm->syntax[4]));
      }
    }
    if (global.highlight) {
      for (i1 = 0; i1 < g_list_length(global.highlight); i1++) {
	str = (char*)(g_list_nth(global.highlight, i1)->data);
	fprintf(file, "Highlight=%s\n", str);
      }
    }
    if (global.frienduser) {
      for (i1 = 0; i1 < g_list_length(global.frienduser); i1++) {
	str = (char*)(g_list_nth(global.frienduser, i1)->data);
	fprintf(file, "Friend=%s\n", str);
      }
    }
    if (global.enemyuser) {
      for (i1 = 0; i1 < g_list_length(global.enemyuser); i1++) {
	str = (char*)(g_list_nth(global.enemyuser, i1)->data);
	fprintf(file, "Enemy=%s\n", str);
      }
    }
    fprintf(file, "Scheme=%s\n", global.scheme);
    fprintf(file, "ShowJoins=%d\n", global.show_joins);
    fprintf(file, "PublicIgnore=%d\n", global.ignore_public_ignore);
    fprintf(file, "SendIgnore=%d\n", global.send_ignore);
    fprintf(file, "Logging=%d\n", global.logging);

    if (global.channels) {
      for (i1 = 0; i1 < g_list_length(global.channels); i1++) {
	str = (char*)(g_list_nth(global.channels, i1)->data);
	fprintf(file, "Channel=%s\n", str);
      }
    }
    fprintf(file, "ColorParse=%d\n", global.parse_color);
    

    // buttons
    fprintf(file, "\nSection [Buttons]\n");
    temp = lookup_widget(global.win, "pix2");
    fprintf(file, "Chat=%d\n", GTK_WIDGET_VISIBLE(temp));
    temp = lookup_widget(global.win, "pix4");
    fprintf(file, "Search=%d\n", GTK_WIDGET_VISIBLE(temp));
    temp = lookup_widget(global.win, "pix16"); 
    fprintf(file, "Mail=%d\n", GTK_WIDGET_VISIBLE(temp));
    fprintf(file, "Connect=%d\n", global.connect_button);

    fprintf(file, "\nSection [Library]\n");
    temp = lookup_widget(global.win, "checkbutton19");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
      fprintf(file, "ShortNames=1\n");
    } else {
      fprintf(file, "ShortNames=0\n");
    }
    temp = lookup_widget(global.win, "checkbutton20");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
      fprintf(file, "AutoSave=1\n");
    } else {
      fprintf(file, "AutoSave=0\n");
    }

    fprintf(file, "\nSection [Search]\n");
    temp = lookup_widget(global.win, "search_results");
    fprintf(file, "MaxResults=%d\n", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp)));

    temp = lookup_widget(global.win, "combo_speed");
    fprintf(file, "Speed=%s", gtk_entry_get_text(GTK_ENTRY(temp)));
    temp = lookup_widget(global.win, "combo_speed2");
    fprintf(file, ":%s\n", gtk_entry_get_text(GTK_ENTRY(temp)));

    temp = lookup_widget(global.win, "combo_bitrate");
    fprintf(file, "Bitrate=%s", gtk_entry_get_text(GTK_ENTRY(temp)));
    temp = lookup_widget(global.win, "combo_bitrate2");
    fprintf(file, ":%s\n", gtk_entry_get_text(GTK_ENTRY(temp)));

    temp = lookup_widget(global.win, "combo_freq");
    fprintf(file, "Frequency=%s", gtk_entry_get_text(GTK_ENTRY(temp)));
    temp = lookup_widget(global.win, "combo_freq2");
    fprintf(file, ":%s\n", gtk_entry_get_text(GTK_ENTRY(temp)));

    temp = lookup_widget(global.win, "entry71");
    fprintf(file, "Duration=%s", gtk_entry_get_text(GTK_ENTRY(temp)));
    temp = lookup_widget(global.win, "spinbutton12");
    fprintf(file, ":%d", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp)));
    temp = lookup_widget(global.win, "spinbutton13");
    fprintf(file, ":%d\n", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp)));

    temp = lookup_widget(global.win, "entry70");
    fprintf(file, "Size=%s", gtk_entry_get_text(GTK_ENTRY(temp)));
    temp = lookup_widget(global.win, "spinbutton10");
    fprintf(file, ":%d", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp)));
    temp = lookup_widget(global.win, "spinbutton11");
    fprintf(file, ":%d\n", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(temp)));

    fclose(file);
  } else {
    g_warning("could not write rc-file %s", RCFile);
  }

}

void setup_commands() {
  command_new(C_WHISPER, "whisper", 
	      "/whipser <user> <text>",
	      "Send a private message <text> to the User <user>",
	      L_USER, F_CHAT);
  command_new(C_EMOTE, "emote", 
	      "/emote <text>",
	      "Send action description <text> on the current channel.",
	      L_USER, F_CHAT);
  command_new(C_JOIN, "join", 
	      "/join <channel>",
	      "Join channel <channel>.",
	      L_USER, F_CHAT);

  command_new(C_EMAIL, "email",
	      "/email <address>",
	      "Change e-mail address to <address>.",
	      L_USER, F_UMOD);
  command_new(C_SPEED, "speed",
	      "/speed [<user>] [<speed>]",
	      "It will change the link speed for <user> (moderator or higher). If no user is given your own link speed is modified",
	      L_USER, F_UMOD);
  command_new(C_DATAPORT, "dataport",
	      "dataport [<user>] <port>",
	      "Changes the dataport of <user> to <port>. If no user is specified it changes your own port",
	      L_USER, F_UMOD);
  command_new(C_PASSWORD, "password",
	      "/password [<user>] <password>",
	      "It will change the password for <user> (moderator or higher). If no user is given your own password is modified",
	      L_USER, F_UMOD);
  command_new(C_LEVEL, "level",
	      "/level <user> <level>",
	      "Change level of <user> with a lower level than yours.",
	      L_USER, F_UMOD);
  command_new(C_USERMODE, "usermode",
	      "/usermode [<ALL|NONE|<[-]<modes>>>]",
	      "Set user mode which controls the messages you receive from the server. If no argument is given the current mode is displayed. Known modes: ERROR BAN CHANGE KILL LEVEL SERVER MUZZLE PORT WALLOP CLOAK FLOOD PING.",
	      L_USER, F_UMOD);

  command_new(C_WHOIS, "whois", 
	      "/whois [<user>]",
	      "Show whois info for <user>.",
	      L_USER, F_INFO);
  command_new(C_HELP, "help", 
	      "/help [<command>]",
	      "Type /help for a list of commands, type /help <command> for a description of command <command>",
	      L_USER, F_INFO);
  /*
  command_new(C_BROWSE, "browse",
	      "/browse <user>",
	      "Browse <user>'s shared files.",
	      L_USER, F_INFO);
  */
  command_new(C_DISC, "disc",
	      "/disc",
	      "Disconnect from server.",
	      L_USER, F_MISC);

  command_new(C_SEND, "send", 
	      "/send <type> [<arg>....]",
	      "?",
	      L_MOD, F_MOD);
  command_new(C_BAN, "ban", 
	      "/ban <user|ip> [<reason>]",
	      "Ban user or ip from server (moderator or higher)",
	      L_MOD, F_MOD);
  command_new(C_UNBAN, "unban",
	      "/unban <user|ip>",
	      "Remove the ban on a user or ip (moderator or higher).",
	      L_MOD, F_MOD);
  command_new(C_CLOAK, "cloak", 
	      "/cloak",
	      "Toggle your invisible state (moderator or higher).",
	      L_MOD, F_MOD);


  command_new(C_CHCLEAR, "chclear", 
	      "/chclear",
	      "Clear the current channel from users with a lower level than yours.",
	      L_USER, F_OP);
  command_new(C_KILL, "kill",
	      "/kill <user>",
	      "Kill (disconnect) <user> with a lower lower level than yours.",
	      L_USER, F_MOD);
  command_new(C_LINKS , "links",
	      "/links",
	      "Show links between servers.",
	      L_USER, F_INFO);
  command_new(C_LIST, "list",
	      "/list",
	      "List available channels.",
	      L_USER, F_CLIENT);
  command_new(C_LISTBANS, "listbans",
	      "/listbans [<channel>]",
	      "List banned users and ip's for <channel> if channel is given. If no argument is given, the server bans are listed",
	      L_USER, F_INFO);
  command_new(C_PART, "part",
	      "/part",
	      "Part the current channel.",
	      L_USER, F_CHAT);
  command_new(C_WALLOP, "wallop",
	      "/wallop <text>",
	      "Send a message to all users with level moderator or higher (moderator or higher).",
	      L_MOD, F_MOD);
  command_new(C_STATS, "stats",
	      "/stats",
	      "Show server statistics.",
	      L_USER, F_INFO);
  command_new(C_SCHEME, "scheme",
	      "/scheme <name>",
	      "Loading Color scheme <name>.",
	      L_USER, F_CLIENT);
  command_new(C_ALIAS, "alias",
	      "/alias <alias> <command>",
	      "Making new <alias> for <command>.",
	      L_USER, F_CLIENT);
  command_new(C_AFK, "afk",
	      "/afk [<text>]",
	      "If argument text is given, your status is set to AFK. Users sending you private messages will receive <text> as a private message. If no argument is given AFK is deactivated.",
	      L_USER, F_CLIENT);
  command_new(C_PUBLIC, "public",
	      "/public <text>",
	      "Force to send public message",
	      L_USER, F_CHAT);
  command_new(C_REPLY, "reply",
	      "/reply <text>",
	      "Reply to the use who sent the last private message",
	      L_USER, F_CHAT);
  command_new(C_PRIVATE, "private",
	      "/private <user>",
	      "Create new page for private messaging to <user>",
	      L_USER, F_CHAT);
  command_new(C_MAIL, "mail",
	      "/mail [<user> [<message>]]",
	      "This command is used to save a <message> for <user> that is sent as a private message when <user> joins. If argument <message> is not given, the current saved message for <user> is displayed if argument <user> is given, otherwise all mails are displayed",
	      L_USER, F_CLIENT);
  command_new(C_DELMAIL, "delmail",
	      "/delmail [<user>]",
	      "This command deletes the saved message for <user>. If no argument is given all messages will be deleted.",
	      L_USER, F_CLIENT);
  command_new(C_CONN, "conn",
	      "/conn",
	      "Connect to the last server used.",
	      L_USER, F_MISC);
  command_new(C_MOTD, "motd",
	      "/motd",
	      "Show server's \"Message Of The Day\".",
	      L_USER, F_INFO);
  command_new(C_QUIT, "quit",
	      "/quit",
	      "Exit program.",
	      L_USER, F_MISC);
  command_new(C_IGNORE, "ignore",
	      "/ignore [<user>]",
	      "ignore user <user>. If User is not given, ignore list is shown",
	      L_USER, F_CHAT);
  command_new(C_UNIGNORE, "unignore",
	      "/unignore <user>",
	      "Unignores <user>",
	      L_USER, F_CHAT);
  command_new(C_MUZZLE, "muzzle",
	      "/muzzle <user> [<reason>]",
	      "Muzzle <user> with a lower level than yours.",
	      L_MOD, F_MOD);
  command_new(C_UNMUZZLE, "unmuzzle",
	      "/unmuzzle <user>",
	      "Unmuzzle <user> with a lower level than yours.",
	      L_MOD, F_MOD);
  command_new(C_CHBAN, "chban",
	      "/chban <channel> <user|ip> [<reason>]",
	      "Ban <user|ip> from <channel>",
	      L_USER, F_OP);
  command_new(C_CHUNBAN, "chunban",
	      "/chunban <channel> [<user|ip> [<reason>]]",
	      "Unban <user|ip> from <channel>. If no user is specified, the ban list is cleared",
	      L_USER, F_OP);
  command_new(C_TOPIC, "topic",
	      "/topic <chn> <topic>",
	      "Set the topic of channel <chn> to <topic> (moderator or higher).",
	      L_USER, F_OP);
  command_new(C_VERSION, "version",
	      "/version",
	      "Show server version (moderator or higher).",
	      L_MOD, F_MOD);
  command_new(C_CLEAR, "clear",
	      "/clear",
	      "Clears to current channel window",
	      L_USER, F_CLIENT);
  command_new(C_NUKE, "nuke",
	      "/nuke <user>",
	      "Nuke a <user> (delete account)",
	      L_MOD, F_MOD);
  command_new(C_UNNUKE, "unnuke",
	      "/unnuke <user>",
	      "Nuke a <user> (delete accout)",
	      L_MOD, F_MOD);
  command_new(C_GLOBAL, "global",
	      "/global <text>",
	      "Send global message to all users (admin or higher).",
	      L_MOD, F_MOD);
  command_new(C_KICK, "kick",
	      "/kick <channel> <user> [<reason>]",
	      "Kick <user> with a lower level than yours from the <channel>.",
	      L_USER, F_OP);
  command_new(C_CVERSION, "cversion",
	      "/cversion",
	      "Prints out the client version, if in a channel via public message",
	      L_USER, F_INFO);
  command_new(C_PING, "ping",
	      "/ping <user>",
	      "Ping <user>.",
	      L_USER, F_MISC);
  command_new(C_SPING, "sping",
	      "/sping <server>",
	      "Ping <server>.",
	      L_USER, F_MISC);
  command_new(C_ECHO, "echo",
	      "/echo <text>",
	      "Echo <text> to current window. What's this command good for?",
	      L_USER, F_CLIENT);
  command_new(C_CHLIMIT, "chlimit",
	      "/chlimit [<channel>] [<limit>]",
	      "Change max. user limit of <channel> to <limit>. If no limit is specified the current limit is shown. If no channel is specified the current channel is used.",
	      L_USER, F_OP);
  command_new(C_CHLEVEL, "chlevel", 
	      "/chlevel [<channel>] [<level>]",
	      "Set the minimum level required to join <channel>. If no level is given the current level is shown.If no channel is given the current channel is used.",
	      L_USER, F_OP);
  command_new(C_SSTATS, "sstats", 
	      "/sstats",
	      "Show the server stats (moderator or higher)",
	      L_USER, F_INFO);
  command_new(C_CHMODE, "chmode",
	      "/chmode [<channel>] [<+|-><mode>]",
	      "Set mode for <channel>. If no mode is given the current mode is displayed. If no channel is given the current channel is used. Mode may be one of PRIVATE, MODERATED, INVITE, TOPIC, REGISTERED.(channel operator or higher)",
	      L_USER, F_OP);
  command_new(C_CHWALLOP, "chwallop",
	      "/chwallop <message>",
	      "Send message to all channel operators",
	      L_USER, F_OP);
  command_new(C_OP, "op",
	      "/op [<channel>] [<user>]",
	      "Set <user> as operator on the current channel. If no channel is given the current channel is used, if no user is given the operator list is shown",
	      L_USER, F_OP);
  command_new(C_DEOP, "deop",
	      "/deop  [<channel>] <user>",
	      "Remove <user> as operator on the current channel or the specified channel (moderator or higher).",
	      L_USER, F_OP);
  command_new(C_KILLGHOST, "killghost",
	      "/killghost <ghost> [<passwd>]",
	      "kill yout <ghost>. if no passwd is given, the current password is used",
	      L_USER, F_MISC);
  command_new(C_FRIEND, "friend",
	      "/friend <user>",
	      "Add/remove a user to/from the friend list",
	      L_USER, F_CLIENT);
  command_new(C_ENEMY, "enemy",
	      "/enemy <user>",
	      "Add/remove a user to/from the enemy list",
	      L_USER, F_CLIENT);
  command_new(C_HOTADD, "hotadd",
	      "/hotadd <user>",
	      "Add a user to the hot list",
	      L_USER, F_CLIENT);
  command_new(C_HOTREM, "hotrem",
	      "/hotrem <user>",
	      "Remove a user from the hot list",
	      L_USER, F_CLIENT);
  command_new(C_LOCATE, "locate",
	      "/locate <user>",
	      "Display on which server <user> is",
	      L_MOD, F_MOD);
  command_new(C_CLIENTS, "clients",
	      "/clients",
	      "Display clients versions if server has enabled logs",
	      L_USER, F_INFO);
  /*
  command_new(C_SLINK , "slink",
	      "/slink <adr>:<pt> [<rmt>]",
	      "Attempt to link the current server to server <adr> at port <pt>.  If <rmt> is given, link from that server instead of the current one. Requires level admin or higher.",
	      L_MOD, F_MOD);
  command_new(C_SUNLINK, "sunlink",
	      "/sunlink <adr> [<reason>]",
	      "Unlink the current server from server <adr> (admin or higher).",
	      L_MOD, F_MOD);
  command_new(C_SREM, "srem",
	      "/srem <srv> [<reason>]",
	      "Request server <srv> to be removed from the list of allowed links (elite only).",
	      L_MOD, F_MOD);
  command_new(C_SKILL, "skill",
	      "/skill <adr> [<reason>]",
	      "Kill (shutdown) server <adr> (elite only).",
	      L_MOD, F_MOD);
  */
  command_new(C_INVITE, "invite",
	      "/invite [<channel>] <user>",
	      "Invite <user> to <channel> or the current channel",
	      L_USER, F_CHAT);
  command_new(C_CHMUZZLE, "chmuzzle",
	      "/chmuzzle [<channel>] [<user> [<reason>]]",
	      "Muzzle <user>. If no channel is given the current channel is used. If no user is specified the muzzle list is shown",
	      L_USER, F_OP);
  command_new(C_CHUNMUZZLE, "chunmuzzle",
	      "/chunmuzzle [<channel>] <user>",
	      "Unmuzzle <user> with a lower level than yours.",
	      L_USER, F_OP);
  command_new(C_CHVOICE, "voice",
	      "/chvoice [<channel>] [<user>]",
	      "Give <user> voice to speak in channel If no channel is given the current channel is used. If no user is specified the voice list is shown",
	      L_USER, F_OP);
  command_new(C_CHUNVOICE, "unvoice",
	      "/chunvoice [<channel>] <user>",
	      "Remove voice from <user> to speak in channel.",
	      L_USER, F_OP);
  command_new(C_CLIENTINFO, "clientinfo",
	      "/clientinfo [<info>]",
	      "Set yout client info to <info>",
	      L_USER, F_CLIENT);
  command_new(C_GUSER, "guser",
	      "/guser [<server> [<flags>]]",
	      "List all users. Flags may be a list of the characters: e=ELITE, a=ADMIN, m=MODERATOR, l=LEECH, z=MUZZLED, c=CLOAKED",
	      L_MOD, F_MOD);
}

void create_pixmaps() {
  GtkStyle *style;

  style = gtk_widget_get_style(global.win);
  global.pix.sgreen = 
    gdk_pixmap_create_from_xpm_d(global.win->window,
				 &(global.pix.sgreenb),
				 &style->bg[GTK_STATE_NORMAL],
				 speedgreen_xpm);
  global.pix.sred = 
    gdk_pixmap_create_from_xpm_d(global.win->window,
				 &(global.pix.sredb),
				 &style->bg[GTK_STATE_NORMAL],
				 speedred_xpm);
  global.pix.syellow = 
    gdk_pixmap_create_from_xpm_d(global.win->window,
				 &(global.pix.syellowb),
				 &style->bg[GTK_STATE_NORMAL],
				 speedyellow_xpm);
  global.pix.sgray = 
    gdk_pixmap_create_from_xpm_d(global.win->window,
				 &(global.pix.sgrayb),
				 &style->bg[GTK_STATE_NORMAL],
				 speedgray_xpm);

  global.pix.folder = 
    gdk_pixmap_create_from_xpm_d(global.win->window,
				 &(global.pix.folderb),
				 &style->bg[GTK_STATE_NORMAL],
				 folder);
  global.pix.folder_open = 
    gdk_pixmap_create_from_xpm_d(global.win->window,
				 &(global.pix.folder_openb),
				 &style->bg[GTK_STATE_NORMAL],
				 folder_open);
  global.pix.user1 = 
    gdk_pixmap_create_from_xpm_d(global.win->window,
				 &(global.pix.user1b),
				 &style->bg[GTK_STATE_NORMAL],
				 user_on);
  global.pix.user2 = 
    gdk_pixmap_create_from_xpm_d(global.win->window,
				 &(global.pix.user2b),
				 &style->bg[GTK_STATE_NORMAL],
				 user_off);
  global.pix.ignore = 
    gdk_pixmap_create_from_xpm_d(global.win->window,
				 &(global.pix.ignoreb),
				 &style->bg[GTK_STATE_NORMAL],
				 ignore_xpm);
  global.pix.enemy = 
    gdk_pixmap_create_from_xpm_d(global.win->window,
				 &(global.pix.enemyb),
				 &style->bg[GTK_STATE_NORMAL],
				 enemy_xpm);
  global.pix.enemy2 = 
    gdk_pixmap_create_from_xpm_d(global.win->window,
				 &(global.pix.enemy2b),
				 &style->bg[GTK_STATE_NORMAL],
				 enemy2_xpm);
  global.pix.friend = 
    gdk_pixmap_create_from_xpm_d(global.win->window,
				 &(global.pix.friendb),
				 &style->bg[GTK_STATE_NORMAL],
				 friend_xpm);
  global.pix.dummy = 
    gdk_pixmap_create_from_xpm_d(global.win->window,
				 &(global.pix.dummyb),
				 &style->bg[GTK_STATE_NORMAL],
				 dummy_xpm);
}

void setup_widgets() {
  GtkWidget* temp;
  char* pos;
  GList* dlist;
  int row;
  server_t* server;
  char text[1024];

  sprintf(text, "Lopster %s", VERSION);
  gtk_window_set_title (GTK_WINDOW(global.win), text);

  temp = lookup_widget(global.win, "notebook3");
  gtk_notebook_popup_enable(GTK_NOTEBOOK(temp));

  temp = lookup_widget(global.win, "search_list");
  gtk_clist_set_sort_column(GTK_CLIST(temp), 0);
  gtk_clist_set_auto_sort(GTK_CLIST(temp), TRUE);
  gtk_clist_set_compare_func(GTK_CLIST(temp), search_compare);

  temp = lookup_widget(global.win, "lib_tree");
  gtk_clist_set_compare_func(GTK_CLIST(temp), lib_compare);

  temp = lookup_widget(global.win, "Console_text");
  gtk_text_set_word_wrap(GTK_TEXT(temp), 1);

  temp = lookup_widget(global.win, "entry45");
  pos = make_string_from_list(global.highlight, "\"");
  gtk_entry_set_text(GTK_ENTRY(temp), pos);

  temp = lookup_widget(global.win, "entry46");
  pos = make_string_from_list(global.frienduser, " ");
  gtk_entry_set_text(GTK_ENTRY(temp), pos);

  temp = lookup_widget(global.win, "entry51");
  pos = make_string_from_list(global.enemyuser, " ");
  gtk_entry_set_text(GTK_ENTRY(temp), pos);


  temp = lookup_widget(global.win, "checkbutton7");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
			       global.show_joins);

  temp = lookup_widget(global.win, "checkbutton14");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
			       global.send_ignore);

  temp = lookup_widget(global.win, "checkbutton10");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
			       global.ignore_public_ignore);

  temp = lookup_widget(global.win, "checkbutton21");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
			       global.parse_color);

  temp = lookup_widget(global.win, "spinbutton5");
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
			    global.network.max_downloads);
  temp = lookup_widget(global.win, "spinbutton6");
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
			    global.network.max_uploads);
  temp = lookup_widget(global.win, "spinbutton8");
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(temp),
			    global.network.max_transfers);

  temp = lookup_widget(global.win, "button122");
  gtk_widget_set_sensitive(temp, FALSE);
  temp = lookup_widget(global.win, "button123");
  gtk_widget_set_sensitive(temp, FALSE);
  temp = lookup_widget(global.win, "button124");
  gtk_widget_set_sensitive(temp, FALSE);

  temp = lookup_widget(global.win, "channel_list");
  gtk_clist_set_sort_column(GTK_CLIST(temp), 0);
  gtk_clist_set_auto_sort(GTK_CLIST(temp), TRUE);
  gtk_clist_set_compare_func(GTK_CLIST(temp), channel_compare);

  temp = lookup_widget(global.win, "server_list");
  gtk_clist_set_sort_column(GTK_CLIST(temp), 0);
  gtk_clist_set_auto_sort(GTK_CLIST(temp), TRUE);
  gtk_clist_set_compare_func(GTK_CLIST(temp), server_compare);

  temp = lookup_widget(global.win, "clist11");
  if (global.servers) {
    dlist = g_list_first(global.servers);
    while (dlist) {
      server = (server_t*)(dlist->data);
      strcpy(tstr[0], server->socket.ip);
      sprintf(tstr[1], "%d", server->socket.port);
      strcpy(tstr[2], server->socket.description);
      sprintf(tstr[3], "%d", server->meta);
      strcpy(tstr[4], server->nick);
      row = gtk_clist_append(GTK_CLIST(temp), list);
      gtk_clist_set_row_data(GTK_CLIST(temp), row, dlist->data);
      
      dlist = g_list_next(dlist);
    }
  }
  
  temp = lookup_widget(global.win, "global_users");
  gtk_widget_set_sensitive(temp, FALSE);

  temp = lookup_widget(global.win, "transfer_down");
  gtk_clist_set_reorderable (GTK_CLIST(temp), TRUE);
  gtk_clist_set_use_drag_icons (GTK_CLIST(temp), TRUE);

  temp = lookup_widget(global.win, "clist11");
  gtk_clist_set_reorderable (GTK_CLIST(temp), TRUE);
  gtk_clist_set_use_drag_icons (GTK_CLIST(temp), TRUE);

  temp = lookup_widget(global.win, "frame145");
  gtk_widget_set_style(temp, global.style[3]);
  temp = lookup_widget(global.win, "frame143");
  gtk_widget_set_style(temp, global.style[3]);
  temp = lookup_widget(global.win, "frame142");
  gtk_widget_set_style(temp, global.style[3]);
  temp = lookup_widget(global.win, "frame141");
  gtk_widget_set_style(temp, global.style[3]);
  temp = lookup_widget(global.win, "frame3");
  gtk_widget_set_style(temp, global.style[3]);
}

void setup_buttons() {
  GtkWidget* temp;

  temp = lookup_widget(global.win, "button114");  //chat
  set_up_button(temp, "pix1",  "pix2",  "frame98");
  temp = lookup_widget(global.win, "button130");  //mail
  set_up_button(temp, "pix15", "pix16", "vbox54");
  temp = lookup_widget(global.win, "button115");  //search
  set_up_button(temp, "pix3",  "pix4",  "frame3");
}

void global_init() {
  GtkWidget* temp;
  time_t tim;
  char str[1024];
  hot_t* hot;

  global.connect_win = NULL;
  global.ban_win = NULL;
  global.user_win = NULL;
  global.ignore_win = NULL;
  global.dir_select_win = NULL;
  global.whois_win = NULL;
  global.usermode_win = NULL;
  global.resume_win = NULL;

  global.status.searching = 0;
  global.status.building = 0;
  global.status.exiting = 0;
  global.status.connected = 0;
  global.status.connecting = 0;
  global.status.whois_hide = 0;
  strcpy(global.current_room, "Console");
  global.servers = NULL;
  global.ignored_users = NULL;
  global.network.cur_uploads = 0;
  global.network.cur_downloads = 0;
  global.network.max_uploads = 1;
  global.network.max_downloads = 3;
  global.network.max_transfers = 3;
  global.network.firewall = 0;
  global.network.port = 6699;
  global.commands = NULL;
  global.upload_socket.fd = -1;
  global.colors = NULL;
  global.highlight = NULL;
  global.frienduser = NULL;
  global.enemyuser = NULL;
  global.chat_history = NULL;
  global.show_joins = 1;
  global.ignore_public_ignore = 0;
  global.send_ignore = 0;
  global.auto_connect = 0;
  global.logging = 0;
  global.afk = NULL;
  global.private_users = NULL;
  global.reply = NULL;
  global.mails = NULL;
  global.last_server = NULL;
  global.player = NULL;
  global.path.shared = NULL;
  global.path.incomplete = NULL;
  global.path.download = NULL;
  global.connect_button = 0;
  global.parse_color = 1;
  global.user.level = L_USER;
  global.user.email[0] = 0;
  global.user.status = STATUS_INACTIVE;
  global.napster.network = N_UNKNOWN;
  global.usermode = NULL;
  global.incomplete = NULL;
  global.queue = NULL;
  global.share_queue = NULL;
  global.clientinfo = NULL;
  global.online_width[0] = 100;
  global.online_width[1] = 30;
  global.online_width[2] = 60;
  global.online_show[0] = 1;
  global.online_show[1] = 1;
  global.online_show[2] = 1;
  global.paned_pos = 730;
  global.popup_create = 3;

  global.style[0] = gtk_widget_get_style(global.win);

  global.style[1] = gtk_style_new();
  global.style[1]->font = global.style[0]->font;
  global.style[1]->fg[GTK_STATE_NORMAL].pixel = 
    global.style[0]->fg[GTK_STATE_SELECTED].pixel;
  global.style[1]->fg[GTK_STATE_NORMAL].red = 65535;
  global.style[1]->fg[GTK_STATE_NORMAL].green = 0;
  global.style[1]->fg[GTK_STATE_NORMAL].blue = 0;

  global.style[2] = gtk_style_new();
  global.style[2]->font = global.style[0]->font;
  global.style[2]->base[GTK_STATE_NORMAL].pixel = 
    global.style[0]->base[GTK_STATE_NORMAL].pixel;
  global.style[2]->base[GTK_STATE_NORMAL].red = 0;
  global.style[2]->base[GTK_STATE_NORMAL].green = 0;
  global.style[2]->base[GTK_STATE_NORMAL].blue = 0;

  global.style[2]->text[GTK_STATE_NORMAL].pixel = 
    global.style[0]->text[GTK_STATE_NORMAL].pixel;
  global.style[2]->text[GTK_STATE_NORMAL].red = 65535;
  global.style[2]->text[GTK_STATE_NORMAL].green = 65535;
  global.style[2]->text[GTK_STATE_NORMAL].blue = 65535;

  global.style[3] = gtk_style_new();
  global.style[3]->font = gdk_font_load("-adobe-helvetica-bold-r-normal-*-*-120-*-*-p-*-iso8859-1");

  temp = lookup_widget(global.win, "Console_text");
  gtk_widget_set_style(temp, global.style[2]);  

  sprintf(str, "%s/.lopster", getenv("HOME"));
  global.lopster_home = strdup(str);
  create_dir(str);
  sprintf(str, "%s/log", global.lopster_home);
  create_dir(str);
  sprintf(str, "%s/schemes", global.lopster_home);
  create_dir(str);

  time(&tim);
  global.session_start = strdup(ctime(&tim));
  global.session_start[strlen(global.session_start)-1] = 0;

  printf("Lopster started at %s\n", global.session_start);

  temp = lookup_widget(global.win, "hot_tree");
  hot = (hot_t*)malloc(sizeof(hot_t));
  hot->files = 0;
  hot->bytes = 0;
  gtk_object_set_data(GTK_OBJECT(temp), "stats", hot);

  gdk_color_parse("#ffffff", &global.color_table[0]);  // W
  gdk_color_parse("#000000", &global.color_table[1]);  // Bl
  gdk_color_parse("#0000aa", &global.color_table[2]);  // b_
  gdk_color_parse("#00aa00", &global.color_table[3]);  // g_
  gdk_color_parse("#aa0000", &global.color_table[4]);  // r_
  gdk_color_parse("#aa5500", &global.color_table[5]);  // y_
  gdk_color_parse("#aa00aa", &global.color_table[6]);  // p_
  gdk_color_parse("#ff5555", &global.color_table[7]);  // R_
  gdk_color_parse("#ffff55", &global.color_table[8]);  // Y_
  gdk_color_parse("#55ff55", &global.color_table[9]);  // G
  gdk_color_parse("#00aaaa", &global.color_table[10]); // c
  gdk_color_parse("#55ffff", &global.color_table[11]); // C
  gdk_color_parse("#5555ff", &global.color_table[12]); // B_
  gdk_color_parse("#ff55ff", &global.color_table[13]); // P_
  gdk_color_parse("#555555", &global.color_table[14]); //dG
  gdk_color_parse("#aaaaaa", &global.color_table[15]); // w_

  // setting up
  read_geometry();

  // setting up signal handlers
  signal(SIGPIPE, sigpipe_handler);

  // setting up commands
  setup_commands();

  // setting up show/hide buttons
  setup_buttons();

  // creating pixmaps
  create_pixmaps();

  // reading rc file
  read_rc();
  
  // setting up colors
  if (!global.scheme) global.scheme = strdup("default");
  global.colors = scheme_load(global.colors, global.scheme);

  // setting up some GtkWidgets
  setup_widgets();

  // setting up shared files
  if (!lib_load()) lib_refresh();

  // creating  listening port
  create_upload_port(TRUE);
  
  // loading resume list
  resume_list_load();

  update_user_stats();

  gtk_idle_add(idle_function, NULL);
  global.dl = gtk_timeout_add(2000, download_inspector, NULL);
}

void global_exit() {
  GtkWidget* temp;
  
  gtk_timeout_remove(global.dl);

  if (global.upload_socket.fd >= 0) {
    if (global.upload_socket.input > 0)
      gdk_input_remove(global.upload_socket.input);
    close(global.upload_socket.fd);
  }
  
  write_rc();

  write_geometry();

  temp = lookup_widget(global.win, "checkbutton20");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(temp))) {
    on_button134_clicked(NULL, NULL);
  }
  
  // deleting commands
  command_delete_all();
  
  // and delete the global file list
}


char* extract_short_name(char* longname) {
  char dir_sep;
  char* pos1;
  char* pos2;

  // extracting short name
  dir_sep = '/';   // ext file system

  pos1 = strchr(longname, dir_sep);
  pos2 = strrchr(longname, dir_sep);
  if (pos1 == pos2) {  // none or only one sep found
    return longname;
  } else {
    pos2[0] = 'Q';
    pos1 = strrchr(longname, dir_sep);
    pos2[0] = dir_sep;
    pos1++;
    return pos1;
  }
}

char* convert_to_win(char* name) {
  char* ptr;
   
  ptr = name;
  if (!ptr || !*ptr) return NULL;

  while(*ptr) {
    if (*ptr == '/') *ptr = '\\';
    ptr++;
  }

  return ptr;
}

char* convert_to_unix(char* name) {
  char* ptr;
   
  convert_file(name);
  ptr = name;
  if (!ptr || !*ptr) return NULL;

  while(*ptr) {
    if (*ptr == '\\') *ptr = '/';
    ptr++;
  }

  return ptr;
}

void destroy_file_row(gpointer data) {
  file_t* file;

  file = (file_t*)data;
  
  if (file->longname) free(file->longname);
  if (file->shortname) free(file->shortname);
  if (file->winname) free(file->winname);
  if (file->user) free(file->user);
  if (file->md5) free(file->md5);
  free(file);
}

unsigned int get_bits(unsigned char* buffer, int start, int no) {
  unsigned int res = 0;

  res |= buffer[0]<<24 | buffer[1]<<16 |
    buffer[2]<<8 | buffer[3];
  
  res <<= start;
  res >>= (32-no);
  return res;
}

void file_parse_header(file_t *file) {
  FILE* fd;
  unsigned char buffer[4];  // header
  int mpeg_version;
  int mpeg_layer;
  int res;
  long framesize;
  long totalframes;
  short br[2][4][16] = {
    {  // mpeg 2
      { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
      { 0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, 0 }, // 
      { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 },      // 
      { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 }       // 
    },
    {  // mpeg1
      { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
      { 0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 0 },     // layer III
      { 0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384, 0 },    // layer II
      { 0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, 0 }  // layer I
    }
  };

  int freq[2][4] = {
    {22050, 24000, 16000, 0},
    {44100, 48000, 32000, 0}
  };

  if ((fd = fopen(file->longname, "r")) != NULL) {
    if (fread(buffer, sizeof(char), 4, fd) != 4) {
      //      printf("could not read header [%s]\n", file->longname);
      return;
    }
    // i think this is not correct sometimes
    while (get_bits(buffer, 0, 11) != 0x7ff) {
      buffer[0] = buffer[1];
      buffer[1] = buffer[2];
      buffer[2] = buffer[3];
      if (fread(&buffer[3], sizeof(char), 1, fd) != 1) {
	//	printf("could not read header [%s]\n", file->longname);
	return;
      }
    }
  } else {
//    printf("could not open file for reading header [%s]\n", file->longname);
    return;
  }

  // sync word
  res = get_bits(buffer, 0, 12);
  
  // MPEG Version
  mpeg_version = get_bits(buffer, 12, 1);

  // MPEG Layer
  mpeg_layer = get_bits(buffer, 13, 2);

  // Protection Bit
  res = get_bits(buffer, 15, 1);

  // Bitrate
  res = get_bits(buffer, 16, 4);
  file->bitrate = br[mpeg_version][mpeg_layer][res];

  // Fequency
  res = get_bits(buffer, 20, 2);
  file->frequency = freq[mpeg_version][res];
  if ((file->bitrate == 0) || (file->frequency == 0)) {
    //    printf("corrupted header! [%s]\n", file->longname);
    return;
  }

  framesize = 144000 * file->bitrate / file->frequency;
  totalframes = (file->size / (framesize + 1)) - 1;
  file->length = (time_t) (totalframes * 1152 / file->frequency);

  // Padding Bit
  res = get_bits(buffer, 22, 1);
  
  // Private Bit
  res = get_bits(buffer, 23, 1);
  
  // Mode
  res = get_bits(buffer, 24, 2);
  
  // Mode Extention
  res = get_bits(buffer, 26, 2);
  
  // Copyright
  res = get_bits(buffer, 28, 1);
  
  // Original Home
  res = get_bits(buffer, 29, 1);
  
  // Enphasis
  res = get_bits(buffer, 30, 2);
  
  fclose(fd);
}

void file_calc_md5(file_t *file) {
  //  file->md5 = MD5File(file->longname, file->md5);
  file->md5 = strdup("NOTCOMPUTED");
}

gint
search_compare (GtkCList      *clist,
		gconstpointer  ptr1,
		gconstpointer  ptr2) {
  file_t* file1;
  file_t* file2;

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  file1 = row1->data;
  file2 = row2->data;

  if (!file2) return (file1!=NULL);
  if (!file1) return -1;

  if (clist->sort_column == 0) {
    return strcasecmp (file1->shortname, file2->shortname);
  } else if (clist->sort_column == 1) {
    if (file1->size < file2->size) return -1;
    if (file1->size > file2->size) return 1;
    return 0;
  } else if (clist->sort_column == 2) {
    if (file1->bitrate < file2->bitrate) return -1;
    if (file1->bitrate > file2->bitrate) return 1;
    return 0;
  } else if (clist->sort_column == 3) {
    if (file1->length < file2->length) return -1;
    if (file1->length > file2->length) return 1;
    return 0;
  } else if (clist->sort_column == 4) {
    return strcasecmp (file1->user, file2->user);
  } else if (clist->sort_column == 5) {
    if (file1->linespeed < file2->linespeed) return -1;
    if (file1->linespeed > file2->linespeed) return +1;
    return 0;
  } else if (clist->sort_column == 6) {
    if (file1->ping < file2->ping) return -1;
    if (file1->ping > file2->ping) return +1;
    return 0;
  } else {
    return strcasecmp (file1->shortname, file2->shortname);
  }
}

gint
lib_compare (GtkCList      *clist,
	     gconstpointer  ptr1,
	     gconstpointer  ptr2) {
  char *text1 = NULL;
  char *text2 = NULL;

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
  text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;

  if (!text2)
    return (text1 != NULL);

  if (!text1)
    return -1;

  if (gtk_events_pending()) gtk_main_iteration();
  
  return strcasecmp (text1, text2);
}

gint
channel_compare (GtkCList      *clist,
		 gconstpointer  ptr1,
		 gconstpointer  ptr2) {

  char *text1 = NULL;
  char *text2 = NULL;
  int u1, u2;

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
  text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;

  if (!text2)
    return (text1 != NULL);

  if (!text1)
    return -1;

  if ((clist->sort_column == 4) || (clist->sort_column == 0)) {
    return strcasecmp (text1, text2);
  } else if ((clist->sort_column == 1) || 
	     (clist->sort_column == 2) || 
	     (clist->sort_column == 3)) {
    sscanf(text1, "%d", &u1);
    sscanf(text2, "%d", &u2);
    if (u1 < u2) return -1;
    if (u1 > u2) return 1;
    return 0;
  } else {
    return 0;
  }
}

gint
dir_compare (GtkCList      *clist,
	     gconstpointer  ptr1,
	     gconstpointer  ptr2) {

  char *text1 = NULL;
  char *text2 = NULL;

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
  text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;

  if (!text2)
    return (text1 != NULL);

  if (!text1)
    return -1;

  return strcasecmp (text1, text2);
}

gint
server_compare (GtkCList      *clist,
		 gconstpointer  ptr1,
		 gconstpointer  ptr2) {

  char *text1 = NULL;
  char *text2 = NULL;
  int u1, u2;

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
  text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;

  if (!text2)
    return (text1 != NULL);

  if (!text1)
    return -1;

  if ((clist->sort_column == 3) || 
      (clist->sort_column == 0)) {
    return strcasecmp (text1, text2);
  } else if ((clist->sort_column == 2) || 
	     (clist->sort_column == 4) || 
	     (clist->sort_column == 5) || 
	     (clist->sort_column == 6)) {
    sscanf(text1, "%d", &u1);
    sscanf(text2, "%d", &u2);
    if (u1 < u2) return -1;
    if (u1 > u2) return 1;
    return 0;
  } else {
    return 0;
  }
}

gint
client_compare (GtkCList      *clist,
		gconstpointer  ptr1,
		gconstpointer  ptr2) {

  char *text1 = NULL;
  char *text2 = NULL;
  int u1, u2;

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
  text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;

  if (!text2)
    return (text1 != NULL);

  if (!text1)
    return -1;

  if (clist->sort_column == 1) {
    return strcasecmp (text1, text2);
  } else if (clist->sort_column == 0) {
    sscanf(text1, "%d", &u1);
    sscanf(text2, "%d", &u2);
    if (u1 < u2) return -1;
    if (u1 > u2) return 1;
    return 0;
  } else {
    return 0;
  }
}

void cmd_eval_whois(char* data) {
  char* pos1;
  GtkEntry* entry;
  GtkWidget* temp;
  int t1;
  char str[200];
  int self;
  char nick[200];

  if (global.whois_win == NULL) {
    global.whois_win = create_whois_win();
  }
  gtk_window_set_title (GTK_WINDOW (global.whois_win), "Whois User");
  temp = lookup_widget(global.whois_win, "label116");
  gtk_label_set_text(GTK_LABEL(temp), "Online");

#ifdef GLOBAL_DEBUG
  printf("whois: %d\n", global.status.whois_hide);
#endif
  if (global.status.whois_hide == 0) {
    gtk_widget_show(global.whois_win);
  } else {
    global.status.whois_hide--;
  }

  pos1 = arg(data, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry56"));
  gtk_entry_set_text(entry, pos1);
  if (strcmp(pos1, global.napster.nick) == 0) self = 1;
  else self = 0;
  strcpy(nick, pos1);

  pos1 = arg(NULL, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry25"));
  gtk_entry_set_text(entry, "Level: ");
  gtk_entry_append_text(entry, pos1);
  if (self) {
    global.user.level = level2int(pos1);
#ifdef GLOBAL_DEBUG
    printf("user level: %d\n", global.user.level);
#endif
    if (global.user.level >= L_MOD) {
      temp = lookup_widget(global.win, "global_users");
      gtk_widget_set_sensitive(temp, TRUE);
      temp = lookup_widget(global.win, "banned_users");
      gtk_widget_set_sensitive(temp, TRUE);
    }
  }

  pos1 = arg(NULL, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry26"));
  sscanf(pos1, "%d", &t1);
  if (t1 > 60*60*24) 
    sprintf(str, "%d days %d hours %d minutes %d seconds", 
	    t1/(24*60*60), (t1%(24*60*60))/(60*60), 
	    (t1%(60*60))/60, t1%60);
  else if (t1 > 60*60)
    sprintf(str, "%d hours %d minutes %d seconds", 
	    t1/(60*60), (t1%(60*60))/60, t1%60);
  else
    sprintf(str, "%d minutes %d seconds", t1/60, t1%60);
  gtk_entry_set_text(entry, str);
  
  pos1 = arg(NULL, '\"');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry27"));
  gtk_widget_set_sensitive(GTK_WIDGET(entry), TRUE);
  gtk_entry_set_text(entry, pos1);

  pos1 = arg(NULL, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry28"));
  gtk_widget_set_sensitive(GTK_WIDGET(entry), TRUE);
  gtk_entry_set_text(entry, "Status: ");
  gtk_entry_append_text(entry, pos1);
  if (self) {
    global.user.status = status2int(pos1);
  }

  pos1 = arg(NULL, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry62"));
  gtk_widget_set_sensitive(GTK_WIDGET(entry), TRUE);
  gtk_entry_set_text(entry, "Shared: ");
  gtk_entry_append_text(entry, pos1);

  pos1 = arg(NULL, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry30"));
  gtk_widget_set_sensitive(GTK_WIDGET(entry), TRUE);
  gtk_entry_set_text(entry, pos1);
  gtk_entry_append_text(entry, " downs");

  pos1 = arg(NULL, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry59"));
  gtk_widget_set_sensitive(GTK_WIDGET(entry), TRUE);
  gtk_entry_set_text(entry, pos1);
  gtk_entry_append_text(entry, " ups");

  pos1 = arg(NULL, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry61"));
  gtk_widget_set_sensitive(GTK_WIDGET(entry), TRUE);
  sscanf(pos1, "%d", &t1);
  gtk_entry_set_text(entry, "Link: ");
  gtk_entry_append_text(entry, LineSpeed[t1]);

  pos1 = arg(NULL, '\"');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry60"));
  gtk_widget_set_sensitive(GTK_WIDGET(entry), TRUE);
  gtk_entry_set_text(entry, pos1);

  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry34"));
  gtk_entry_set_text(entry, "");

  if (self) update_user_stats();
  
  // Vetinari18 "User" 3027 " #German " "Active" 1415 0 0 5 "v2.0 BETA 7"
  // 3 1 213.6.110.212 1083 0 unknown
  temp = lookup_widget(global.whois_win, "vbox60");
  if (global.user.level < L_MOD) {
    gtk_widget_hide(temp);
    return;
  } else {
    gtk_widget_show(temp);
  }

  pos1 = arg(NULL, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry63"));
  gtk_entry_set_text(entry, pos1);
  gtk_entry_append_text(entry, " downs");

  pos1 = arg(NULL, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry64"));
  gtk_entry_set_text(entry, pos1);
  gtk_entry_append_text(entry, " ups");

  pos1 = arg(NULL, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry65"));
  gtk_entry_set_text(entry, pos1);

  pos1 = arg(NULL, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry66"));
  gtk_entry_set_text(entry, "Server: ");
  gtk_entry_append_text(entry, pos1);

  pos1 = arg(NULL, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry67"));
  gtk_entry_set_text(entry, "Data: ");
  gtk_entry_append_text(entry, pos1);

  pos1 = arg(NULL, ' ');
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry68"));
  gtk_entry_set_text(entry, pos1);
}

void cmd_eval_whowas(char* data) {
  char* pos1;
  GtkEntry* entry;
  GtkWidget* temp;
  time_t last_seen;
  char str[200];
  
  if (global.whois_win == NULL) {
    global.whois_win = create_whois_win();
  }
  gtk_widget_show(global.whois_win);

  gtk_window_set_title (GTK_WINDOW (global.whois_win), "Whowas User");
  
  temp = lookup_widget(global.whois_win, "label116");
  gtk_label_set_text(GTK_LABEL(temp), "Last Seen");

  pos1 = strchr(data, ' ');
  pos1[0] = 0;
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry56"));
  gtk_entry_set_text(entry, data);
  data = pos1+1;

  if (data[0] == '\"') {
    data++;
    pos1 = strchr(data, '"');
  } else {
    pos1 = strchr(data, ' ');
  }
  pos1[0] = 0;
  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry25"));
  gtk_entry_set_text(entry, "Level: ");
  gtk_entry_append_text(entry, data);
  if (data[0] == '\"') pos1++;
  data = pos1+1;

  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry26"));
  sscanf(data, "%ld", &last_seen);
  sprintf(str, "%s", ctime(&last_seen));
  gtk_entry_set_text(entry, str);

  entry = GTK_ENTRY(lookup_widget(global.whois_win, "entry34"));
  gtk_entry_set_text(entry, "");


  temp = lookup_widget(global.whois_win, "entry27");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  gtk_widget_set_sensitive(temp, FALSE);

  temp = lookup_widget(global.whois_win, "entry28");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  gtk_widget_set_sensitive(temp, FALSE);

  temp = lookup_widget(global.whois_win, "entry62");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  gtk_widget_set_sensitive(temp, FALSE);

  temp = lookup_widget(global.whois_win, "entry30");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  gtk_widget_set_sensitive(temp, FALSE);

  temp = lookup_widget(global.whois_win, "entry59");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  gtk_widget_set_sensitive(temp, FALSE);

  temp = lookup_widget(global.whois_win, "entry61");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  gtk_widget_set_sensitive(temp, FALSE);

  temp = lookup_widget(global.whois_win, "entry60");
  gtk_entry_set_text(GTK_ENTRY(temp), "");
  gtk_widget_set_sensitive(temp, FALSE);


  temp = lookup_widget(global.whois_win, "vbox60");
  gtk_widget_hide(temp);
}

void cmd_server_stats(char* data) {
  GtkWidget* temp;
  int songs, gics, users;
  char t[500];

  sscanf(data, "%d %d %d", &users, &songs, &gics);
  temp = lookup_widget(global.win, "label280");
  sprintf(t, "%d Users", users);
  gtk_label_set_text(GTK_LABEL(temp), t);
  temp = lookup_widget(global.win, "label281");
  sprintf(t, "%d Songs",songs);
  gtk_label_set_text(GTK_LABEL(temp), t);
  temp = lookup_widget(global.win, "label282");
  sprintf(t, "%d GB", gics);
  gtk_label_set_text(GTK_LABEL(temp), t);
}

void make_list_from_string(GList** glist, char* text, char* sep) {
  char* pos2;

  if (*glist) g_list_free(*glist);
  *glist = NULL;

  pos2 = strtok(text, sep);
  while (pos2) {
    if (strlen(pos2) > 0) {
      *glist = g_list_append(*glist, strdup(pos2));
    }
    pos2 = strtok(NULL, sep);
  }
}

char* make_string_from_list(GList* glist, char* sep) {
  static char result[1024];
  char* text;
  int i1;

  result[0] = 0;
  if (glist == NULL) {
    return result;
  }

  for (i1 = 0; i1 < g_list_length(glist); i1++) {
    text = (char*)(g_list_nth(glist, i1)->data);
    if (strlen(result) > 0) strcat(result, sep);
    strcat(result, text);
  }
  
  return result;
}

char* is_string_in_list(GList* glist, char* t) {
  GList* dlist;
  char* text;

  dlist = g_list_first(glist);
  
  while (dlist) {
    text = (char*)(dlist->data);
    if (!strcasecmp(t, text)) {
      return text;
    }
    dlist = g_list_next(dlist);
  }
  return NULL;
}

char* is_prefix_in_list(GList* glist, char* t) {
  GList* dlist;
  char* text;

  dlist = g_list_first(glist);
  
  //  printf("user: %s\n", t);
  while (dlist) {
    text = (char*)(dlist->data);
    //    printf("%s\n", text);
    if (t == NULL) return text;
    if (!strncasecmp(t, text, strlen(t))) {
      return text;
    }
    dlist = g_list_next(dlist);
  }
  return NULL;
}

int cmp_str(char* s1, char* s2) {
  int i1, len;
  
  i1 = strlen(s1);
  len = strlen(s2);

  if (i1 < len) len = i1;

  for (i1 = 0; i1 < len; i1++) {
    if (toupper(s1[i1]) != toupper(s2[i1]))
      break;
  }
  return i1;
}

int calc_list_prefix(GList* glist) {
  int res;
  int i1;
  int temp;
  char* text;

  if (g_list_length(glist) < 1) return 0;
  
  text = g_list_nth(glist, 0)->data;
  res = strlen(text);

  for (i1 = 0; i1 < g_list_length(glist); i1++) {
    temp = cmp_str(g_list_nth(glist, i1)->data, text);
    if (temp < res) res = temp;
  }
  return res;
}

int calc_command_prefix(GList* glist) {
  int res;
  int i1;
  int temp;
  char* text;
  command_t* command;

  if (g_list_length(glist) < 1) return 0;
  
  command = g_list_nth(glist, 0)->data;
  text = command->name;
  res = strlen(text);

  for (i1 = 0; i1 < g_list_length(glist); i1++) {
    command = g_list_nth(glist, i1)->data;
    temp = cmp_str(command->name, text);
    if (temp < res) res = temp;
  }
  return res;
}

void play_file(char* filename) {
  char command[1024];
  int i;
  gchar *argv[4];
  gint pid;

  if (!global.player) return;

  sprintf(command, "%s \"%s\"", global.player, filename);
#ifdef GLOBAL_DEBUG
  printf("%s\n", command);
#endif
  pid = fork();
  if(pid == 0) {
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = command;
    argv[3] = 0;
    for (i = 3; i < 256; i++) close(i);
    setsid();
    execve("/bin/sh", argv, environ);
    exit(-1);
  }
}

void set_last_server(server_t* server) {
  if (global.last_server) {
    free(global.last_server->socket.ip);
    free(global.last_server->socket.description);
    free(global.last_server->nick);
    free(global.last_server->passwd);
  } else {
    global.last_server = (server_t*)malloc(sizeof(server_t));
  }

  global.last_server->socket.description = strdup(server->socket.description);
  global.last_server->socket.ip = strdup(server->socket.ip);
  global.last_server->socket.ip_long = server->socket.ip_long;
  global.last_server->socket.port = server->socket.port;
  global.last_server->nick = strdup(server->nick);
  global.last_server->passwd = strdup(server->passwd);
  global.last_server->meta = server->meta;
}

void set_options_channels() {
  GList* dlist;
  int i1;
  GtkWidget* temp;
  char* pos;

  if (!global.options_win) return;

  dlist = NULL;
  temp = lookup_widget(global.win, "channel_list");
  for (i1 = 0; i1 < GTK_CLIST(temp)->rows; i1++) {
    gtk_clist_get_text(GTK_CLIST(temp), i1, 0, &pos);
    dlist = g_list_append(dlist, strdup(pos));
  }
  if (dlist) {
    temp = lookup_widget(global.options_win, "combo8");
    gtk_combo_set_popdown_strings (GTK_COMBO (temp), dlist);
  }
}

void set_button_state(GtkWidget* button, int open) {
  GtkWidget* temp;
  
  if (open) {
    temp = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(button), "pix_open"));
    gtk_widget_hide(temp);
    temp = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(button), "pix_closed"));
    gtk_widget_show(temp);
    temp = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(button), "con_widget"));
    gtk_widget_show(temp);
  } else {
    temp = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(button), "pix_open"));
    gtk_widget_show(temp);
    temp = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(button), "pix_closed"));
    gtk_widget_hide(temp);
    temp = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(button), "con_widget"));
    gtk_widget_hide(temp);
  }
}

int toggle_button_state(GtkWidget* button) {
  GtkWidget* temp;
  
  temp = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(button), "con_widget"));

  if (GTK_WIDGET_VISIBLE(temp)) {
    set_button_state(button, 0);
    return 0;
  } else {
    set_button_state(button, 1);
    return 1;
  }
}

void set_up_button(GtkWidget* button, 
		   char* pix1, char* pix2, char *wid) {
  GtkObject* obj;
  GtkWidget* temp;

  obj = GTK_OBJECT(button);
  temp = lookup_widget(button, pix1);
  gtk_object_set_data(obj, "pix_open", temp);
  temp = lookup_widget(button, pix2);
  gtk_object_set_data(obj, "pix_closed", temp);
  temp = lookup_widget(button, wid);
  gtk_object_set_data(obj, "con_widget", temp);
}

void show_bans(char* channel) {
  GtkWidget* temp;
  char str[1024];

  if (global.ban_win) {
    temp = lookup_widget(global.ban_win, "clist3");
    gtk_clist_clear(GTK_CLIST(temp));
  } else {
    global.ban_win = create_ban_win();
    gtk_widget_show(global.ban_win);
  }

  if (channel) {
    sprintf(str, "Banned Users for Channel %s\n", channel);
    send_command(CMD_CLIENT_CHANNEL_BAN_LIST, channel);
  } else {
    sprintf(str, "Global banned Users");
    send_command(CMD_CLIENT_BANLIST, "");
  }
  gtk_window_set_title(GTK_WINDOW(global.ban_win), str);
}

void cmd_banlist_entry(char* data) {
  char* pos;
  time_t l1;
  GtkWidget* temp;

  if (!global.ban_win) return;

  temp = lookup_widget(global.ban_win, "clist3");
  sscanf(data, "%s %s", tstr[0], tstr[1]);
  data = strchr(data, '\"')+1;
  pos = strrchr(data, '\"');
  pos[0] = 0;
  strcpy(tstr[3], data);
  data = pos + 2;
  sscanf(data, "%ld\n", &l1);
  sprintf(tstr[2], "%s", ctime(&l1));
  gtk_clist_append(GTK_CLIST(temp), list);
}

/////////////////////////////////////////////////////////////
// the code of the cparse() is almost copied from TekNap
/////////////////////////////////////////////////////////////
char* cparse(char *input) {
  int read;
  int write;
  int noappend = 1;
  static char output[2048];

  read = write = 0;

  while (input[read]) {
    if (input[read] == '%') {
      read++;
      switch (input[read]) {
      case 'k':
	output[write++] = '\003';
	output[write++] = '3';
	output[write++] = '0';
	break;
      case 'r':
	output[write++] = '\003';
	output[write++] = '3';
	output[write++] = '1';
	break;
      case 'g':
	output[write++] = '\003';
	output[write++] = '3';
	output[write++] = '2';
	break;
      case 'y':
	output[write++] = '\003';
	output[write++] = '3';
	output[write++] = '3';
	break;
      case 'b':
	output[write++] = '\003';
	output[write++] = '3';
	output[write++]   = '4';
	break;
      case 'm':
      case 'p':
	output[write++] = '\003';
	output[write++] = '3';
	output[write++]   = '5';
	break;
      case 'c':
	output[write++] = '\003';
	output[write++] = '3';
	output[write++]   = '6';
	break;
      case 'w':
	output[write++] = '\003';
	output[write++] = '3';
	output[write++]   = '7';
	break;
	
      case 'K':
	output[write++] = '\003';
	output[write++] = '5';
	output[write++]   = '0';
	break;
      case 'R':
	output[write++] = '\003';
	output[write++] = '5';
	output[write++]   = '1';
	break;
      case 'G':
	output[write++] = '\003';
	output[write++] = '5';
	output[write++]   = '2';
	break;
      case 'Y':
	output[write++] = '\003';
	output[write++] = '5';
	output[write++]   = '3';
	break;
      case 'B':
	output[write++] = '\003';
	output[write++] = '5';
	output[write++]   = '4';
	break;
      case 'M':
      case 'P':
	output[write++] = '\003';
	output[write++] = '5';
	output[write++]   = '5';
	break;
      case 'C':
	output[write++] = '\003';
	output[write++] = '5';
	output[write++]   = '6';
	break;
      case 'W':
	output[write++] = '\003';
	output[write++] = '5';
	output[write++]   = '7';
	break;
	
      case '0': case '1': case '2': case '3': 
      case '4': case '5': case '6': case '7':
	output[write++] = '\003';
	output[write++] = ',';
	output[write++] = '4';
	output[write++]   = input[read];
	break;
	
      case 'F':
	output[write++] = '\006';
	break;
      case 'n':
	output[write++] = '\003';
	output[write++] = '\017';
	break;
      case 'N':
	noappend = 1;
	break;
      case '%':
	output[write++] = input[read];
	break;
      default:
	output[write++] = '%';
	output[write++] = input[read];
	break;
      }
      read++;
    } else {
      output[write++] = input[read++];
    }
  }
  if (noappend == 0) {
    output[write++] = '\003';
    output[write++] = '-';
    output[write++] = '1';
  }
  output[write] = 0;

  return output;
}

void client_message(char* text, const char *fmt, ...) {
  va_list ap;
  char text2[1024];

  va_start (ap, fmt);
  vsprintf (text2, fmt, ap);
  va_end (ap);

  chat_print("message", "(");
  chat_print("message", text);
  chat_print("message", ") ");
  chat_print_ln("text", text2);
}

int in_channel() {
  if (is_string_in_list(global.private_users, global.current_room)) return 0;
  if (!strcmp("Console", global.current_room)) return 0;
  if (!strcmp("Channels", global.current_room)) return 0;

  return 1;
}

void cmd_sping(char* data) {
  time_t tim;
  char t[1024];

  time(&tim);
  strcpy(t, ctime(&tim));
  t[strlen(t)-1] = 0;
  client_message("Pong", "Server pinged you [%s]", t);
}

void cmd_ping(char* data) {
  client_message("Message", "%s has pinged you!", data);
  send_command(CMD_CLIENT_PONG, data);
}

void cmd_pong(char* data) {
  time_t tim;
  char t[1024];

  time(&tim);
  strcpy(t, ctime(&tim));
  t[strlen(t)-1] = 0;
  client_message("message", "pong from %s [%s]", data, t);
}

void cmd_set_port(char* data) {
  global.network.port = atoi(data);
  create_upload_port(FALSE);
  client_message("message", "Your data port was set to %d!",
		 global.network.port);
}

void toggle_friend(char* user) {
  char *text;
  char t2[1024];
  GtkWidget* temp;

  if ((text=is_string_in_list(global.frienduser, user)) == NULL) {
    global.frienduser = g_list_append(global.frienduser, strdup(user));
    sprintf(t2, "is now a friend to %s", user);
  } else {
    global.frienduser = g_list_remove(global.frienduser, text);
    sprintf(t2, "is no longer a friend to %s", user);
  }
  if (global.send_ignore) send_emote(t2);
  update_user(user);

  temp = lookup_widget(global.win, "entry46");
  text = make_string_from_list(global.frienduser, " ");
  gtk_entry_set_text(GTK_ENTRY(temp), text);
  if (global.options_win) {
    temp = lookup_widget(global.options_win, "entry44");
    gtk_entry_set_text(GTK_ENTRY(temp), text);
  }
  temp = lookup_widget(global.win, "button123");
  gtk_widget_set_sensitive(temp, FALSE);
}

void toggle_enemy(char* user) {
  char *text;
  char t2[1024];
  GtkWidget* temp;

  if ((text=is_string_in_list(global.enemyuser, user)) == NULL) {
    global.enemyuser = g_list_append(global.enemyuser, strdup(user));
    sprintf(t2, "has a new enemy: %s", user);
  } else {
    global.enemyuser = g_list_remove(global.enemyuser, text);
    sprintf(t2, "is no longer an enemy to %s", user);
  }
  if (global.send_ignore) send_emote(t2);
  update_user(user);

  temp = lookup_widget(global.win, "entry51");
  text = make_string_from_list(global.enemyuser, " ");
  gtk_entry_set_text(GTK_ENTRY(temp), text);
  if (global.options_win) {
    temp = lookup_widget(global.options_win, "entry52");
    gtk_entry_set_text(GTK_ENTRY(temp), text);
  }
  temp = lookup_widget(global.win, "button124");
  gtk_widget_set_sensitive(temp, FALSE);
}

void opennap_notify() {
  client_message("Message", "Opennap commands are not allowed on this server");
}

int opennap_version(int major, int minor) {
  if (global.napster.network != N_OPENNAP) return 0;
  
  return ((global.napster.major > major) ||
	  ((global.napster.major == major) &&
	   (global.napster.minor >= minor)));
}

char* arg(char* data, char sep) {
  static char* string;
  char* pos;
  char* pos2;

  if (data) string = data;
  if (!string) return NULL;
  
  while (string[0] == ' ') string++;
  if (string[0] == sep) string++;
  pos = string;
  if (pos[0] == 0) {
    string = NULL;
    return NULL;
  }
  pos2 = strchr(pos, sep);
  if (!pos2) {
    string = NULL;
  } else {
    string = pos2+1;
    pos2[0] = 0;
  }

  return pos;
}

void convert_chars(char* text, char from, char to) {
  char* pos;

  while ((pos = strchr(text, from)) != NULL) pos[0] = to;
}

GtkWidget* create_customize_win(GtkCList* clist) {
  GtkWidget *customize_win;
  GtkWidget *frame138;
  GtkWidget *vbox84;
  GtkWidget *vbox85;
  GtkWidget *checkbutton;
  GtkWidget *hseparator19;
  GtkWidget *frame139;
  GtkWidget *hbox213;
  GtkWidget *button151;
  GtkWidget *button152;
  int i1;
  char wname[1024];
  GtkWidget* title;

  customize_win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (customize_win), "customize_win", customize_win);
  gtk_window_set_title (GTK_WINDOW (customize_win), "Customize List");
  gtk_window_set_modal (GTK_WINDOW (customize_win), TRUE);
  gtk_window_set_policy (GTK_WINDOW (customize_win), FALSE, TRUE, TRUE);

  frame138 = gtk_frame_new (NULL);
  gtk_widget_ref (frame138);
  gtk_object_set_data_full (GTK_OBJECT (customize_win), "frame138", frame138,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame138);
  gtk_container_add (GTK_CONTAINER (customize_win), frame138);
  gtk_container_set_border_width (GTK_CONTAINER (frame138), 5);

  vbox84 = gtk_vbox_new (FALSE, 5);
  gtk_widget_ref (vbox84);
  gtk_object_set_data_full (GTK_OBJECT (customize_win), "vbox84", vbox84,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox84);
  gtk_container_add (GTK_CONTAINER (frame138), vbox84);
  gtk_container_set_border_width (GTK_CONTAINER (vbox84), 5);

  vbox85 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox85);
  gtk_object_set_data_full (GTK_OBJECT (customize_win), "vbox85", vbox85,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox85);
  gtk_box_pack_start (GTK_BOX (vbox84), vbox85, TRUE, TRUE, 0);

  for (i1 = 0; i1 < clist->columns; i1++) {
    title = gtk_clist_get_column_widget(clist, i1);
    sprintf(wname, "Show column \"%s\"", GTK_LABEL(title)->label);
    checkbutton = gtk_check_button_new_with_label(wname);
    gtk_widget_ref (checkbutton);
    sprintf(wname, "check%d", i1);
    gtk_object_set_data_full (GTK_OBJECT (customize_win), wname, checkbutton,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (checkbutton);
    gtk_box_pack_start (GTK_BOX (vbox85), checkbutton, FALSE, FALSE, 0);
    if (clist->column[i1].visible)
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
    else
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), FALSE);
  }

  hseparator19 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator19);
  gtk_object_set_data_full (GTK_OBJECT (customize_win), "hseparator19", hseparator19,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator19);
  gtk_box_pack_start (GTK_BOX (vbox84), hseparator19, FALSE, FALSE, 0);

  frame139 = gtk_frame_new (NULL);
  gtk_widget_ref (frame139);
  gtk_object_set_data_full (GTK_OBJECT (customize_win), "frame139", frame139,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame139);
  gtk_box_pack_start (GTK_BOX (vbox84), frame139, FALSE, FALSE, 0);

  hbox213 = gtk_hbox_new (TRUE, 0);
  gtk_widget_ref (hbox213);
  gtk_object_set_data_full (GTK_OBJECT (customize_win), "hbox213", hbox213,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox213);
  gtk_container_add (GTK_CONTAINER (frame139), hbox213);

  button151 = gtk_button_new_with_label ("Apply");
  gtk_widget_ref (button151);
  gtk_object_set_data_full (GTK_OBJECT (customize_win), "button151", button151,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button151);
  gtk_box_pack_start (GTK_BOX (hbox213), button151, TRUE, TRUE, 0);

  button152 = gtk_button_new_with_label ("Close");
  gtk_widget_ref (button152);
  gtk_object_set_data_full (GTK_OBJECT (customize_win), "button152", button152,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button152);
  gtk_box_pack_start (GTK_BOX (hbox213), button152, TRUE, TRUE, 0);

  gtk_signal_connect (GTK_OBJECT (customize_win), "destroy",
                      GTK_SIGNAL_FUNC (on_customize_win_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button151), "clicked",
                      GTK_SIGNAL_FUNC (on_button1x_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button152), "clicked",
                      GTK_SIGNAL_FUNC (on_button2x_clicked),
                      NULL);
  gtk_object_set_data(GTK_OBJECT(customize_win), "ref_clist", clist);
  
  return customize_win;
}

void show_clients() {
  GtkWidget* temp;

  if (!global.client_win) {
    global.client_win = create_client_win();
    temp = lookup_widget(global.client_win, "clist8");
    gtk_clist_set_sort_column(GTK_CLIST(temp), 0);
    gtk_clist_set_auto_sort(GTK_CLIST(temp), TRUE);
    gtk_clist_set_compare_func(GTK_CLIST(temp), client_compare);
  } else {
    temp = lookup_widget(global.client_win, "clist8");
    gtk_clist_freeze (GTK_CLIST(temp));
    gtk_clist_clear(GTK_CLIST(temp));
    gtk_clist_thaw(GTK_CLIST(temp));
  }
  gtk_widget_show(global.client_win);
  send_command(CMD_CLIENT_VERSION_STATS, "");
}

gint
user2_compare (GtkCList      *clist,
	       gconstpointer  ptr1,
	       gconstpointer  ptr2) {

  char *text1 = NULL;
  char *text2 = NULL;
  unsigned long l1, l2;
  GdkColor color = { 0, 0xff00, 0xbf00, 0xbf00 };

  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
  text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;

  if (!text2)
    return (text1 != NULL);

  if (!text1)
    return -1;

  if (clist->sort_column == 0) {
    return strcasecmp (text1, text2);
  } else if (clist->sort_column == 1) {
    l1 = inet_network(text1);
    l2 = inet_network(text2);
    if (l1 < l2) return -1;
    else if (l1 > l2) return 1;
    else {
      row1->background = color;
      row1->bg_set = TRUE;
      if (GTK_WIDGET_REALIZED (clist))
        gdk_color_alloc (gtk_widget_get_colormap (GTK_WIDGET (clist)),
                         &row1->background);
      row2->background = color;
      row2->bg_set = TRUE;
      if (GTK_WIDGET_REALIZED (clist))
        gdk_color_alloc (gtk_widget_get_colormap (GTK_WIDGET (clist)),
                         &row2->background);
      
      return 0;
    }
  }
  return 0;
}

void show_global_users(char* args, int clear) {
  GtkWidget* temp;

  if (global.user_win) {
    temp = lookup_widget(global.user_win, "clist12");
  } else {
    global.user_win = create_user_win();
    on_entry74_changed(NULL, NULL);
    temp = lookup_widget(global.user_win, "clist12");
    gtk_clist_set_compare_func(GTK_CLIST(temp), 
			       (GtkCListCompareFunc)user2_compare);
  }
  if (clear) gtk_clist_clear(GTK_CLIST(temp));
  gtk_widget_show(global.user_win);

  if (args) {
    send_command(CMD_CLIENT_GLOBAL_USER_LIST, args);
    temp = lookup_widget(global.user_win, "button164");
    gtk_widget_set_sensitive(temp, FALSE);
  }
}

void convert_file(char* name) {
  char* pos;
  char* pos2;
  int mode = 0;
  char ext[1024];

  if (!strncmp(name, "|||", 3)) mode = 1;
  if (name[0] == '/') mode = 2;
  if (name[0] == '\\') mode = 2;
  if ((name[1] == ':') && (name[2] == '\\')) mode = 3;
  
  if ((mode == 2) || (mode == 3)) return;

  if (mode == 1) {
    pos2 = name;
    pos = strrchr(name, '.');
    if (!pos) return;
    while (pos2+3 < pos) {
      pos2[0] = pos2[3];
      pos2++;
    }
    pos2[0] = 0;
    pos2 = strrchr(name, '|');
    if (!pos2) return;
    pos2[0] = '.';
  }
  if (mode == 0) {
    pos2 = name;
    pos = strchr(name, ' ');
    if (!pos) return;
    pos[0] = 0;
    strcpy(ext, name);
    pos += 3;
    while (1) {
      pos2[0] = pos[0];
      if (pos[0] == 0) break;
      pos2++;
      pos++;
    }
    pos = strrchr(name, '.');
    if (!pos) pos = pos2;
    else pos++;
    strcpy(pos, ext);
  }
  return;
}

void detect_speed_pixs(int speed,
		       GdkPixmap** pixmap, GdkBitmap** bitmap) {
  if (speed > 10 || speed < 1) {
    *pixmap = global.pix.sgray;
    *bitmap = global.pix.sgrayb;
  } else if (speed <= 3) {
    *pixmap = global.pix.sred;
    *bitmap = global.pix.sredb;
  } else if (speed <= 6) {
    *pixmap = global.pix.syellow;
    *bitmap = global.pix.syellowb;
  } else if (speed <= 10) {
    *pixmap = global.pix.sgreen;
    *bitmap = global.pix.sgreenb;
  } else {
    *pixmap = global.pix.sgray;
    *bitmap = global.pix.sgrayb;
  }
}
