#ifndef _GLOBAL_H_
#define _GLOBAL_H_

#define STYLE_NUMBER         4
#define STYLE_TAB1           0
#define STYLE_TAB2           1
#define STYLE_CHAT           2
#define STYLE_LABEL          3

#ifdef WORDS_BIGENDIAN
#define BSWAP16(x) ((((x) & 0xff) << 8) | (((x) >> 8) & 0xff))
#define BSWAP32(x) (((x) << 24) | (((x) & 0x0000ff00) << 8) | ((x) >> 24) | (((x) & 0x00ff0000) >> 8))
#else
#define BSWAP16(x) (x)
#define BSWAP32(x) (x)
#endif

#define STATUS_INACTIVE   0
#define STATUS_ACTIVE     1
#define STATUS_REMOTE     2
#define STATUS_CLOAKED    3

#define FILE_NONE         0
#define FILE_DOWNLOAD     1
#define FILE_ERROR        2
#define FILE_SUCCESS      3

//#define SHARE_DEBUG
//#define GLOBAL_DEBUG
//#define SEARCH_DEBUG
//#define TRANSFER_DEBUG
//#define CONNECTION_DEBUG

#define N_UNKNOWN         0
#define N_OPENNAP         1
#define N_NAPSTER         2

extern const char Level[5][20];
extern const char UserStatus[5][20];
extern const char LineSpeed[11][20];
extern const char LineSpeedShort[11][20];
extern const int ColorTable[10][10];
extern const int ColorTable2[8];
extern char tstr[10][512];
extern char* list[10];
extern char Network[3][20];

extern char **environ;

typedef struct {
  int files;
  unsigned long bytes;
} hot_t;

typedef struct {
  char username[100];
  char password[100];
  char email[100];
  int level;
  int status;
} user_t;

typedef struct {
  char* download;
  char* incomplete;
  GList* shared;
} path_t;

typedef struct {
  int linespeed;
  int port;
  int max_downloads;
  int max_uploads;
  int max_transfers;
  int firewall;
  int cur_uploads;
  int cur_downloads;
} network_t;

typedef struct {
  char connected;
  char connecting;
  char searching;
  char building;
  char exiting;
  char whois_hide;
} status_t;

typedef struct {
  char* longname;
  char* shortname;
  char* winname;
  long size;
  char* user;
  unsigned long ip;
  char* md5;
  int bitrate;
  int frequency;
  int length;
  int linespeed;
  int ping;
  int flags;   // 1=in lib  2=shared
  int status;
} file_t;

typedef struct {
  GdkPixmap *sgray;
  GdkBitmap *sgrayb;
  GdkPixmap *sgreen;
  GdkBitmap *sgreenb;
  GdkPixmap *syellow;
  GdkBitmap *syellowb;
  GdkPixmap *sred;
  GdkBitmap *sredb;

  GdkPixmap *folder;
  GdkBitmap *folderb;
  GdkPixmap *folder_open;
  GdkBitmap *folder_openb;
  GdkPixmap *user1;
  GdkBitmap *user1b;
  GdkPixmap *user2;
  GdkBitmap *user2b;
  GdkPixmap *ignore;
  GdkBitmap *ignoreb;
  GdkPixmap *enemy;
  GdkBitmap *enemyb;
  GdkPixmap *enemy2;
  GdkBitmap *enemy2b;
  GdkPixmap *friend;
  GdkBitmap *friendb;
  GdkPixmap *dummy;
  GdkBitmap *dummyb;
  
} pix_t;

typedef struct {
  //wins
  GtkWidget* win;
  GtkWidget* connect_win;
  GtkWidget* dir_select_win;
  GtkWidget* whois_win;
  GtkWidget* options_win;
  GtkWidget* resume_win;
  GtkWidget* ban_win;
  GtkWidget* ignore_win;
  GtkWidget* usermode_win;
  GtkWidget* client_win;
  GtkWidget* user_win;

  //options
  server_t napster;             // connected server
  socket_t napigator;           // napigator server
  socket_t upload_socket;       // listen socket file transfer
  user_t user;                  // you
  path_t path;                  // the dl and ul paths
  status_t status;
  network_t network;            // network options
  server_t* last_server;
  pix_t pix;                    // pix for intern use
  GList* servers;               // list of servers
  GList* commands;              // the list of commands
  GList* colors;                // the color scheme
  GList* highlight;             // the highlighted text
  GList* ignored_users;         // list of ignored users
  GList* frienduser;            // the friend users
  GList* enemyuser;             // the enemy users
  GList* private_users;         // users with private chat page
  GList* chat_history;          // history of input line
  GList* hotlist;               // users in hot list
  GList* mails;                 // list of mails
  GList* channels;              // autojoin channels
  GList* logs;                  // list of open logfiles
  GList* incomplete;            // list of incomplete downloads

  int popup_row;
  GtkWidget* popup_list;

  int history_pos;
  char current_room[100];
  int connect_button;
  int online_width[3];
  int online_show[3];
  int paned_pos;
  int dl;
  char* lopster_home;
  char* scheme;
  char* afk;
  char* reply;
  char* player;
  char* session_start;
  char* usermode;

  int show_joins;
  int auto_connect;
  int ignore_public_ignore;
  int send_ignore;
  int parse_color;
  int logging;
  GtkStyle* style[STYLE_NUMBER];
  GdkColor color_table[16];
  GList* queue;
  GList* share_queue;
  char* clientinfo;
  int popup_create;
} global_t;

extern global_t global;

int directory_exists(char* dir);
void create_dir(char* dir);

void port_dialog(int port);
void download_dialog(char* title);

struct hostent* resolv_address(char* address);
char* strcasestr(char* str, const char* sub);
int speed2int(char* linespeed);
int level2int(char* level);
int idle_function(gpointer data);
void global_init();
void global_exit();

char* extract_short_name(char* longname);
char* convert_to_win(char* name);
char* convert_to_unix(char* name);

void destroy_file_row(gpointer data);
void file_parse_header(file_t *file);
void file_calc_md5(file_t *file);

gint search_compare (GtkCList      *clist,
		     gconstpointer  ptr1,
		     gconstpointer  ptr2);
gint lib_compare (GtkCList      *clist,
		  gconstpointer  ptr1,
		  gconstpointer  ptr2);
gint channel_compare (GtkCList      *clist,
		      gconstpointer  ptr1,
		      gconstpointer  ptr2);
gint
dir_compare (GtkCList      *clist,
	     gconstpointer  ptr1,
	     gconstpointer  ptr2);
gint
server_compare (GtkCList      *clist,
		 gconstpointer  ptr1,
		 gconstpointer  ptr2);
gint
client_compare (GtkCList      *clist,
		gconstpointer  ptr1,
		gconstpointer  ptr2);

void send_search_request();
void cmd_eval_whois(char* data);
void cmd_eval_whowas(char* data);

void cmd_server_stats(char* data);

void make_list_from_string(GList** glist, char* text, char* sep);
char* make_string_from_list(GList* glist, char* sep);
char* is_string_in_list(GList* glist, char* t);
char* is_prefix_in_list(GList* glist, char* t);
int calc_list_prefix(GList* glist);
int calc_command_prefix(GList* glist);
void write_rc();
void play_file(char* filename);
void set_last_server(server_t* server);
void set_options_channels();
void set_button_state(GtkWidget* button, int open);
int toggle_button_state(GtkWidget* button);
void set_up_button(GtkWidget* button, 
		   char* pix1, char* pix2, char *wid);
void show_bans(char* channel);
void cmd_banlist_entry(char* data);
char* cparse(char *input);
void client_message(char* text, const char *fmt, ...);
int in_channel();
void cmd_sping(char* data);
void cmd_ping(char* data);
void cmd_pong(char* data);

void cmd_set_port(char* data);
void update_user_stats();

void toggle_friend(char* user);
void toggle_enemy(char* user);

void opennap_notify();
int opennap_version(int major, int minor);
char* arg(char* data, char sep);
void convert_chars(char* text, char from, char to);

GtkWidget* create_customize_win (GtkCList* clist);
void show_clients();
void show_global_users(char* args, int clear);
void convert_file(char* name);
void detect_speed_pixs(int speed,
		       GdkPixmap** pixmap, GdkBitmap** bitmap);

#endif
