#include <stdio.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/stat.h>

#include <gtk/gtk.h>

#include "callbacks.h"
#include "connection.h"
#include "global.h"
#include "transfer.h"
#include "resume.h"
#include "support.h"

GtkWidget* create_resume_popup() {
  GtkWidget *resume_popup;
  GtkAccelGroup *resume_popup_accels;
  GtkWidget *item;
  GtkWidget *separator5;
  GtkCTree* ctree;
  GtkCTreeNode* node;
  resume_t* resume;
  int flags = 0;
  int status = -1;

  ctree = GTK_CTREE(global.popup_list);
  node = gtk_ctree_node_nth(ctree, global.popup_row);
  
  if (!node) {
    flags = 0;     // no row
  } else {
    if (GTK_CTREE_ROW(node)->parent != NULL) {
      node = GTK_CTREE_NODE(GTK_CTREE_ROW(node)->parent);
      flags = 1;  // file
    } else {
      flags = 2;  // resume
    }
    resume = (resume_t*)gtk_ctree_node_get_row_data(ctree, node);
    status = resume->status;
  }

  if (flags == 0) return NULL;

  resume_popup = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (resume_popup), "resume_popup", resume_popup);
  resume_popup_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (resume_popup));

  if (flags == 2) {
    item = gtk_menu_item_new_with_label ("Delete");
    gtk_widget_ref (item);
    gtk_object_set_data_full (GTK_OBJECT (resume_popup), "item", item,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (item);
    gtk_container_add (GTK_CONTAINER (resume_popup), item);
    gtk_signal_connect (GTK_OBJECT (item), "activate",
			GTK_SIGNAL_FUNC (on_resume_delete_activate),
			NULL);
    if (status > R_SEARCH)
      gtk_widget_set_sensitive (item, FALSE);
      
    separator5 = gtk_menu_item_new ();
    gtk_widget_ref (separator5);
    gtk_object_set_data_full (GTK_OBJECT (resume_popup), "separator5", separator5,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (separator5);
    gtk_container_add (GTK_CONTAINER (resume_popup), separator5);
    gtk_widget_set_sensitive (separator5, FALSE);

    if (status <= R_NONE)
      item = gtk_menu_item_new_with_label ("Search Files");
    else
      item = gtk_menu_item_new_with_label ("Search Again");
    gtk_widget_ref (item);
    gtk_object_set_data_full (GTK_OBJECT (resume_popup), "item", item,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (item);
    gtk_container_add (GTK_CONTAINER (resume_popup), item);
    gtk_signal_connect (GTK_OBJECT (item), "activate",
			GTK_SIGNAL_FUNC (on_resume_search_activate),
			NULL);
  } else if (flags == 1) {
    item = gtk_menu_item_new_with_label ("Download from User");
    gtk_widget_ref (item);
    gtk_object_set_data_full (GTK_OBJECT (resume_popup), "item", item,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (item);
    gtk_container_add (GTK_CONTAINER (resume_popup), item);
    gtk_signal_connect(GTK_OBJECT (item), "activate",
		       GTK_SIGNAL_FUNC (on_download2_activate),
		       NULL);
    if (status >= R_RESUME)
      gtk_widget_set_sensitive (item, FALSE);

    separator5 = gtk_menu_item_new ();
    gtk_widget_ref (separator5);
    gtk_object_set_data_full (GTK_OBJECT (resume_popup), "separator5", separator5,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (separator5);
    gtk_container_add (GTK_CONTAINER (resume_popup), separator5);
    gtk_widget_set_sensitive (separator5, FALSE);

    item = gtk_menu_item_new_with_label ("Add User to Hotlist");
    gtk_widget_ref (item);
    gtk_object_set_data_full (GTK_OBJECT (resume_popup), "item", item,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (item);
    gtk_container_add (GTK_CONTAINER (resume_popup), item);
    gtk_signal_connect (GTK_OBJECT (item), "activate",
			GTK_SIGNAL_FUNC (on_add_user_to_hotlist_activate),
			NULL);
    item = gtk_menu_item_new_with_label ("Whois User");
    gtk_widget_ref (item);
    gtk_object_set_data_full (GTK_OBJECT (resume_popup), "item", item,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (item);
    gtk_container_add (GTK_CONTAINER (resume_popup), item);
    gtk_signal_connect (GTK_OBJECT (item), "activate",
			GTK_SIGNAL_FUNC (on_who_is_user_activate),
			NULL);
  }
  return resume_popup;
}

void file_insert_resume(resume_t* resume, file_t* file) {
  GtkCTree* ctree;
  GtkCTreeNode* node;
  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;

  resume->potentials = g_list_append(resume->potentials, file);
  if (!global.resume_win) return;

  ctree = GTK_CTREE(lookup_widget(global.resume_win, "resume_tree"));
  node = gtk_ctree_find_by_row_data(ctree, NULL, resume);

  sprintf(tstr[0], "%s", file->user);
  sprintf(tstr[1], "%lu", file->size);
  sprintf(tstr[2], "%s", LineSpeed[file->linespeed]);
  tstr[3][0] = 0;

  detect_speed_pixs(file->linespeed, &pixmap, &bitmap);
  node =
    gtk_ctree_insert_node(ctree, node, NULL, list, 5,
			  pixmap, bitmap, pixmap, bitmap,
			  TRUE, FALSE);

  resume_clist_update(resume);
  gtk_ctree_node_set_row_data_full(ctree, node, file, NULL);
}

void resume_list_load() {
  FILE* fd;
  resume_t* resume;
  char* fname;
  char line[2048];

  fname = g_strdup_printf("%s/incomplete.list", 
			  global.lopster_home);
  if ((fd = fopen(fname, "r")) == NULL) {
    free(fname);
    return;
  }
  free(fname);

  while (fgets(line, 2048-1, fd)) {
    resume = (resume_t*)malloc(sizeof(resume_t));
    resume->filename = strdup(arg(line, '\"'));
    resume->size = strtoul(arg(NULL, ' '), NULL, 10);
    resume->potentials = NULL;
    resume->status = R_NONE;
    global.incomplete = g_list_append(global.incomplete, resume);
  }
  fclose(fd);
}

void resume_list_save() {
  GList* dlist;
  resume_t* resume;
  char* fname;
  FILE* fd;
  
  fname = g_strdup_printf("%s/incomplete.list", 
			  global.lopster_home);
  if ((fd = fopen(fname, "w")) == NULL) {
    g_warning("Could not write [%s]\n", fname);
    free(fname);
    return;
  }

  free(fname);

  dlist = g_list_first(global.incomplete);
  while (dlist) {
    resume = (resume_t*)(dlist->data);
    fprintf(fd, "\"%s\" %lu\n", resume->filename, resume->size);
    dlist = g_list_next(dlist);
  }
  fclose(fd);
}

resume_t* resume_list_add(transfer_t* transfer) {
  char* text;
  resume_t* resume;

  text = strrchr(transfer->localname, '/');
  if (!text) text = transfer->localname;
  else text++;

  if (resume_list_search(text)) {
    g_warning("File already exists already as an incomplete file");
    return NULL;
  }

  resume = (resume_t*)malloc(sizeof(resume_t));
  resume->filename = 
    g_strdup_printf("%s/%s", global.path.incomplete, text);
  resume->size = transfer->size;
  resume->potentials = NULL;
  resume->status = R_NONE;
  global.incomplete = g_list_append(global.incomplete, resume);
  resume_list_save();

  return resume;
}

void resume_list_remove(resume_t* resume) {
  GList* dlist;
  
  resume = resume_clist_remove(resume);
  if (!resume) return;

  dlist = g_list_first(resume->potentials);
  while (dlist) {
    destroy_file_row(dlist->data);
    dlist = g_list_next(dlist);
  }
  g_list_free(resume->potentials);
  if (resume->filename) {
    // only delete if not resuming at the moment
    unlink(resume->filename);
    free(resume->filename);
  }

  global.incomplete = g_list_remove(global.incomplete, resume);
  resume_list_save();
  free(resume);
}

void resume_clist_add(resume_t* resume) {
  GtkCTree* ctree;
  char* pos;
  GtkCTreeNode* node;
  GtkCTreeNode* node2;
  file_t* file;
  GList* dlist;
  GdkPixmap *pixmap = NULL;
  GdkBitmap *bitmap = NULL;

  if (!global.resume_win) return;
  if (!resume) return;

  ctree = GTK_CTREE(lookup_widget(global.resume_win, "resume_tree"));
  node = gtk_ctree_find_by_row_data(ctree, NULL, resume);
  if (node) return;

  pos = strrchr(resume->filename, '/');
  if (!pos) pos = resume->filename;
  else pos++;

  strcpy(tstr[0], pos);
  sprintf(tstr[1], "%lu", resume->size);
  
  tstr[2][0] = 0;
  tstr[3][0] = 0;

  node = gtk_ctree_insert_node(ctree, NULL, NULL, list, 5,
			       global.pix.folder, 
			       global.pix.folderb,
			       global.pix.folder_open, 
			       global.pix.folder_openb,
			       FALSE, FALSE);
  gtk_ctree_node_set_row_data_full(ctree, node, resume, NULL);

  dlist = g_list_first(resume->potentials);
  while (dlist) {
    file = (file_t*)(dlist->data);
    
    detect_speed_pixs(file->linespeed, &pixmap, &bitmap);

    sprintf(tstr[0], "%s", file->user);
    sprintf(tstr[1], "%lu", file->size);
    sprintf(tstr[2], "%s", LineSpeed[file->linespeed]);
    tstr[3][0] = 0;
    node2 =
      gtk_ctree_insert_node(ctree, node, NULL, list, 5,
			    pixmap, bitmap, pixmap, bitmap,
			    TRUE, FALSE);
    gtk_ctree_node_set_row_data_full(ctree, node2, file, NULL);
    dlist = g_list_next(dlist);
  }

  resume_clist_update(resume);
}

resume_t*  resume_clist_remove(resume_t* resume) {
  GtkCTree* ctree;
  GtkCTreeNode* node;

  if (!global.resume_win) return NULL;
  if (!resume) return NULL;

  ctree = GTK_CTREE(lookup_widget(global.resume_win, "resume_tree"));
  node = gtk_ctree_find_by_row_data(ctree, NULL, resume);
  if (node) {
    gtk_ctree_remove_node(ctree, node);
    return resume;
  } else return NULL;
}

void resume_clist_update(resume_t* resume) {
  GtkCTree* ctree;
  GtkCTreeNode* node;
  char text[1024];
  struct stat st;

  if (!global.resume_win) return;
  if (!resume) return;

  ctree = GTK_CTREE(lookup_widget(global.resume_win, "resume_tree"));
  node = gtk_ctree_find_by_row_data(ctree, NULL, resume);
  if (!node) {
    g_warning("Resume type not found in clist");
    return;
  }

  if (stat(resume->filename, &st) < 0) {
    strcpy(text, "non existent");
  } else {
    sprintf(text, "%lu", st.st_size);
  }
  gtk_ctree_node_set_text(ctree, node, 2, text);

  if (resume->status == R_NONE)
    sprintf(text, "Not Searched");
  else if (resume->status == R_SEARCH)
    sprintf(text, "%d results", g_list_length(resume->potentials));
  else if (resume->status == R_RESUME)
    sprintf(text, "Resuming...");
  else if (resume->status == R_DOWNLOAD)
    sprintf(text, "Downloading...");
  else
    sprintf(text, "Unknown status");

  gtk_ctree_node_set_text(ctree, node, 3, text);
}

resume_t* resume_list_search(char* name) {
  GList* dlist;
  resume_t* resume;
  char* pos;
  char* pos2;
  dlist = g_list_first(global.incomplete);
  
  pos2 = strrchr(name, '/');
  if (!pos2) pos2 = name;
  else pos2++;

  while (dlist) {
    resume = (resume_t*)(dlist->data);
    pos = strrchr(resume->filename, '/');
    if (!pos) pos = resume->filename;
    else pos++;

    if (!strcmp(pos2, pos)) return resume;
    dlist = g_list_next(dlist);
  }
  return NULL;
}

