#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netdb.h>
#include <errno.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <gtk/gtk.h>

#include "callbacks.h"
#include "support.h"
#include "connection.h"
#include "global.h"
#include "server.h"

server_t* search_server() {
  GtkWidget* temp;
  GtkCList* clist;
  server_t* server;
  int i1;
  char* pos;
  int port;
  char* des;
  char* text;

  temp = lookup_widget(global.connect_win, "combo_entry12");
  text = strdup(gtk_entry_get_text(GTK_ENTRY(temp)));
  
  if (!strcasecmp("Last Server", text) && global.last_server) {
    return global.last_server;
  } else {
    pos = strrchr(text, ' ');
    pos[0] = 0;
    des = text;
    port = atoi(pos+1);
  }

  clist = GTK_CLIST(lookup_widget(global.win, "clist11"));
  for (i1 = 0; i1 < clist->rows; i1++) {
    server = (server_t*)gtk_clist_get_row_data(clist, i1);
    if (!strcasecmp(server->socket.description, des))
      if (server->socket.port == port) {
	free(text);
	return server;
      }
  }
  free(text);
  return NULL;
}

void add_server(int meta) {
  GtkWidget* temp1;
  GtkWidget* temp2;
  server_t* server;
  char* addr = NULL;
  char* des = NULL;
  struct hostent* h = NULL;
  char t[1024];

  temp1 = lookup_widget(global.win, "entry73");
  addr = strdup(gtk_entry_get_text(GTK_ENTRY(temp1)));
  temp2 = lookup_widget(global.win, "entry72");
  des = strdup(gtk_entry_get_text(GTK_ENTRY(temp2)));

  h = resolv_address(addr);
  if (h) {
    sprintf(t, "%u.%u.%u.%u", 
	   h->h_addr[0] & 0xff,
	   h->h_addr[1] & 0xff,
	   h->h_addr[2] & 0xff,
	   h->h_addr[3] & 0xff);
    server = (server_t*)malloc(sizeof(server_t));
    server->socket.ip = strdup(addr);
    server->socket.ip_long = inet_addr(t);
    server->socket.port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget(global.win, "spinbutton14")));
    server->meta = meta;
    if (global.user.username)
      server->nick = strdup(global.user.username);
    else server->nick = strdup("");
    if (global.user.password)
      server->passwd = strdup(global.user.password);
    else server->passwd = strdup("");
    if (strlen(des) > 0) {
      server->socket.description = strdup(des);
    } else {
      server->socket.description = strdup(h->h_name);
    }

    if (add_server_to_list(server) == 0) {
      // already in list, do not insert
      sprintf(t, "Server [%s] [%s] already in list\n",
	      server->socket.ip, server->socket.description);
      temp1 = lookup_widget(global.win, "text14");
      gtk_text_insert(GTK_TEXT(temp1), NULL, NULL, NULL, 
		      t, strlen(t));
      free(server);
    } else {
      gtk_entry_set_text(GTK_ENTRY(temp2), "");
      gtk_entry_set_text(GTK_ENTRY(temp1), "");
    }
  } else {
    sprintf(t, "Address [%s] is not valid (no internet connection?)\n", addr);
    temp1 = lookup_widget(global.win, "text14");
    gtk_text_insert(GTK_TEXT(temp1), NULL, NULL, NULL, 
		    t, strlen(t));
  }

  free(addr);
  free(des);
}

void napigator_read(gpointer data, int source, 
		    GdkInputCondition condition) {
  int n;
  static char buf[1024];
  static int progress = 0;
  char* line = buf;
  char* line_end;
  static int offset = 0;
  char* pos;
  GtkWidget* temp;

  temp = lookup_widget(global.win, "server_list");

  n = recv(source, buf+offset, sizeof(buf) -1-offset, 0);
  if (n <= 0) {
    gdk_input_remove(global.napigator.input);
    close(source);
    line = buf;
    offset = 0;
    progress = 0;
    return;
  }
  buf[n+offset] = 0;

  if (progress == 0) {  // header
    while ((line_end = strchr(line, '\r')) != NULL) {
      line_end[0] = 0;
      if (!strcmp(line, "Content-Type: text/html")) {
	progress = 1;
	line = line_end+2;
	break;
      }
      line = line_end+2;
    }
    strcpy(buf, line);
    line = buf;
    offset = strlen(buf);
  }

  if (progress == 1) {
    while ((line_end = strchr(line, '\n')) != NULL) {
      line_end[0] = 0;
      if (strlen(line) >= 3) {
	pos = strtok(line, " ");
	if (!pos) {
	  gdk_input_remove(global.napigator.input);
	  close(source);
	  return;
	}
	strcpy(tstr[1], pos);
	pos = strtok(NULL, " ");
	if (!pos) {
	  gdk_input_remove(global.napigator.input);
	  close(source);
	  return;
	}
	strcpy(tstr[2], pos);
	pos = strtok(NULL, " ");
	if (!pos) {
	  gdk_input_remove(global.napigator.input);
	  close(source);
	  return;
	}
	strcpy(tstr[3], pos);
	pos = strtok(NULL, " ");
	if (!pos) {
	  gdk_input_remove(global.napigator.input);
	  close(source);
	  return;
	}
	strcpy(tstr[4], pos);

	pos = strtok(NULL, " ");
	if (pos) {
	  strcpy(tstr[5], pos);
	  pos = strtok(NULL, " ");
	  if (!pos) {
	    gdk_input_remove(global.napigator.input);
	    close(source);
	    return;
	  }
	  strcpy(tstr[6], pos);
	  pos = strtok(NULL, " ");
	  if (!pos) {
	    gdk_input_remove(global.napigator.input);
	    close(source);
	    return;
	  }
	  strcpy(tstr[0], pos);
	  gtk_clist_append(GTK_CLIST(temp), list);
	}
      }
      line = line_end+1;
    }
    strcpy(buf, line);
    line = buf;
    offset = strlen(buf);
  }
}


void napigator_request(gpointer data, int source, 
		       GdkInputCondition condition) {
  int err, len;
  GtkWidget* temp;
  char t[1024];

  gdk_input_remove(global.napigator.input);

  len = sizeof(err);
  getsockopt(source, SOL_SOCKET, SO_ERROR, &err, &len);
  if (err) {
    sprintf(t, "Could not connect to www.napigator.com %d\n", err);
    temp = lookup_widget(global.win, "text14");
    gtk_text_insert(GTK_TEXT(temp), NULL, NULL, NULL, 
		    t, strlen(t));
    return;
  }

  send(source, "GET /servers.php?version=107&client=lopster HTTP/1.0\n\n", 
       strlen("GET /servers.php?version=107&client=lopster HTTP/1.0\n\n"), 0);
   
  global.napigator.input = gdk_input_add(source, GDK_INPUT_READ,
					 napigator_read, NULL);
}

void napigator_get_list() {
  int res;
  struct hostent* h = NULL;
  char IP[100];
  GtkWidget* temp;
  char t[1024];

  h = gethostbyname("www.napigator.com");
  if (h)
    sprintf(IP, "%u.%u.%u.%u", 
	    h->h_addr[0] & 0xff,
	    h->h_addr[1] & 0xff,
	    h->h_addr[2] & 0xff,
	    h->h_addr[3] & 0xff);
  else {
    sprintf(IP, "209.25.178.23");
  }

  global.napigator.ip = strdup(IP);
  global.napigator.ip_long = 
    inet_addr(global.napigator.ip);
  global.napigator.port = 80;
  
  res = connect_socket(&(global.napigator));

  if ((res != 0) && (res != EINPROGRESS)) {
    sprintf(t, "Could not connect to www.napigator.com\n");
    temp = lookup_widget(global.win, "text14");
    gtk_text_insert(GTK_TEXT(temp), NULL, NULL, NULL, 
		    t, strlen(t));
    return;
  }

  global.napigator.input =
    gdk_input_add(global.napigator.fd, GDK_INPUT_WRITE, 
		  GTK_SIGNAL_FUNC(napigator_request),
		  &(global.napigator));  
}

server_t* search_server_in_glist(server_t* server) {
  server_t* server2;
  GList* dlist;

  if (global.servers) {
    dlist = g_list_first(global.servers);
    while (dlist) {
      server2 = (server_t*)(dlist->data);
      if (!strcasecmp(server2->socket.ip, 
		      server->socket.ip))
	if (server2->socket.port == server->socket.port)
	  return server2;
      dlist = g_list_next(dlist);
    }
  }

  return NULL;
}

int add_server_to_list(server_t* server) {
  GtkWidget* temp;
  int row;

  if (search_server_in_list(server)) return 0;
  
  strcpy(tstr[0], server->socket.ip);
  sprintf(tstr[1], "%d", server->socket.port);
  strcpy(tstr[2], server->socket.description);
  sprintf(tstr[3], "%d", server->meta);
  strcpy(tstr[4], server->nick);

  temp = lookup_widget(global.win, "clist11");

  row = gtk_clist_append(GTK_CLIST(temp), list);
  gtk_clist_set_row_data(GTK_CLIST(temp), row, server);

  global.servers = g_list_append(global.servers, server);

  return 1;
}

server_t* search_server_in_list(server_t* server) {
  GtkCList* clist;
  server_t* server2;
  int i1;

  clist = GTK_CLIST(lookup_widget(global.win, "clist11"));
  
  for (i1 = 0; i1 < clist->rows; i1++) {
    server2 = (server_t*)gtk_clist_get_row_data(clist, i1);
    if (!strcasecmp(server2->socket.ip, server->socket.ip))
      if (server2->socket.port == server->socket.port)
	return server2;
  }
  return NULL;
}

GtkWidget* create_napigator_popup(int row) {
  GtkWidget *popup;
  GtkAccelGroup *popup_accels;
  GtkWidget *menu_entry;
  GtkWidget *separator;

  popup = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (popup), "popup", popup);
  popup_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (popup));

  menu_entry = gtk_menu_item_new_with_label ("Refresh List");
  gtk_widget_ref (menu_entry);
  gtk_object_set_data_full (GTK_OBJECT (popup), "menu_entry", menu_entry,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menu_entry);
  gtk_container_add (GTK_CONTAINER (popup), menu_entry);
  gtk_signal_connect (GTK_OBJECT (menu_entry), "activate",
		      GTK_SIGNAL_FUNC (on_refresh_activate),
		      NULL);

  if (row >= 0) {
    separator = gtk_menu_item_new ();
    gtk_widget_ref (separator);
    gtk_object_set_data_full (GTK_OBJECT (popup), "separator", separator,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (separator);
    gtk_container_add (GTK_CONTAINER (popup), separator);
    gtk_widget_set_sensitive (separator, FALSE);

    menu_entry = gtk_menu_item_new_with_label ("Add Selected");
    gtk_widget_ref (menu_entry);
    gtk_object_set_data_full (GTK_OBJECT (popup), "menu_entry", menu_entry,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (menu_entry);
    gtk_container_add (GTK_CONTAINER (popup), menu_entry);
    gtk_signal_connect (GTK_OBJECT (menu_entry), "activate",
			GTK_SIGNAL_FUNC (on_add_servers_activate),
			NULL);
  }

  separator = gtk_menu_item_new ();
  gtk_widget_ref (separator);
  gtk_object_set_data_full (GTK_OBJECT (popup), "separator", separator,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (separator);
  gtk_container_add (GTK_CONTAINER (popup), separator);
  gtk_widget_set_sensitive (separator, FALSE);

  menu_entry = gtk_menu_item_new_with_label ("Customize List");
  gtk_widget_ref (menu_entry);
  gtk_object_set_data_full (GTK_OBJECT (popup), "menu_entry", menu_entry,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menu_entry);
  gtk_container_add (GTK_CONTAINER (popup), menu_entry);
  gtk_signal_connect (GTK_OBJECT (menu_entry), "activate",
                      GTK_SIGNAL_FUNC (on_customize_list_activate),
                      NULL);

  return popup;
}

GtkWidget* create_server_popup(int row) {
  GtkWidget *popup;
  GtkAccelGroup *popup_accels;
  GtkWidget *menu_entry;
  GtkWidget *separator;

  popup = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (popup), "popup", popup);
  popup_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (popup));

  if ((row >= 0) && (!global.status.connecting)) {
    if (global.status.connected)
      menu_entry = gtk_menu_item_new_with_label ("Reconnect");
    else
      menu_entry = gtk_menu_item_new_with_label ("Connect");
    gtk_widget_ref (menu_entry);
    gtk_object_set_data_full (GTK_OBJECT (popup), "menu_entry", menu_entry,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (menu_entry);
    gtk_container_add (GTK_CONTAINER (popup), menu_entry);
    gtk_signal_connect (GTK_OBJECT (menu_entry), "activate",
			GTK_SIGNAL_FUNC (on_connect2_activate),
			NULL);

    separator = gtk_menu_item_new ();
    gtk_widget_ref (separator);
    gtk_object_set_data_full (GTK_OBJECT (popup), "separator", separator,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (separator);
    gtk_container_add (GTK_CONTAINER (popup), separator);
    gtk_widget_set_sensitive (separator, FALSE);
  }
   
  menu_entry = gtk_menu_item_new_with_label ("Delete Selected");
  gtk_widget_ref (menu_entry);
  gtk_object_set_data_full (GTK_OBJECT (popup), "menu_entry", menu_entry,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menu_entry);
  gtk_container_add (GTK_CONTAINER (popup), menu_entry);
  gtk_signal_connect (GTK_OBJECT (menu_entry), "activate",
		      GTK_SIGNAL_FUNC (on_delete_selected_activate),
		      NULL);

  separator = gtk_menu_item_new ();
  gtk_widget_ref (separator);
  gtk_object_set_data_full (GTK_OBJECT (popup), "separator", separator,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (separator);
  gtk_container_add (GTK_CONTAINER (popup), separator);
  gtk_widget_set_sensitive (separator, FALSE);

  menu_entry = gtk_menu_item_new_with_label ("Customize List");
  gtk_widget_ref (menu_entry);
  gtk_object_set_data_full (GTK_OBJECT (popup), "menu_entry", menu_entry,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menu_entry);
  gtk_container_add (GTK_CONTAINER (popup), menu_entry);
  gtk_signal_connect (GTK_OBJECT (menu_entry), "activate",
                      GTK_SIGNAL_FUNC (on_customize_list_activate),
                      NULL);

  return popup;
}

